-- t Time value used to compute current value
-- b Starting value
-- c Delta between start and end values
-- d Total length of animation
-- s = startValue, e = endValue, f = easing, h = handler
-- i = starttime
-- h = function which will be called by
---------------------------------------------------
-- Usage:
-- Library.LibAnUI.Animate(chains)
-- chains = {{d, s, e, f, h}, {d, s, e, f, h}}



if not Library then Library = {} end
if not Library.LibAnUI then Library.LibAnUI = {} end





-- Utilities

function Library.LibAnUI.NewId()
  local idnum = 0
  local avail = false
  while avail == false do
  if Library.LibAnUI.activeTicks[idnum] then
    avail =  false
    idnum = idnum + 1
  else
    return idnum
  end
  end
end

Library.LibAnUI.activeTicks = {}

function Library.LibAnUI.Calculate(t,b,c,d, f)
  return Library.LibAnUI.Ease[f](t,b,c,d)
end
function Library.LibAnUI.Tick()
  for chain, chaintable in pairs(Library.LibAnUI.activeTicks)do  -- chaintable = {v,v,v,v}
    for i, v in pairs(chaintable) do
      -- v = d, s, e, f, h
      if not v then
        table.remove(Library.LibAnUI.activeTicks, chain)
        return
      end
      if i == 1  then
        if not v.i then Library.LibAnUI.activeTicks[chain][i].i = Inspect.Time.Frame() end -- if we have not caught the time, do so
        --print("Time Left: " .. v.i - Inspect.Time.Real() + v.d .. "Running:" .. v.f)
        local a = Library.LibAnUI.Ease[v.f](v.i - Inspect.Time.Frame(),v.b,v.c,v.d) -- Get our value
        v.h(a) -- run function
        if v.i - Inspect.Time.Frame() + v.d < 0 then  -- If the time is up
          table.remove(Library.LibAnUI.activeTicks[chain], i) --Remove from the queue
          v.h(v.e) -- make sure to run with end value
        end
      else
      end
    end
  end
end
function Library.LibAnUI.Animate(chains)
  local chainid = Library.LibAnUI.NewId()
  local chaintable = {}
  for each, ch in pairs(chains) do
    if not ch[1] then return end
    if not ch[2] then ch[2] = 0 end
    if not ch[3] then ch[3] = 0  end
    if not ch[4] then ch[4] = "easeNone" end
    if not ch[5] then ch[5] = function() end end
    chaintable[each] = {b = ch[2], c = math.abs(ch[2] - ch[3]), d = ch[1], f = ch[4], h = ch[5], e = ch[3] }
  end
  Library.LibAnUI.activeTicks[chainid] = chaintable


  local funcs = {}
  function funcs:Stop()
    if Library.LibAnUI.activeTicks[chainid] then
      for each, ch in pairs(Library.LibAnUI.activeTicks[chainid]) do
        ch.h(ch.e)
      end
      table.remove(Library.LibAnUI.activeTicks, chainid)
    end
  end

  return funcs

end

table.insert(Event.System.Update.Begin, {Library.LibAnUI.Tick, "LibAnUI", "Animation ticker"})


