------------------------------
-- BreakMover               --
-- Author: Jor @ Deepstrike --
------------------------------

--[[

Licensed to the Apache Software Foundation (ASF) under one or more
contributor license agreements.  See the NOTICE file distributed with
this work for additional information regarding copyright ownership.
The ASF licenses this file to You under the Apache License, Version 2.0
(the "License"); you may not use this file except in compliance with
the License.  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

--]]

BM = {
	version = "0.2",
	self = Inspect.Unit.Lookup("player"),
	abilityId = nil,
	startBreak = nil,
	BreakSpells = {
		["Runebreak"] = true,
		["Runenbrechen"] = true,
		["Salvage Armor"] = true,
		["Salvage Accessories"] = true,
		["Salvage Weapons"] = true,
	},
	Event = {}
}
BreakMover_item = ""
BreakMover_enabled = false

function BM.Init(addon)
	if addon == "BreakMover" then
		print(BM.version .. " Loaded.  Type /bm to see a full list of commands.")
		BM.Enable()
	end
end

function BM.RemoveEvent(event, handler)
	for i, value in ipairs(event) do
		if value[1] == handler then
			table.remove(event, i)
		end
	end
end

function BM.Enable()
	if not BreakMover_enabled then
		return
	end
	table.insert(Event.Unit.Castbar, {BM.Event.Castbar, "BreakMover", "Watch castbar"})
	table.insert(Event.Item.Slot, {BM.Event.Slot, "BreakMover", "Watch slots"})
end

function BM.Disable()
	if BreakMover_enabled then
		return
	end
	BM.RemoveEvent(Event.Unit.Castbar, BM.Event.Castbar)
	BM.RemoveEvent(Event.Item.Slot, BM.Event.Slot)
end

function BM.SlashHandler(str)
	local cmd, arg = string.match(str, "(%a+) ?(.*)")
	if cmd == "set" then
		BreakMover_item = arg
		print("Setting move item to \"" .. arg .. "\"")
	elseif cmd == "enable" then
		if not BreakMover_enabled then
			BreakMover_enabled = true
			BM.Enable()
		end
		print("enabled")
	elseif cmd == "disable" then
		if BreakMover_enabled then
			BreakMover_enabled = false
			BM.Disable()
		end
		print("disabled")
	else
		print("Current move item is \"" .. BreakMover_item .. "\"")
		print("Use /bm set to set the move item, /bm enable/disable to enable or disable the addon")
	end
end

function BM.Event.Castbar(params)
	if not params[BM.self] then
		return
	end
	local castbar = Inspect.Unit.Castbar(BM.self)
	if BM.BreakSpells[castbar.abilityName] then
		BM.startBreak = Inspect.Time.Frame()
	end
end

function BM.Event.Slot(params)
	if not BM.startBreak then
		return
	elseif BreakMover_item == "" or Inspect.Time.Frame() > (BM.startBreak + 4) then
		BM.startBreak = nil
		return
	end
	BM.startBreak = nil
	for slot,hasItem in pairs(params) do
		if not hasItem then
			local items = Inspect.Item.List(Utility.Item.Slot.Inventory())
			local slots = {}
			for s,_ in pairs(items) do
				table.insert(slots, s)
			end
			table.sort(slots)
			for _,s in pairs(slots) do
				local id = items[s]
				if id then
					local item = Inspect.Item.Detail(id)
					if item.name == BreakMover_item and not item.bound then
						Command.Item.Move(id, slot)
						break
					end
				end
			end
		end
	end
end

table.insert(Event.Addon.Load.End, {BM.Init, "BreakMover", "Initital Setup"})
table.insert(Command.Slash.Register("bm"), {BM.SlashHandler, "BreakMover", "Slash Command"})

