--[[
	Project: BookOfTheFallen
	Version: 1.0.0
	Author: NerfedWar (http://www.nerfedwar.net - nerfed.war@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

]]

-- initialise globals
if not BOTF then BOTF = {} end
if not BOTF.Book then BOTF.Book = {} end

-- @field define the Home class
BOTF.Book.Home = class(BOTF.Book.BasicPage)

-- Home class constructor
-- @param book the Book pagesFrame
function BOTF.Book.Home:init(book)

  -- @field store a reference to the Book pagesFrame
  self.theBook = book

  -- create homePVPStats text block
  self.homePVPStats = UI.CreateFrame("Text", "BOTFBookHomePVPStats", self.theBook.bookFrame)
  self.homePVPStats:SetWidth(260)
  self.homePVPStats:SetHeight(180)
  self.homePVPStats:SetPoint("TOPLEFT", self.theBook.bookFrame, "TOPLEFT", 492, 60)
  self.homePVPStats:SetLayer(2)
  self.homePVPStats:SetText("")
  self.homePVPStats:SetFont("BOTF", "MONOFONT.TTF")
  self.homePVPStats:SetFontSize(15)
  self.homePVPStats:SetFontColor(0.3, 0.2, 0.2, 1)
  self.homePVPStats:SetVisible(true)

  -- create homePVPStats text block
  self.homePVEStats = UI.CreateFrame("Text", "BOTFBookHomePVEStats", self.theBook.bookFrame)
  self.homePVEStats:SetWidth(260)
  self.homePVEStats:SetHeight(180)
  self.homePVEStats:SetPoint("TOPLEFT", self.theBook.bookFrame, "TOPLEFT", 492, 290)
  self.homePVEStats:SetLayer(2)
  self.homePVEStats:SetText("")
  self.homePVEStats:SetFont("BOTF", "MONOFONT.TTF")
  self.homePVEStats:SetFontSize(15)
  self.homePVEStats:SetFontColor(0.2, 0.3, 0.2, 1)
  self.homePVEStats:SetVisible(true)

  -- create show alerts icon
  self.icon = UI.CreateFrame("Texture", "BOTFBookHomeBOTFAlertIcon", self.theBook.bookFrame)
  self.icon:SetTexture("BOTF", "media/tick_2.png")
  self.icon:ResizeToTexture()
  self.icon.object = self
  self.icon:SetPoint("TOPLEFT", self.theBook.bookFrame, "TOPLEFT", 320, 400)
  self.icon:SetLayer(2)
  self.icon:SetVisible(true)
  function self.icon.Event:LeftClick(a)
    local _,tex=self:GetTexture()
    if tex=="media/tick_2.png" then
      self:SetTexture("BOTF", "media/tick_1.png")
      BOTF_CONFIG['showAlerts']=false
    else
      self:SetTexture("BOTF", "media/tick_2.png")
      BOTF_CONFIG['showAlerts']=true
    end
  end

    -- create show alerts text
  self.iconText = UI.CreateFrame("Text", "BOTFBookHomeBOTFAlertIconText", self.theBook.bookFrame)
  self.iconText:SetWidth(100)
  self.iconText:SetHeight(40)
  self.iconText:SetPoint("TOPLEFT", self.theBook.bookFrame, "TOPLEFT", 220, 403)
  self.iconText:SetLayer(2)
  self.iconText:SetText("Show alerts ?")
  self.iconText:SetFont("BOTF", "MONOFONT.TTF")
  self.iconText:SetFontSize(16)
  self.iconText:SetFontColor(0, 0, 0, 1)
  self.iconText:SetVisible(true)

  -- create reset text
  self.resetText = UI.CreateFrame("Text", "BOTFBookHomeBOTFAlertResetText", self.theBook.bookFrame)
  self.resetText:SetWidth(300)
  self.resetText:SetHeight(40)
  self.resetText:SetPoint("TOPLEFT", self.theBook.bookFrame, "TOPLEFT", 160, 433)
  self.resetText:SetLayer(2)
  self.resetText:SetText("Type /botfreset into the chat console\n      to reset your statistics.")
  self.resetText:SetFont("BOTF", "MONOFONT.TTF")
  self.resetText:SetFontSize(16)
  self.resetText:SetFontColor(0, 0, 0, 1)
  self.resetText:SetVisible(true)

end

-- update and display the UI elements
function BOTF.Book.Home:show()

  -- set the background of the pageFrame
  self.theBook.pagesFrame:SetTexture("BOTF", "media/book_home.png")

  -- perform statistics calculations

  -- session pvp
  local pvpVictory_SoloKills = 0
  local pvpVictory_KillingBlows = 0
  local pvpVictory_DamageAssists = 0
  local pvpVictory_DistractAssists = 0
  local pvpVictory_HealAssists = 0
  local pvpDefeat_SoloKills = 0
  local pvpDefeat_KillingBlows = 0
  local pvpDefeat_DamageAssists = 0
  local pvpDefeat_DistractAssists = 0
  local pvpDefeat_HealAssists = 0
  local pveSessionScore = 0
  -- session pve
  local pveVictory_SoloKills = 0
  local pveVictory_KillingBlows = 0
  local pveVictory_DamageAssists = 0
  local pveVictory_DistractAssists = 0
  local pveVictory_HealAssists = 0
  local pveDefeat_SoloKills = 0
  local pveDefeat_KillingBlows = 0
  local pveDefeat_DamageAssists = 0
  local pveDefeat_DistractAssists = 0
  local pveDefeat_HealAssists = 0

  local playerLevel = Inspect.Unit.Detail('player').level

  for i, v in pairs(BOTF_DB) do

    if string.sub(i, 1, 2) ~= "__" then
      if v.player then
        pvpVictory_SoloKills = pvpVictory_SoloKills + v.session_statistics.Victory_SoloKills
        pvpVictory_KillingBlows = pvpVictory_KillingBlows + v.session_statistics.Victory_KillingBlows
        pvpVictory_DamageAssists = pvpVictory_DamageAssists + v.session_statistics.Victory_DamageAssists
        pvpVictory_DistractAssists = pvpVictory_DistractAssists + v.session_statistics.Victory_DistractAssists
        pvpVictory_HealAssists = pvpVictory_HealAssists + v.session_statistics.Victory_HealAssists
        pvpDefeat_SoloKills = pvpDefeat_SoloKills + v.session_statistics.Defeat_SoloKills
        pvpDefeat_KillingBlows = pvpDefeat_KillingBlows + v.session_statistics.Defeat_KillingBlows
        pvpDefeat_DamageAssists = pvpDefeat_DamageAssists + v.session_statistics.Defeat_DamageAssists
        pvpDefeat_DistractAssists = pvpDefeat_DistractAssists + v.session_statistics.Defeat_DistractAssists
        pvpDefeat_HealAssists = pvpDefeat_HealAssists + v.session_statistics.Defeat_HealAssists
      else
        pveVictory_SoloKills = pveVictory_SoloKills + v.session_statistics.Victory_SoloKills
        pveVictory_KillingBlows = pveVictory_KillingBlows + v.session_statistics.Victory_KillingBlows
        pveVictory_DamageAssists = pveVictory_DamageAssists + v.session_statistics.Victory_DamageAssists
        pveVictory_DistractAssists = pveVictory_DistractAssists + v.session_statistics.Victory_DistractAssists
        pveVictory_HealAssists = pveVictory_HealAssists + v.session_statistics.Victory_HealAssists
        pveDefeat_SoloKills = pveDefeat_SoloKills + v.session_statistics.Defeat_SoloKills
        pveDefeat_KillingBlows = pveDefeat_KillingBlows + v.session_statistics.Defeat_KillingBlows
        pveDefeat_DamageAssists = pveDefeat_DamageAssists + v.session_statistics.Defeat_DamageAssists
        pveDefeat_DistractAssists = pveDefeat_DistractAssists + v.session_statistics.Defeat_DistractAssists
        pveDefeat_HealAssists = pveDefeat_HealAssists + v.session_statistics.Defeat_HealAssists
        if v.level~='unknown' and tonumber(v.level)>= tonumber(playerLevel) then
          pveSessionScore = pveSessionScore + (((1 * v.session_statistics.Victory_SoloKills) + v.session_statistics.Victory_KillingBlows + v.session_statistics.Victory_DamageAssists + v.session_statistics.Victory_DistractAssists + v.session_statistics.Victory_HealAssists) - ((1 * v.session_statistics.Defeat_SoloKills) + v.session_statistics.Defeat_KillingBlows + v.session_statistics.Defeat_DamageAssists + v.session_statistics.Defeat_DistractAssists + v.session_statistics.Defeat_HealAssists))
        end
      end
    end
  end
  local pvp_score = ((1 * pvpVictory_SoloKills) + pvpVictory_KillingBlows + pvpVictory_DamageAssists + pvpVictory_DistractAssists + pvpVictory_HealAssists) - ((1 * pvpDefeat_SoloKills) + pvpDefeat_KillingBlows + pvpDefeat_DamageAssists + pvpDefeat_DistractAssists + pvpDefeat_HealAssists)
  if pvp_score > 0 then
    pvp_score = "+" .. pvp_score
  end
  local pvp_block = [[
   Victories:
       Solo:              ]] .. string.lpad(pvpVictory_SoloKills, 12, " ") .. [[

       Killing blows:     ]] .. string.lpad(pvpVictory_KillingBlows, 12, " ") .. [[

       Damage assists:    ]] .. string.lpad(pvpVictory_DamageAssists, 12, " ") .. [[

       Distract assists:  ]] .. string.lpad(pvpVictory_DistractAssists, 12, " ") .. [[

       Heal assists:      ]] .. string.lpad(pvpVictory_HealAssists, 12, " ") .. [[

   Defeats:
       Solo:              ]] .. string.lpad(pvpDefeat_SoloKills, 12, " ") .. [[

       Killing blows:     ]] .. string.lpad(pvpDefeat_KillingBlows, 12, " ") .. [[

       Damage assists:    ]] .. string.lpad(pvpDefeat_DamageAssists, 12, " ") .. [[

       Distract assists:  ]] .. string.lpad(pvpDefeat_DistractAssists, 12, " ") .. [[

       Heal assists:      ]] .. string.lpad(pvpDefeat_HealAssists, 12, " ") .. [[

   Score:                 ]] .. string.lpad(pvp_score, 12, " ") .. [[
	]]
  if pveSessionScore > 0 then
    pveSessionScore = "+" .. pveSessionScore
  end
  local pve_block = [[
   Victories:
       Solo:              ]] .. string.lpad(pveVictory_SoloKills, 12, " ") .. [[

       Killing blows:     ]] .. string.lpad(pveVictory_KillingBlows, 12, " ") .. [[

       Damage assists:    ]] .. string.lpad(pveVictory_DamageAssists, 12, " ") .. [[

       Distract assists:  ]] .. string.lpad(pveVictory_DistractAssists, 12, " ") .. [[

       Heal assists:      ]] .. string.lpad(pveVictory_HealAssists, 12, " ") .. [[

   Defeats:
       Solo:              ]] .. string.lpad(pveDefeat_SoloKills, 12, " ") .. [[

       Killing blows:     ]] .. string.lpad(pveDefeat_KillingBlows, 12, " ") .. [[

       Damage assists:    ]] .. string.lpad(pveDefeat_DamageAssists, 12, " ") .. [[

       Distract assists:  ]] .. string.lpad(pveDefeat_DistractAssists, 12, " ") .. [[

       Heal assists:      ]] .. string.lpad(pveDefeat_HealAssists, 12, " ") .. [[

   Score:                 ]] .. string.lpad(pveSessionScore, 12, " ") .. [[
	]]

  -- update the text frames and make them visible
  self.homePVPStats:SetText(pvp_block)
  self.homePVPStats:SetVisible(true)
  self.homePVEStats:SetText(pve_block)
  self.homePVEStats:SetVisible(true)
  self.icon:SetVisible(true)
  self.iconText:SetVisible(true)
  self.resetText:SetVisible(true)
end

-- hide the UI elements
function BOTF.Book.Home:hide()

  -- reset the contents of the text frames and hide them
  self.homePVPStats:SetText('')
  self.homePVPStats:SetVisible(false)
  self.homePVEStats:SetText('')
  self.homePVEStats:SetVisible(false)
  self.icon:SetVisible(false)
  self.iconText:SetVisible(false)
  self.resetText:SetVisible(false)
end

