local buffsaddon = ...

table.insert(Event.Addon.Load.End, {function(addon) if (addon == buffsaddon.identifier) then myUI:RegisterModule(buffsaddon) end end, buffsaddon.identifier, "_startup"})

local L =buffsaddon.data.language

local ifnil = myUI.Utility.ifnil
local isnil = myUI.Utility.isnil
local rgba = myUI.Utility.rgba
local iif = myUI.Utility.iif
local pprint = myUI.Utility.pprint
local round = myUI.Utility.round
local minmax = myUI.Utility.minmax
local lpad = myUI.Utility.lpad
local trim = myUI.Utility.trim
local isempty = myUI.Utility.isempty
local ifempty = myUI.Utility.ifempty
local nilstr = myUI.Utility.nilstr
local notnil = myUI.Utility.notnil
local csv = myUI.Utility.fromCSV

local tinsert = table.insert
local tremove = table.remove
local tsort = table.sort
local min = math.min
local ceil = math.ceil
local floor = math.floor
local sformat = string.format
local sfind = string.find
local sgsub = string.gsub
local abs = math.abs
local slen = string.len
local modf = math.modf

local colors
local myuicolors = myUI.Colors

-- addon --

local _aprofile, _bprofile -- account and character profiles, respectively
local _configFrame = nil
local _sensitivity = 10	-- moving/resizing
 
 local _barsets = {}
 local _boxsets = {}
 
local _available_units = {
	boxes = {
		"player", "player.target", "player.target.target", "player.pet", "player.pet.target", "focus", "focus.target",
		"mouseover", "mouseover.target", "group01", "group01.pet",
	},
	bars = {
		"player", "player.target", "player.target.target", "player.pet", "player.pet.target", "focus", "focus.target",
		"mouseover", "mouseover.target", "group01", "group01.pet",
	}
	

}



local function _buildWBList(ubuff, ubuffSetting, configSetting)
	local available_lists = _aprofile[configSetting]
	local unitlists = ubuff.def[ubuffSetting]
	local output_list = {}
	local added
	
	if (unitlists) then
		unitlists = csv(unitlists)	-- convert our comma-separated string into a table
		if (unitlists and (#unitlists > 0)) then
			for _, listname in ipairs(unitlists) do
				if (available_lists[trim(listname)]) then
					-- there is a defined white/black list with this name so add the buffs to our output list
				
					for _, buff in ipairs(available_lists[trim(listname)]) do
						if (not output_list[buff]) then 
							output_list[buff] = true 
							added = true
						end
					end
				end
		
			end
		end	
	end
	
	if (added) then
		return output_list
	end
end

local function _buildWBLists(ubuff)
	local wlist_buffs = _buildWBList(ubuff, "buff_whitelists", "whitelists")
	local blist_buffs = _buildWBList(ubuff, "buff_blacklists", "blacklists")
	local wlist_debuffs = _buildWBList(ubuff, "debuff_whitelists", "whitelists")
	local blist_debuffs = _buildWBList(ubuff, "debuff_blacklists", "blacklists")
	
	-- If we have both a white and black list, then only things appearring in the white list but not in the blacklist will be show.
	-- We can optimize a bit by simply removing any items from the whitelist that are in the blacklist and setting the blacklist to nil.
	
	if (wlist_buffs and blist_buffs) then
		for buff, _ in pairs(blist_buffs) do
			wlist_buffs[buff] = nil	
		end
		
		blist_buffs = nil
	end
	if (wlist_debuffs and blist_debuffs) then
		for debuff, _ in pairs(blist_debuffs) do
			wlist_debuffs[debuff] = nil
		end
		
		blists_debuffs = nil
	end
	
	return wlist_buffs, blist_buffs, wlist_debuffs, blist_debuffs
end



local function _unitToDef(unit)
	return sgsub(unit, "%d%d", "01")
end

local function _subGroup(unit, group)
	return sgsub(unit, "%d%d", lpad(tostring(group), 2, "0"))
end


local function _setPairs()
	return ipairs({_barsets, _boxsets})
end



local function _timeStrBOX(t)
	if (t) and (t > 0) then
		if (t < 60) then
			return tostring(ceil(t)) .. "s"
		elseif (t >= 3600) then
			return sformat("%.1f", t / 3600) .. "h"
		else
			return sformat("%02d", t/60) .. "m"
		end	
	else
		return nil
	end
end
local function _timeStrBAR(t)
	if (t) and (t > 0) then
		if (t < 10) then
			return sformat("%.1f", t)
		else
			return tostring(ceil(t))
		end
	else
		return nil
	end
end

-- buff box events
local function _mouseInBOX(box)
	if (box:GetVisible() and box.detail._buffid) then
		local info = ""
		
		if (box.detail.description ~= nil) then
			info = box.detail.description
		end
		
		if (box.detail.caster ~= nil) then
			local cdetail = Inspect.Unit.Detail(box.detail.caster)
			if (cdetail) then
				info = info .. "\n\n" .. L["cast by"] .. cdetail.name
			end						
		end
		
		
		myUI:ShowTooltip(box, box.detail.name, info)
	end
end
local function _mouseOutBOX(box)
	if (box:GetVisible()) then
		myUI:HideTooltip(box)
	end
end

local function _rightClickBOX(box)
	if (box:GetVisible() and box.detail._buffid) then
		Command.Buff.Cancel(box.detail._buffid)
		myUI:HideTooltip(box)
	end
end

-- unit frame mover --
local function _leftDownM(mover)
	if (mover.allowdrag) then
		local m = Inspect.Mouse()
		if (m) then
			mover.x = m.x
			mover.y = m.y
			mover.dragging = true
		end
	end
end

local function _leftUpM(mover)
	mover.dragging = false
end
local function _mouseMoveM(mover, x, y)	
	if (mover.allowdrag) and (mover.dragging) then
		local deltax = floor(x - mover.x)
		local deltay = floor(y - mover.y)
		
		-- apply sensitivity setting
		deltax = floor(deltax / _sensitivity) * _sensitivity
		deltay = floor(deltay / _sensitivity) * _sensitivity
		
		
		if (abs(deltax) > 0) or (abs(deltay) > 0) then
			local xnow = mover:GetLeft() + deltax
			local ynow = mover:GetTop() + deltay
			
			mover:SetPoint("TOPLEFT", UIParent, "TOPLEFT", xnow, ynow)
			
			mover.coordtext:SetText(tostring(floor(mover:GetLeft())) .. ", " .. tostring(floor(mover:GetTop())))
			
			local ubuff = mover.ubuff
			local def = ubuff.def
			def.x = xnow
			def.y = ynow
			
			ubuff:Layout(true)
			
			mover.x = mover.x + deltax
			mover.y = mover.y + deltay
		end
	end
end
local function _rightDownM(mover)
	if (mover:GetVisible()) then
		local s = _sensitivity - 1
		if (s < 1) then s = 10 end		
		_sensitivity = s
		
		for _, sets in _setPairs() do
			for _, set in pairs(sets) do
				set.mover.sensitivity:SetText("+/- " .. tostring(_sensitivity))
			end
		end
	end
end

local function _wheelForwardBOXSET(mover)
	local ubuff = mover.ubuff
	local def = ubuff.def
	
	local cxybuff = ifnil(def.cxybuff, ubuff._def.cxybuff) - 1
	if (cxybuff >= 12) then
		def.cxybuff = cxybuff

		if (_configFrame) then
			_configFrame.buffcontrols.boxsets[ubuff.unit].cxybuff:SetText(tostring(cxybuff))
		end
		ubuff:Layout(true)
		
		if (ubuff.group == 1) then	
			-- only boxes will have a .group item so ...
			for i=2,20 do
				_boxsets[_subGroup(ubuff.unit, i)]:Layout(true)
			end
		end
	end
end
local function _wheelBackBOXSET(mover)
	local ubuff = mover.ubuff
	local def = ubuff.def
	
	local cxybuff = ifnil(def.cxybuff, ubuff._def.cxybuff) + 1
	if (cxybuff <= 60) then
		def.cxybuff = cxybuff

		if (_configFrame) then
			_configFrame.buffcontrols.boxsets[ubuff.unit].cxybuff:SetText(tostring(cxybuff))
		end
		ubuff:Layout(true)
		
		if (ubuff.group == 1) then	
			-- only boxes will have a .group item so ...
			for i=2,20 do
				_boxsets[_subGroup(ubuff.unit, i)]:Layout(true)
			end
		end
	end
end
local function _wheelForwardBARSET(mover)
	local ubuff = mover.ubuff
	local def = ubuff.def
	
	local cybar = ifnil(def.cybar, ubuff._def.cybar) - 1
	if (cybar >= 14) then
		def.cybar = cybar
		
		if (_configFrame) then
			_configFrame.buffcontrols.barsets[ubuff.unit].cybar:SetText(tostring(cybar))
		end
		ubuff:Layout(true)
		
		if (ubuff.group == 1) then
			for i=2,20 do
				_barsets[_subGroup(ubuff.unit, i)]:Layout(true)
			end
		end
	end
end
local function _wheelBackBARSET(mover)
	local ubuff = mover.ubuff
	local def = ubuff.def
	
	local cybar = ifnil(def.cybar, ubuff._def.cybar) + 1
	if (cybar <= 32) then
		def.cybar = cybar
		
		if (_configFrame) then
			_configFrame.buffcontrols.barsets[ubuff.unit].cybar:SetText(tostring(cybar))
		end
		ubuff:Layout(true)
		
		if (ubuff.group == 1) then
			for i=2,20 do
				_barsets[_subGroup(ubuff.unit, i)]:Layout(true)
			end
		end
	end
end
-- unit frame sizer --
local function _leftDownS(sizer)
	if (sizer.allowsize) then
		local m = Inspect.Mouse()
		if (m) then
			sizer.x = m.x
			sizer.y = m.y
			sizer.sizing = "size"
		end
	end
end
local function _rightDownS(sizer)
	if (sizer.allowsize) then
		local m = Inspect.Mouse()
		if (m) then 
			sizer.x = m.x
			sizer.y = m.y
			sizer.sizing = "spacing"
		end
	end
end

local function _leftUpS(sizer)
	sizer.sizing = false
end

local function _mouseMoveS(sizer, x, y)
	if (sizer.allowsize) and (sizer.sizing) then
		local deltax = floor(x - sizer.x)
		local deltay = floor(y - sizer.y)
		
		if (sizer.sizing == "spacing") then			
			-- should only happend for 'group01' and 'group01.pet'
			
			deltax = floor(deltax/10)*10	-- make this less sensitive than normal resizing
			deltay = floor(deltay/10)*10
					
			if (abs(deltax) > 0) or (abs(deltay) > 0) then
				local mover = sizer:GetParent()	
				local ubuff = mover.ubuff
				
				if (ubuff.group == 1) then					
					local colspacing = minmax(ifnil(ubuff.def.colspacing,0) + deltax, 0, 200)
					local rowspacing = minmax(ifnil(ubuff.def.rowspacing,0) + deltay,0, 200)
					
					ubuff.def.colspacing = colspacing
					ubuff.def.rowspacing = rowspacing
					
					for i=1,20 do
						mover.set[_subGroup(ubuff.unit, i)]:Layout(true)
					end
					
					sizer.x = sizer.x + deltax
					sizer.y = sizer.y + deltay
				end
			end
		else
			-- apply sensitivity setting
			deltax = floor(deltax / _sensitivity) * _sensitivity
			deltay = floor(deltay / _sensitivity) * _sensitivity
			
			if (abs(deltax) > 0) or (abs(deltay) > 0) then
				local mover = sizer:GetParent()
				
				local cxmover = mover:GetWidth()
				local cymover = mover:GetHeight()
				
				local cxnow = cxmover + deltax
				local cynow = cymover + deltay
				
				cxnow = floor(minmax(cxnow, 40, UIParent:GetWidth()))
				cynow = floor(minmax(cynow, 28, UIParent:GetHeight()))
				
				if (cxnow ~= cxmover) or (cynow ~= cymover) then
					mover.sizetext:SetText(tostring(cxnow) .. " x " .. tostring(cynow))
					local ubuff = mover.ubuff
					
					ubuff.def.cx = cxnow
					ubuff.def.cy = cynow
					
					ubuff:Layout(true)
					
					if (ubuff.group == 1) then	
						-- only boxes will have a .group item so ...
						for i=2,20 do
							mover.set[_subGroup(ubuff.unit, i)]:Layout(true)
						end
					end
					
					sizer.x = sizer.x + deltax
					sizer.y = sizer.y + deltay				
				end
			
			end
		end
	end
end



-- boxes/bars --

local LAYER_BUFFS = myUI.Layers.UnitFrame + 2
local LAYER_MOVER = LAYER_BUFFS + 20
local LAYER_ICON = 2
local LAYER_BAR =  1
local LAYER_TEXT = 4
local LAYER_CASTER = 3


local _recycledBOXES = {}
local _recycledBARS = {}


local function _disposeBOX(box)
	box:SetVisible(false)	
	tinsert(_recycledBOXES, box)
end
local function _disposeBAR(bar)
	bar:SetVisible(false)
	tinsert(_recycledBARS, bar)
end

local function _newBOX(boxset)
	local box
	if (#_recycledBOXES > 0) then
		box = tremove(_recycledBOXES)
		box:SetParent(boxset)		
	else
		box = myUI:CreateFrame("Frame", boxset)
		box.icon = myUI:CreateFrame("Texture", box); box.icon:SetLayer(LAYER_ICON)
		box.caster = myUI:CreateFrame("Frame", box); box.caster:SetLayer(LAYER_CASTER)
		box.stacks = myUI:CreateFrame("Text", box); box.stacks:SetLayer(LAYER_TEXT); box.stacks.customFontSize = true
		box.remaining = myUI:CreateFrame("Text", box); box.remaining:SetLayer(LAYER_TEXT); box.remaining.customFontSize = true
		
		box:SetMouseMasking("limited")
		box.Event.RightDown = _rightClickBOX
		box.Event.MouseIn = _mouseInBOX
		box.Event.MouseOut = _mouseOutBOX
		
		box.Dispose = _disposeBOX
		
	end
	
	box.remaining:SetVisible(false)
	box.stacks:SetVisible(false)
	
	box.detail = {}
	return box
end

local function _newBAR(barset)
	local bar
	if (#_recycledBARS > 0) then
		bar = tremove(_recycledBARS)
		bar:SetParent(barset)
	else
		bar = myUI:CreateFrame("Frame", barset)
		bar.icon = myUI:CreateFrame("Texture", bar); bar.icon:SetLayer(LAYER_ICON)
		bar.caster = myUI:CreateFrame("Frame", bar); bar.caster:SetLayer(LAYER_CASTER)
		bar.remaining = myUI:CreateFrame("Text", bar); bar.remaining:SetLayer(LAYER_TEXT); bar.remaining.customFontSize = true
		bar.name = myUI:CreateFrame("Text", bar); bar.name:SetLayer(LAYER_TEXT); bar.name.customFontSize = true
		bar.bar = myUI:CreateFrame("Frame", bar); bar.bar:SetLayer(LAYER_BAR)
		
		bar.Dispose = _disposeBAR
	end
	
	bar.detail = {}
	
	return bar
end

local function _getDefinitionBOX(unit)
	unit = _unitToDef(unit)		--  group01 and group01.pet have the settings for all group frames
	local def = _aprofile.boxsets[unit]
	
	-- make sure some of the sub-tables exist 
	if (not def.caster_indicator) then def.caster_indicator = {} end
	if (not def.remaining) then def.remaining = {} end
	if (not def.stacks) then def.stacks = {} end
	
	return def, _aprofile.boxset		-- specific definition + defaults
end

local function _getDefinitionBAR(unit)
	local unit = _unitToDef(unit)
	local def = _aprofile.barsets[unit]
	
	-- make sure sub-tables exist
	if (not def.caster_indicator) then def.caster_indicator = {} end
	if (not def.remaining) then def.remaining = {} end
	if (not def.name) then def.name = {} end
	
	return def, _aprofile.barset		-- specific definition + defaults
end


local function _recacheSET(ubuff)
	local def = ubuff.def
	local _def = ubuff._def
	
	-- cacheing these for later but make sure to recache if these params change in the config
	ubuff.show_own_buffs = ifnil(def.show_own_buffs, _def.show_own_buffs)
	ubuff.show_units_buffs = ifnil(def.show_units_buffs, _def.show_units_buffs)
	ubuff.show_others_buffs = ifnil(def.show_others_buffs, _def.show_others_buffs)
	ubuff.show_own_debuffs = ifnil(def.show_own_debuffs, _def.show_own_debuffs)
	ubuff.show_units_debuffs = ifnil(def.show_units_debuffs, _def.show_units_debuffs)
	ubuff.show_others_debuffs = ifnil(def.show_others_debuffs, _def.show_others_debuffs)
end


local function _layoutBOXSET(ubuff, doExternal)
	local def = ubuff.def
	local _def = ubuff._def
	local mover = ubuff.mover
	local cxybuff = ifnil(def.cxybuff, _def.cxybuff)
	local padding = ifnil(def.padding, _def.padding)
	
	if (doExternal) then
		mover:ClearAll()
		mover:SetSize(ifnil(def.cx, _def.cx), ifnil(def.cy, _def.cy))
		
		if (ubuff.group == nil) or (ubuff.group == 1) then		
			mover:SetPoint("TOPLEFT", UIParent, "TOPLEFT", ifnil(def.x, _def.x), ifnil(def.y, _def.y))
		
			mover.coordtext:SetText(tostring(floor(mover:GetLeft())) .. ", " .. tostring(floor(mover:GetTop())))
			mover.sizetext:SetText(tostring(floor(mover:GetWidth())) .. " x " .. tostring(floor(mover:GetHeight())))
		else
			local group = ubuff.group
			local col = modf((group-1)/5)
			local row = group - ((col) * 5) - 1
			
			local xoff = (ifnil(def.cx, _def.cx) * col) + (ifnil(def.colspacing, _def.colspacing) * col)
			local yoff = (ifnil(def.cy, _def.cy) * row) + (ifnil(def.rowspacing, _def.rowspacing) * row)
			
			local anchor = _boxsets[_subGroup(ubuff.unit, 1)]
			mover:SetPoint("TOPLEFT", anchor, "TOPLEFT", xoff, yoff)
			mover.sizer:SetVisible(false)
		
			mover.coordtext:SetText("")
			mover.sizetext:SetText("")
		end
		
		local cx = mover:GetWidth()
		local cy = mover:GetHeight()
		
		local cols = floor((cx + padding) / (cxybuff + padding))
		local rows = floor((cy + padding) / (cxybuff + padding))
		
		ubuff.cols = cols
		ubuff.rows = rows
		
		local nboxes = cols * rows
		
		while (#ubuff.boxes < nboxes) do
			tinsert(ubuff.boxes, _newBOX(ubuff))
		end
		
		while (#ubuff.boxes > nboxes) do
			tremove(ubuff.boxes):Dispose()
		end
	end
	
	local col = 0
	local row = 0
	local maxcol = ubuff.cols
	local maxrow = ubuff.rows
	local m = ifnil(def.margin, _def.margin)
	local anchor
	local grow = iif(ifnil(def.grow_up, _def.grow_up), "BOTTOMLEFT", "TOPLEFT")
	local dir = iif(grow == "TOPLEFT", 1, -1)
	
	ubuff:ClearAll()
	
	if (ifnil(def.grow_up, _def.grow_up)) then
		grow = "BOTTOMLEFT"
		dir = -1
		ubuff:SetPoint("BOTTOMLEFT", mover, "BOTTOMLEFT")
	else
		grow = "TOPLEFT"
		dir = 1
		ubuff:SetPoint("TOPLEFT", mover, "TOPLEFT")
	end	
	
	for _, box in ipairs(ubuff.boxes) do
		box:ClearAll()
		box:SetBackgroundColor(0,0,0,0.4)	-- until it's updated with a specific buff
		box:SetSize(cxybuff)
	
		box.icon:ClearAll()
		box.icon:SetPoint("TOPLEFT", box, "TOPLEFT", m, m)
		box.icon:SetPoint("BOTTOMRIGHT", box, "BOTTOMRIGHT", -m, -m)
		box.icon:SetBackgroundColor(0,0,0,0)
		
		box.stacks:ClearAll()
		box.stacks:SetBackgroundColor(rgba(ifnil(def.stacks.bgcolor, _def.stacks.bgcolor)))
		box.stacks:SetFontColor(rgba(ifnil(def.stacks.fgcolor, _def.stacks.fgcolor)))
		box.stacks:SetFontSize(ifnil(def.stacks.fontsize, _def.stacks.fontsize))
		anchor = ifnil(def.stacks.anchor, _def.stacks.anchor)
		box.stacks:SetPoint(anchor, box, anchor, ifnil(def.stacks.xoff, _def.stacks.xoff), ifnil(def.stacks.yoff, _def.stacks.yoff))
		
		box.remaining:ClearAll()
		box.remaining:SetBackgroundColor(rgba(ifnil(def.remaining.bgcolor, _def.remaining.bgcolor)))
		box.remaining:SetFontColor(rgba(ifnil(def.remaining.fgcolor, _def.remaining.fgcolor)))
		box.remaining:SetFontSize(ifnil(def.remaining.fontsize, _def.remaining.fontsize))
		anchor = ifnil(def.remaining.anchor, _def.remaining.anchor)
		box.remaining:SetPoint(anchor, box, anchor, ifnil(def.remaining.xoff, _def.remaining.xoff), ifnil(def.remaining.yoff, _def.remaining.yoff))
		
		box.caster:ClearAll()
		box.caster:SetSize(ifnil(def.caster_indicator.cx, _def.caster_indicator.cx), ifnil(def.caster_indicator.cy, _def.caster_indicator.cy))		
		anchor = ifnil(def.caster_indicator.anchor, _def.caster_indicator.anchor)
		box.caster:SetPoint(anchor, box, anchor, ifnil(def.caster_indicator.xoff, _def.caster_indicator.xoff), ifnil(def.caster_indicator.yoff, _def.caster_indicator.yoff))
		
		if (col < maxcol) then
			col = col + 1
		else
			col = 1
			row = row + 1
		end
		
		box:SetPoint(grow, ubuff, grow, ((col-1) * cxybuff)+((col-2) * padding), (row * cxybuff + ((row-1) * padding) + padding) * dir)
		
	end
	
	ubuff.buff_whitelist, ubuff.buff_blacklist, ubuff.debuff_whitelist, ubuff.debuff_blacklist = _buildWBLists(ubuff)
	_recacheSET(ubuff)
	
end
local function _layoutBARSET(ubuff, doExternal)
	local def = ubuff.def
	local _def = ubuff._def
	local mover = ubuff.mover
	local cybar = ifnil(def.cybar, _def.cybar)
	local padding = ifnil(def.padding, _def.padding)
	
	if (doExternal) then
		mover:ClearAll()
		mover:SetSize(ifnil(def.cx, _def.cx), ifnil(def.cy, _def.cx))
		
		if (ubuff.group == nil) or (ubuff.group == 1) then
			mover:SetPoint("TOPLEFT", UIParent, "TOPLEFT", ifnil(def.x, _def.x), ifnil(def.y, _def.x))
		
			mover.coordtext:SetText(tostring(floor(mover:GetLeft())) .. ", " .. tostring(floor(mover:GetTop())))
			mover.sizetext:SetText(tostring(floor(mover:GetWidth())) .. " x " .. tostring(floor(mover:GetHeight())))
		else
			local group = ubuff.group
			local col = modf((group-1)/5)
			local row = group - ((col) * 5) - 1
			
			local xoff = (ifnil(def.cx, _def.cx) * col) + (ifnil(def.colspacing, _def.colspacing) * col)
			local yoff = (ifnil(def.cy, _def.cy) * row) + (ifnil(def.rowspacing, _def.rowspacing) * row)
			
			local anchor = _barsets[_subGroup(ubuff.unit, 1)]
			mover:SetPoint("TOPLEFT", anchor, "TOPLEFT", xoff, yoff)
			mover.sizer:SetVisible(false)
			
			mover.coordtext:SetText("")
			mover.sizetext:SetText("")
		end
		
		local cx = mover:GetWidth()
		local cy = mover:GetHeight()
		local rows = floor((cy + padding) / (cybar + padding))
		
		while (#ubuff.bars < rows) do
			tinsert(ubuff.bars, _newBAR(ubuff))
		end
		while (#ubuff.bars > rows) do
			tremove(ubuff.bars):Dispose()
		end
	end
	
	local cxcaster = iif(ifnil(def.caster_indicator.show, _def.caster_indicator.show), 8, 0)
	
	local row = 1
	local grow_up = ifnil(def.grow_up, _def.grow_up)
	local yoff 
	
	for _, bar in ipairs(ubuff.bars) do
		
		bar:SetBackgroundColor(0,0,0,0.4)	-- until it's updated with a specific buff
		
		bar.caster:ClearAll()		
		bar.caster:SetPoint("LEFTCENTER", bar, "LEFTCENTER", 0, 0)		
		bar.caster:SetSize(cxcaster,cybar)
		
		bar.icon:ClearAll()
		bar.icon:SetSize(cybar)
		bar.icon:SetPoint("LEFTCENTER", bar.caster, "RIGHTCENTER", 0, 0)
		
		bar.bar:ClearAll()
		bar.bar:SetHeight(cybar)
		bar.bar:SetPoint("LEFTCENTER", bar.icon, "RIGHTCENTER", 0, 0)
		
		bar.remaining:ClearAll()
		bar.remaining:SetFontSize(ifnil(def.remaining.fontsize, _def.remaining.fontsize))
		bar.remaining:SetFontColor(rgba(ifnil(def.remaining.fgcolor, _def.remaining.fgcolor)))
		bar.remaining:SetPoint("RIGHTCENTER", bar, "RIGHTCENTER", -4, 0)
			
		bar.name:ClearAll()
		bar.name:SetFontSize(ifnil(def.name.fontsize, _def.name.fontsize))
		bar.name:SetFontColor(rgba(ifnil(def.name.fgcolor, _def.name.fgcolor)))
		bar.name:SetPoint("LEFTCENTER", bar.icon, "RIGHTCENTER", 4, 0)
		bar.name:SetPoint("RIGHTCENTER", bar.remaining, "LEFTCENTER", -4, 0)
		
		yoff = (row-1) * (cybar + padding)
		
		bar:ClearAll()
		if (grow_up) then 
			bar:SetPoint("BOTTOMLEFT", ubuff.mover, "BOTTOMLEFT", 0, -yoff)
			bar:SetPoint("BOTTOMRIGHT", ubuff.mover, "BOTTOMRIGHT", 0, -yoff)
		else
			bar:SetPoint("TOPLEFT", ubuff.mover, "TOPLEFT", 0, yoff)
			bar:SetPoint("TOPRIGHT", ubuff.mover, "TOPRIGHT", 0, yoff)			
		end
		bar:SetHeight(cybar)
		
		row = row + 1
	end
	
	ubuff.cx = ubuff.mover:GetWidth() - cxcaster - cybar
	
	ubuff.buff_whitelist, ubuff.buff_blacklist, ubuff.debuff_whitelist, ubuff.debuff_blacklist = _buildWBLists(ubuff)	
	_recacheSET(ubuff)
end


local function _updateBOXSET(boxset, now)
	if not (boxset.rows and boxset.cols) then return end -- layout hasn't been done yet. "Shouldn't" happen but better to be safe and check.
	
	local good = {}
	local def = boxset.def
	local _def = boxset._def
	
	local buffids = Inspect.Buff.List(boxset.unit)
	
	if (buffids) then
		local details = Inspect.Buff.Detail(boxset.unit, buffids)
		if (details) then
			local pid = Inspect.Unit.Lookup("player")
			local tid = Inspect.Unit.Lookup(boxset.unit)
			
			local show_own_buffs = boxset.show_own_buffs
			local show_units_buffs = boxset.show_units_buffs
			local show_others_buffs = boxset.show_others_buffs
			local show_own_debuffs = boxset.show_own_debuffs
			local show_units_debuffs = boxset.show_units_debuffs
			local show_others_debuffs = boxset_show_others_debuffs
			
			local buff_whitelist = boxset.buff_whitelist
			local buff_blacklist = boxset.buff_blacklist
			local debuff_whitelist = boxset.debuff_whitelist
			local debuff_blacklist = boxset.debuff_blacklist
			
			for buffid, detail in pairs(details) do
				detail._byPlayer = (detail.caster == pid)
				detail._byUnit = (detail.caster == tid) -- note, it's possible to be both byPlayer and byUnit
				detail._byOther = not (detail._byPlayer or detail._byUnit)
				detail._buffid = buffid
				
				if (detail.remaining == nil) or (detail.remaining > 0) then
					if (detail.debuff) then	
						if (not debuff_whitelist) or (debuff_whitelist[detail.name]) then
							if (not debuff_blacklist) or (not debuff_blacklist[detail.name]) then
								if (detail._byPlayer) then
									if (show_own_debuffs) then tinsert(good, detail) end
								elseif (detail._byUnit) then
									if (show_units_debuffs) then tinsert(good, detail) end
								else
									if (show_others_debuffs) then tinsert(good, detail) end
								end								
							end
						end
					else			
						if (not buff_whitelist) or (buff_whitelist[detail.name]) then
							if (not buff_blacklist) or (not buff_blacklist[detail.name]) then
								if (detail._byPlayer) then
									if (show_own_buffs) then tinsert(good, detail) end
								elseif (detail._byUnit) then
									if (show_units_buffs) then tinsert(good, detail) end
								else
									if (show_others_buffs) then tinsert(good, detail) end
								end
							end
						end
					end
				end	

				if (details.description) then
					detail.description = details.description
				end
			end
		end
	end
	
	local maxboxes = boxset.cols * boxset.rows
	local ngood = #good
	local mover = boxset.mover:GetVisible()
	
	local show_caster = ifnil(def.caster_indicator.show, _def.caster_indicator.show)
	local show_remaining = ifnil(def.remaining.show, _def.remaining.show)
	local show_stacks = ifnil(def.stacks.show, _def.stacks.show)
	
	-- sort our buffs/debuffs
	if (ngood > 1) then
		tsort(good, function(a, b)
			if (a.debuff == b.debuff) then
				if (a._byPlayer == b._byPlayer) then
					if (a._byUnit == b._byUnit) then
						if (a.remaining == b.remaining) then
							return (a.name < b.name)
						else
							if (a.remaining == nil) then
								return false
							elseif (b.remaining == nil) then
								return true
							else
								return a.remaining < b.remaining
							end
						end
					else
						return a._byUnit	-- unit cast before other cast
					end				
				else
					return a._byPlayer	-- returns true if a is player cast
				end
			else
				return  b.debuff 	--  return true b/c  a is a buff
			end
		
		end)
		
	end
	
	local detial, bdetail, tstring
	for i, box in ipairs(boxset.boxes) do
		if (i > ngood) then
			if (mover) then
				box:SetVisible(true)
			else
				if (box:GetVisible()) then box:SetVisible(false) end
			end
		else
			detail = good[i]
			
			if (box._reset) then
				bdetail = {}; box._reset = nil
			else
				bdetail = box.detail
			end
			
			if (detail.icon ~= bdetail.icon) then box.icon:SetTexture("Rift", detail.icon) end
			
			if (show_stacks) then
				if (detail.stack ~= bdetail.stack) then
					if (detail.stack) then
						box.stacks:SetText(tostring(detail.stack))
						if (not box.stacks:GetVisible()) then box.stacks:SetVisible(true) end
					else
						if (box.stacks:GetVisible()) then box.stacks:SetVisible(false) end
					end
				end
			else
				if (box.stacks:GetVisible()) then box.stacks:SetVisible(false) end
			end
			
			if (show_caster) then
				if (detail._byPlayer) then --and (not bdetail._byPlayer) then
					box.caster:SetBackgroundColor(rgba(colors.cast_by_player))
				elseif (detail._byUnit) then --and (not bdetail._byUnit) then
					box.caster:SetBackgroundColor(rgba(colors.cast_by_unit))
				elseif (detail._byOther) then --and (not bdetail._byOther) then
					box.caster:SetBackgroundColor(rgba(colors.cast_by_other))
				end
				
				if (not box.caster:GetVisible()) then box.caster:SetVisible(true) end
			else
				if (box.caster:GetVisible()) then box.caster:SetVisible(false) end
			end
			
			if (detail.curse and (not bdetail.curse)) then
				box:SetBackgroundColor(rgba(colors.debuff_curse))
			elseif (detail.disease and (not bdetail.disease)) then
				box:SetBackgroundColor(rgba(colors.debuff_disease))
			elseif (detail.poison and (not bdetail.poison)) then
				box:SetBackgroundColor(rgba(colors.debuff_poison))
			elseif (detail.debuff) then
				box:SetBackgroundColor(rgba(colors.debuff))
			elseif (detail.noncancelable and (not bdetail.noncancelable)) then
				box:SetBackgroundColor(rgba(colors.buff_nocancel))
			elseif (not bdetail.debuff) and (not bdetail.noncancelable) then
				box:SetBackgroundColor(rgba(colors.buff))				
			end
			
			if (show_remaining) then
				if (detail.remaining ~= bdetail.remaining) then
					tstring = _timeStrBOX(detail.remaining)
					if (tstring) then
						if (tstring ~= box.remaining:GetText()) then 
							box.remaining:SetText(tstring)							
						end
						if (not box.remaining:GetVisible()) then box.remaining:SetVisible(true) end
					else
						if (box.remaining:GetVisible()) then box.remaining:SetVisible(false) end
					end
				end
			else
				if (box.remaining:GetVisible()) then box.remaining:SetVisible(false) end
			end
				
			box.detail = detail			
			if (not box:GetVisible()) then box:SetVisible(true) end
		end	
	end
	
	
end

local function _updateBARSET(barset, now)
	if (not barset.cx) then return end
	
	local buffids = Inspect.Buff.List(barset.unit)
	local good = {}
	local def = barset.def
	local _def = barset._def
	
	if (buffids) then
		local details = Inspect.Buff.Detail(barset.unit, buffids)
		if (details) then
			local pid = Inspect.Unit.Lookup("player")
			local tid = Inspect.Unit.Lookup(barset.unit)
			
			local show_own_buffs = barset.show_own_buffs
			local show_units_buffs = barset.show_units_buffs
			local show_others_buffs = barset.show_others_buffs
			local show_own_debuffs = barset.show_own_debuffs
			local show_units_debuffs = barset.show_units_debuffs
			local show_others_debuffs = barset.show_others_debuffs
			local maxd = ifnil(def.max_duration, _def.max_duration)
			
			local buff_whitelist = barset.buff_whitelist
			local buff_blacklist = barset.buff_blacklist
			local debuff_whitelist = barset.debuff_whitelist
			local debuff_blacklist = barset.debuff_blacklist
			
			for buffid, detail in pairs(details) do
				detail._byPlayer = (detail.caster == pid)
				detail._byUnit = (detail.caster == tid) -- note, it's possible to be both byPlayer and byUnit
				detail._byOther = not (detail._byPlayer or detail._byUnit)
				detail._buffid = buffid
				
				if (detail.remaining) and (detail.duration) and (detail.duration <= maxd) and (detail.remaining > 0) then					
					if (detail.debuff) then
						if (not debuff_whitelist) or (debuff_whitelist[detail.name]) then
							if (not debuff_blacklist) or (not debuff_blacklist[name]) then
								if (detail._byPlayer) then
									if (show_own_debuffs) then tinsert(good, detail) end
								elseif (detail._byUnit) then
									if (show_units_debuffs) then tinsert(good, detail) end
								else
									if (show_others_debuffs) then tinsert(good, detail) end
								end								
							end
						end
					else			
						if (not buff_whitelist) or (buff_whitelist[detail.name]) then
							if (not buff_blacklist) or (not buff_blacklist[detail.name]) then
								if (detail._byPlayer) then
									if (show_own_buffs) then tinsert(good, detail) end
								elseif (detail._byUnit) then
									if (show_units_buffs) then tinsert(good, detail) end
								else
									if (show_others_buffs) then tinsert(good, detail) end
								end
							end
						end
					end					
				end				
			end
		
		end	
	end

	
	local ngood = #good
	local mover = barset.mover:GetVisible()
	local show_caster = ifnil(def.caster_indicator.show, _def.caster_indicator.show)
	
	-- sort our buffs/debuffs
	if (ngood > 1) then
		tsort(good, function(a, b)
			if (a.debuff == b.debuff) then
				if (a._byPlayer == b._byPlayer) then
					if (a._byUnit == b._byUnit) then
						if (a.remaining == b.remaining) then
							return (a.name < b.name)
						else
							if (a.remaining == nil) then
								return false
							elseif (b.remaining == nil) then
								return true
							else
								return a.remaining < b.remaining
							end
						end
					else
						return a._byUnit	-- unit cast before other cast
					end				
				else
					return a._byPlayer	-- returns true if a is player cast
				end
			else
				return  b.debuff 	--  return true b/c  a is a buff
			end
		
		end)
		
	end
	

	local detail, bdetail, tstring, nstring
	for i, bar in ipairs(barset.bars) do
		if (i > ngood) then
			if (mover) then
				bar:SetVisible(true)
			else
				if (bar:GetVisible()) then bar:SetVisible(false) end
			end
		else
			detail = good[i]
			if (bar._reset) then
				bdetail = {}; bar._reset = nil
			else
				bdetail = bar.detail
			end
			
			bar.bar:SetWidth(minmax((detail.remaining / detail.duration) * barset.cx, 0, barset.cx))
			
			if (detail.icon ~= bdetail.icon) then bar.icon:SetTexture("Rift", detail.icon) end
			
			nstring = detail.name
			if (detail.stack) then
				nstring = nstring .. " (" .. tostring(detail.stack) .. ")"
			end
			if (nstring ~= bar.name:GetText()) then
				bar.name:SetText(nstring)
			end
			
			if (detail.remaining ~= bdetail.remaining) then
				tstring = _timeStrBAR(detail.remaining)
				if (tstring) then
					if (tstring ~= bar.remaining:GetText()) then
						bar.remaining:SetText(tstring)
					end
					if (not bar.remaining:GetVisible()) then bar.remaining:SetVisible(true) end
				else
					if (bar.remaining:GetVisible()) then bar.remaining:SetVisible(false) end
				end
			end
			
			if (show_caster) then
				if (detail._byPlayer) then
					bar.caster:SetBackgroundColor(rgba(colors.cast_by_player))
				elseif (detail._byUnit) then
					bar.caster:SetBackgroundColor(rgba(colors.cast_by_unit))
				elseif (detail._byOther) then
					bar.caster:SetBackgroundColor(rgba(colors.cast_by_other))
				end
				
				if (not bar.caster:GetVisible()) then bar.caster:SetVisible(true) end
			else
				if (bar.caster:GetVisible()) then bar.caster:SetVisible(false) end
			end
			
			if (detail.curse and (not bdetail.curse)) then
				bar.bar:SetBackgroundColor(rgba(colors.debuff_curse))
			elseif (detail.disease and (not bdetail.disease)) then
				bar.bar:SetBackgroundColor(rgba(colors.debuff_disease))
			elseif (detail.poison and (not bdetail.poison)) then
				bar.bar:SetBackgroundColor(rgba(colors.debuff_poison))
			elseif (detail.debuff) then
				bar.bar:SetBackgroundColor(rgba(colors.debuff))
			elseif (detail.noncancelable and (not bdetail.noncancelable)) then
				bar.bar:SetBackgroundColor(rgba(colors.buff_nocancel))
			elseif (not bdetail.debuff) and (not bdetail.noncancelable) then
				bar.bar:SetBackgroundColor(rgba(colors.buff))				
			end
			
			
			bar.detail = detail
			if (not bar:GetVisible()) then bar:SetVisible(true) end
		end
	end
	
end
--
local _recycledBOXSETS = {}
local _recycledBARSETS = {}

local function _resetBOXSET(boxset)
	for _, box in pairs(boxset.boxes) do
		box._reset = true
	end
end
local function _resetBARSET(barset)
	for _, bar in pairs(barset.bars) do
		bar._reset = true
	end
end

local function _disposeSET(ubuff)
	ubuff:SetVisible(false)
	ubuff.mover:SetVisible(false)
	ubuff.enabled = false	
end
local function _disposeBOXSET(ubuff)
	for _, box in ipairs(ubuff.boxes) do
		box:Dispose()
	end
	ubuff.boxes = {}

	_disposeSET(ubuff)
	
	ubuff.lastupdated = 0
	
	table.insert(_recycledBOXSETS, ubuff)
end
local function _disposeBARSET(ubuff)
	for _, bar in ipairs(ubuff.bars) do
		bar:Dispose()
	end
	ubuff.bars = {}
	
	_disposeSET(ubuff)
	
	table.insert(_recycledBARSETS, ubuff)
end


local function _initSET(ubuff, unit, group, fnGetDef)
	ubuff.enabled = false
	
	local mover = ubuff.mover
	local sizer = mover.sizer
	
	ubuff.unit = unit	
	mover.unit = unit 
	sizer.unit = unit
	
	ubuff.isplayer = (unit == "player")
	ubuff.ispet = (unit == "player.pet")	
	ubuff.istarget = (unit == "player.target")
	ubuff.ismouseover = (string.find(unit, "mouseover") == 1)
	ubuff.group = group
	
	local canmove = (group == nil) or (group == 1)
	
	mover.allowdrag = canmove
	sizer.allowsize = canmove

	mover.unittext:SetVisible(canmove)
	mover.coordtext:SetVisible(canmove)
	mover.sizetext:SetVisible(canmove)
	mover.sensitivity:SetVisible(canmove)
		
	mover.unittext:SetText(unit)
	
	local def, _def = fnGetDef(unit)
	
	ubuff.def = def
	ubuff._def = _def
	
	ubuff.enabled = true
	
	if (ubuff.group ~= nil) then
		sizer.Event.RightUp = _leftUpS
		sizer.Event.RightUpoutside = _leftUpS
		sizer.Event.RightDown = _rightDownS
	else
		sizer.Event.RightUp = nil
		sizer.Event.RightUpoutside = nil
		sizer.Event.RightDown = nil
	end
	
	ubuff:Layout(true)	
	
	ubuff.enabled = true
end

local function _initBOXSET(ubuff, unit, group)
	_initSET(ubuff, unit, group, _getDefinitionBOX)
	
	if (ubuff.group == nil) or (ubuff.group == 1) then
		ubuff.mover.Event.WheelForward = _wheelForwardBOXSET
		ubuff.mover.Event.WheelBack = _wheelBackBOXSET
	else
		ubuff.mover.Event.WheelBack = nil
		ubuff.mover.Event.WheelForward = nil
	end
	
	ubuff.mover.unittext:SetText(L["boxes"] .. ": " .. ubuff.unit)
end

local function _initBARSET(ubuff, unit, group)
	_initSET(ubuff, unit, group, _getDefinitionBAR)
	
	if (ubuff.group == nil) or (ubuff.group == 1) then
		ubuff.mover.Event.WheelForward = _wheelForwardBARSET
		ubuff.mover.Event.WheelBack = _wheelBackBARSET
	else
		ubuff.mover.Event.WheelForward = nil
		ubuff.mover.Event.WheelBack = nil
	end
	
	ubuff.mover.unittext:SetText(L["barsets"] .. ": " .. ubuff.unit)
end








local function _newSET()
	local ubuff = myUI:CreateFrame("Frame")
	local mover = myUI:CreateFrame("Frame")
	local sizer = myUI:CreateFrame("Texture", mover)	
	
	ubuff.mover = mover
	mover.sizer = sizer
	mover.ubuff = ubuff -- note: circular reference
	
	mover:SetSecureMode("normal")
	mover:SetLayer(LAYER_MOVER)
	mover:SetBackgroundColor(rgba(myuicolors.Mover))
	mover:SetVisible(false)
	
	mover.unittext = myUI:CreateFrame("Text", mover)
	mover.unittext:SetPoint("BOTTOMLEFT", mover, "TOPLEFT", 0, -4)
	mover.unittext:SetFontSize(12); mover.unittext.customFontSize = true	
	mover.unittext:SetBackgroundColor(0.6,0,0,0.6)
	
	mover.coordtext = myUI:CreateFrame("Text", mover)
	mover.coordtext:SetPoint("TOPLEFT", mover.unittext, "TOPRIGHT", 4, 0)
	mover.coordtext:SetFontSize(12); mover.coordtext.customFontSize = true
	mover.coordtext:SetBackgroundColor(0.6,0,0,0.6)
	
	mover.sizetext = myUI:CreateFrame("Text", mover)
	mover.sizetext:SetPoint("TOPLEFT", mover.coordtext, "TOPRIGHT", 4, 0)
	mover.sizetext:SetFontSize(12); mover.sizetext.customFontSize = true
	mover.sizetext:SetBackgroundColor(0.6,0,0,0.6)
	
	mover.sensitivity = myUI:CreateFrame("Text", mover)
	mover.sensitivity:SetPoint("TOPLEFT", mover.sizetext, "TOPRIGHT", 4, 0)
	mover.sensitivity:SetFontSize(12); mover.sensitivity.customFontSize = true
	mover.sensitivity:SetBackgroundColor(0.6,0,0,0.6)
	mover.sensitivity:SetText("+/- " .. tostring(_sensitivity))
	
	mover.x = 0
	mover.y = 0
	mover.dragging = false
	
	mover.Event.LeftDown = _leftDownM
	mover.Event.LeftUp = _leftUpM
	mover.Event.LeftUpoutside = _leftUpM
	mover.Event.MouseMove = _mouseMoveM
	mover.Event.RightDown = _rightDownM
	
	sizer:SetSize(18)
	sizer:SetTexture("myUI", "images/dragger.png")
	sizer:SetPoint("BOTTOMRIGHT", mover, "BOTTOMRIGHT", 0, 0)
	sizer.x = 0
	sizer.y = 0
	sizer.sizing = false
	
	sizer.Event.LeftDown = _leftDownS
	sizer.Event.LeftUp = _leftUpS
	sizer.Event.LeftUpoutside = _leftUpS
	sizer.Event.MouseMove = _mouseMoveS
	
	ubuff:SetSecureMode("normal")	
	ubuff:SetLayer(LAYER_BUFFS)
	ubuff:SetPoint("TOPLEFT", mover, "TOPLEFT")
	
	ubuff.lastupdated = 0
	
	ubuff.Recache =  _recacheSET
	
	return ubuff
end








local function _newBARSET()
	local barset
	
	if (#_recycledBARSETS > 0) then
		barset = tremove(_recycledBARSETS)
	else
		barset = _newSET()
		barset.bars = {}
		
		barset.Layout = _layoutBARSET
		barset.Initialize = _initBARSET
		barset.Dispose = _disposeBARSET
		barset.Update = _updateBARSET
		barset.Reset = _resetBARSET
		
		barset.mover.set = _barsets
		
	end
	
	
	return barset
end

local function _newBOXSET()
	local boxset
	if (#_recycledBOXSETS > 0) then
		boxset = tremove(_recycledBOXSETS)
	else
		boxset = _newSET()
		boxset.boxes = {}
		
		boxset.Layout = _layoutBOXSET
		boxset.Initialize = _initBOXSET
		boxset.Dispose = _disposeBOXSET	
		boxset.Update = _updateBOXSET
		boxset.Reset = _resetBOXSET
		
		boxset.mover.set = _boxsets
		
	end
	
	
	
	return boxset
end




local function _enableUnitBARSET(unit, enabled)
	_aprofile.barsets[unit].enabled = enabled
	
	if (enabled) then
		if (not _barsets[unit]) then
			if (sfind(unit, "group")) then
				for i=1, 20 do
					local gunit = _subGroup(unit, i)
					_barsets[gunit] = _newBARSET()
					_barsets[gunit]:Initialize(gunit, i)
					_barsets[gunit].mover:SetVisible(true)
				end				
			else
				_barsets[unit] = _newBARSET()
				_barsets[unit]:Initialize(unit)
				_barsets[unit].mover:SetVisible(true)
			end	
		end
	else
		local disposed = {}
		if (_barsets[unit]) then
			if (sfind(unit, "group")) then
				for i=1, 20 do
					local gunit = _subGroup(unit, i)
					_barsets[gunit]:Dispose()
					_barsets[gunit] = nil
				end
			else
				_barsets[unit]:Dispose()
				_barsets[unit] = nil
			end			
		end
	end
end
local function _enabledUnitBOXSET(unit, enabled)
	_aprofile.boxsets[unit].enabled = enabled
	
	if (enabled) then
		if (not _boxsets[unit]) then
			if (sfind(unit, "group")) then
				for i=1, 20 do
					local gunit = _subGroup(unit, i)
					_boxsets[gunit] = _newBOXSET()
					_boxsets[gunit]:Initialize(gunit, i)
					_boxsets[gunit].mover:SetVisible(true)
				end				
			else
				_boxsets[unit] = _newBOXSET()
				_boxsets[unit]:Initialize(unit)
				_boxsets[unit].mover:SetVisible(true)
			end	
		end
	else
		local disposed = {}
		if (_boxsets[unit]) then
			if (sfind(unit, "group")) then
				for i=1, 20 do
					local gunit = _subGroup(unit, i)
					_boxsets[gunit]:Dispose()
					_boxsets[gunit] = nil
				end
			else
				_boxsets[unit]:Dispose()
				_boxsets[unit] = nil
			end			
		end
	end
end
--

local function _animationUpdate(self, now, elsapsed)
	-- we do buff bars here b/c they need smooth animation and time display < 1 sec
	for unit, barset in pairs(_barsets) do
		barset:Update(now)
	end
end

local function _unitUpdate(self, now, elapsed)
	-- buff boxes are done here b/c they don't need smooth animation and it's good to tie their 
	-- responsiveness to that of unit frame updates

	-- we can have a max of 49 boxsets (p + p.t + p.t.t  + pt + pt.t + f + f.t + m + m.t + g(20) + g.pet(20)) 
	-- and this event first 10/s by default. To update each box at least twice a second, we'll need a total
	-- of  98 (49 * 2) updates. That means we should update at most 9.8 boxsets per update interval. We'll be nice
	-- and just call it 10.
	
	-- TODO : adjust update cap bases on number of unit boxes actually in use. This will save some cpu-cycles
	-- when we're not using all of them.
	
	local updated = 0
	for unit, boxset in pairs(_boxsets) do
		if ((now - boxset.lastupdated) > 0.49) then -- updated in the last half a second?
			boxset:Update(now)
			boxset.lastupdated = now
				
			updated = updated + 1
		end
		
		if (updated > 9) then
			break -- that's all for this update
		end	
	end
	
	
end

-- public interface methods

function buffsaddon:Initialize()
	
end

function buffsaddon:LoadProfile(account, character)
	_aprofile = account
	_cprofile = character

	local adefs = buffsaddon._getAccountDefaults()
	
	myUI.Utility.nilCopy(_aprofile, adefs)
	
	colors = _aprofile.colors
	
	for _, sets in _setPairs() do
		for _, set in pairs(sets) do
			set:Dispose()
		end
	end
	
	_boxsets = {}
	for unit, boxset in pairs(_aprofile.boxsets) do
		if (_aprofile.boxsets[unit].enabled) then
			if (sfind(unit, "group")) then
				local gunit
				for i=1,20 do
					gunit = _subGroup(unit, i)
					_boxsets[gunit] = _newBOXSET()
					_boxsets[gunit]:Initialize(gunit, i)
				end

			else
				_boxsets[unit] = _newBOXSET()
				_boxsets[unit]:Initialize(unit)
			end
		end
	end
	_barsets = {}
	
	for unit, barset in pairs(_aprofile.barsets) do
		if (_aprofile.barsets[unit].enabled) then
			if (sfind(unit, "group")) then
				local gunit
				for i=1,20 do
					gunit = _subGroup(unit, i)
					_barsets[gunit] = _newBARSET()
					_barsets[gunit]:Initialize(gunit, i)
				end

			else
				_barsets[unit] = _newBARSET()
				_barsets[unit]:Initialize(unit)
			end		
		end
	
	end
	
	-- Note: we're not hooking the Event.Buff.... events. We could use those to keep track of buffs on units between
	-- timer updates (for durations) but then we'd need to hook all the Event.Unit.Add/Remove/Available/Unavailable events
	-- as well b/c our unit specifier's partner unitid will change w/ those events too, which means we'd have to change the
	-- buffs tracked by each tracking frameset/barset in real time. After all of that, we'd still have to do work on a timer for
	-- the duration updates and we'd still have to filter and resort our list of buffs whenevery one is added.
	-- So basically, that adds a ton of complexity for no real payoff in efficiency and in fact will lose efficiency whenever buffs
	-- change in less than our normal timer interval. Therefore, we'll just 
	-- use a simple update timer to evaluate buffs. To avoid spikes in cpu usage (for boxes) we'll stagger those updates rather 
	-- than doing all of them once at a set interval to share the cpu-love.
		
	local uid = self.identifier .. "._animationUpdate"	
	if (not myUI.Events.AnimationUpdate:IsSubscribed(uid)) then
		myUI.Events.AnimationUpdate:Subscribe(uid, self, _animationUpdate)
	end
	local uid = self.identifier .. "._unitUpdate"
	if (not myUI.Events.UnitUpdate:IsSubscribed(uid)) then
		myUI.Events.UnitUpdate:Subscribe(uid, self, _unitUpdate)
	end
	
	if (_configFrame) then _configFrame:LoadProfile() end	
end

function buffsaddon:GetConfigurationFrame(parent)
	if (not _configFrame) then
		_configFrame = myUI:CreateFrame("Frame", parent)	
		
		_configFrame.buffcontrols = {boxsets = {}, barsets = {}}
		_configFrame.pickers = {boxsets={}, barsets={}, global={}}
		
		local function _loadLists(list)
			_configFrame.activeList = nil
			
			--[[
			_configFrame.ListsLabel
			_configFrame.ListItemsLabel
			_configFrame.ListsList
			_configFrame.ListsListItems
			_configFrame.NewListText
			_configFrame.NewListItemText
			_configFrame.DeleteListButton
			_configFrame.DeleteListItemButton
			]]--
			
			_configFrame.ListsLabel:SetText(L[list])
			_configFrame.ListItemsLabel:SetText("")
			
			_configFrame.ListsList:ClearItems()
			_configFrame.ListsListItems:ClearItems()
			
			_configFrame.DeleteListButton:SetText("")
			_configFrame.DeleteListButton:SetEnabled(false)
			
			_configFrame.DeleteListItemButton:SetText("")
			_configFrame.DeleteListItemButton:SetEnabled(false)
			
			_configFrame.NewListText:SetText("")
			_configFrame.NewListItemText:SetText("")
			
			for list, _ in pairs(_aprofile[list]) do
				_configFrame.ListsList:AddItem(list)			
			end
			
			
			_configFrame.activeList = list
		end
		
		_configFrame.LoadProfile = function(self)
			self._loading = true
			
			for unit, chk in pairs(_configFrame.BoxChecks) do
				chk:SetChecked(_aprofile.boxsets[unit].enabled)
			end
			for unit, chk in pairs(_configFrame.BarChecks) do
				chk:SetChecked(_aprofile.barsets[unit].enabled)
			end
			
			for color, cbox in pairs(_configFrame.ColorBoxes) do
				cbox:SetBackgroundColor(rgba(_aprofile.colors[color]))
			end
			
			local function getSetting(set, defset, subset, unit, setting)
				local s = _aprofile[set][unit]
				local _s = _aprofile[defset]
				
				if (subset) then
					s = s[subset]
					_s = _s[subset]
				end
				
				if (s and (s[setting]~=nil)) then 
					return s[setting]
				else
					return _s[setting]
				end
			end
			
			local ctrls = _configFrame.buffcontrols.boxsets
			for unit, ctrl in pairs(ctrls) do
				ctrl.padding:SetText(tostring(ifnil(_aprofile.boxsets[unit].padding, _aprofile.boxset.padding)))
				ctrl.margin:SetText(tostring(ifnil(_aprofile.boxsets[unit].margin, _aprofile.boxset.margin)))
				ctrl.cxybuff:SetText(tostring(ifnil(_aprofile.boxsets[unit].cxybuff, _aprofile.boxset.cxybuff)))
				ctrl.grow_up:SetChecked(ifnil(_aprofile.boxsets[unit].grow_up, _aprofile.boxset.grow_up))
				
				ctrl.caster_indicator.show:SetChecked(getSetting("boxsets", "boxset", "caster_indicator", unit, "show"))
				ctrl.caster_indicator.anchor:SetText(getSetting("boxsets", "boxset", "caster_indicator", unit, "anchor"))
				ctrl.caster_indicator.xoff:SetText(tostring(getSetting("boxsets", "boxset", "caster_indicator", unit, "xoff")))
				ctrl.caster_indicator.yoff:SetText(tostring(getSetting("boxsets", "boxset", "caster_indicator", unit, "yoff")))
				ctrl.caster_indicator.cx:SetText(tostring(getSetting("boxsets", "boxset", "caster_indicator", unit, "cx")))
				ctrl.caster_indicator.cy:SetText(tostring(getSetting("boxsets", "boxset", "caster_indicator", unit, "cy")))
				
				ctrl.stacks.show:SetChecked(getSetting("boxsets", "boxset", "stacks", unit, "show"))
				ctrl.stacks.anchor:SetText(getSetting("boxsets", "boxset", "stacks", unit, "anchor"))
				ctrl.stacks.xoff:SetText(tostring(getSetting("boxsets", "boxset", "stacks", unit, "xoff")))
				ctrl.stacks.yoff:SetText(tostring(getSetting("boxsets", "boxset", "stacks", unit, "yoff")))
				ctrl.stacks.fontsize:SetText(tostring(getSetting("boxsets", "boxset", "stacks", unit, "fontsize")))
				ctrl.stacks.fgcolor:SetBackgroundColor(rgba(getSetting("boxsets", "boxset", "stacks", unit, "fgcolor")))
				ctrl.stacks.bgcolor:SetBackgroundColor(rgba(getSetting("boxsets", "boxset", "stacks", unit, "bgcolor")))
				
				ctrl.remaining.show:SetChecked(getSetting("boxsets", "boxset", "remaining", unit, "show"))
				ctrl.remaining.anchor:SetText(getSetting("boxsets", "boxset", "remaining", unit, "anchor"))
				ctrl.remaining.xoff:SetText(tostring(getSetting("boxsets", "boxset", "remaining", unit, "xoff")))
				ctrl.remaining.yoff:SetText(tostring(getSetting("boxsets", "boxset", "remaining", unit, "yoff")))
				ctrl.remaining.fontsize:SetText(tostring(getSetting("boxsets", "boxset", "remaining", unit, "fontsize")))
				ctrl.remaining.fgcolor:SetBackgroundColor(rgba(getSetting("boxsets", "boxset", "remaining", unit, "fgcolor")))
				ctrl.remaining.bgcolor:SetBackgroundColor(rgba(getSetting("boxsets", "boxset", "remaining", unit, "bgcolor")))
				
				ctrl.show_own_buffs:SetChecked(getSetting("boxsets", "boxset", nil, unit, "show_own_buffs"))
				ctrl.show_units_buffs:SetChecked(getSetting("boxsets", "boxset", nil, unit, "show_own_buffs"))
				ctrl.show_others_buffs:SetChecked(getSetting("boxsets", "boxset", nil, unit, "show_others_buffs"))
				ctrl.buff_whitelists:SetText(ifnil(getSetting("boxsets", "boxset", nil, unit, "buff_whitelists"), ""))
				ctrl.buff_blacklists:SetText(ifnil(getSetting("boxsets", "boxset", nil, unit, "buff_blacklists"), ""))
				
				ctrl.show_own_debuffs:SetChecked(getSetting("boxsets", "boxset", nil, unit, "show_own_debuffs"))
				ctrl.show_units_debuffs:SetChecked(getSetting("boxsets", "boxset", nil, unit, "show_own_debuffs"))
				ctrl.show_others_debuffs:SetChecked(getSetting("boxsets", "boxset", nil, unit, "show_others_debuffs"))
				ctrl.debuff_whitelists:SetText(ifnil(getSetting("boxsets", "boxset", nil, unit, "debuff_whitelists"), ""))
				ctrl.debuff_blacklists:SetText(ifnil(getSetting("boxsets", "boxset", nil, unit, "debuff_blacklists"), ""))				
			end
			
			ctrls = _configFrame.buffcontrols.barsets
			for unit, ctrl in pairs(ctrls) do
				ctrl.cybar:SetText(tostring(getSetting("barsets", "barset", nil, unit, "cybar")))
				ctrl.padding:SetText(tostring(getSetting("barsets", "barset", nil, unit, "padding")))
				ctrl.grow_up:SetChecked(getSetting("barsets", "barset", nil, unit, "grow_up"))
				ctrl.max_duration:SetText(tostring(getSetting("barsets", "barset", nil, unit, "max_duration")))
				ctrl.caster_indicator.show:SetChecked(getSetting("barsets", "barset", "caster_indicator", unit, "show"))
				
				ctrl.show_own_buffs:SetChecked(getSetting("barsets", "barset", nil, unit, "show_own_buffs"))
				ctrl.show_units_buffs:SetChecked(getSetting("barsets", "barset", nil, unit, "show_units_buffs"))
				ctrl.show_others_buffs:SetChecked(getSetting("barsets", "barset", nil, unit, "show_others_buffs"))
				ctrl.buff_whitelists:SetText(ifnil(getSetting("barsets", "barset", nil, unit, "buff_whitelists"), ""))
				ctrl.buff_blacklists:SetText(ifnil(getSetting("barsets", "barset", nil, unit, "buff_blacklists"), ""))
				
				ctrl.show_own_debuffs:SetChecked(getSetting("barsets", "barset", nil, unit, "show_own_debuffs"))
				ctrl.show_units_debuffs:SetChecked(getSetting("barsets", "barset", nil, unit, "show_units_debuffs"))
				ctrl.show_others_debuffs:SetChecked(getSetting("barsets", "barset", nil, unit, "show_others_debuffs"))
				ctrl.debuff_whitelists:SetText(ifnil(getSetting("barsets", "barset", nil, unit, "debuff_whitelists"), ""))
				ctrl.debuff_blacklists:SetText(ifnil(getSetting("barsets", "barset", nil, unit, "debuff_blacklists"), ""))
				
				ctrl.name.fontsize:SetText(tostring(getSetting("barsets", "barset", "name", unit, "fontsize")))
				ctrl.name.fgcolor:SetBackgroundColor(rgba(getSetting("barsets", "barset", "name", unit, "fgcolor")))
				ctrl.remaining.fontsize:SetText(tostring(getSetting("barsets", "barset", "remaining", unit, "fontsize")))
				ctrl.remaining.fgcolor:SetBackgroundColor(rgba(getSetting("barsets", "barset", "remaining", unit, "fgcolor")))
				
				
			end
			
			self._loading = false
		end
		
		local margin = 16
		local tstrip = myUI:CreateFrame("myUI_Tabstrip", _configFrame)
		
		tstrip:SetPoint("TOPLEFT", _configFrame, "TOPLEFT")
		tstrip:SetPoint("BOTTOMRIGHT", _configFrame, "BOTTOMLEFT", 160, 0)
		
		local tpages = {}
		local clrDropdownBg = {0.1,0.1,0.1,1}
		
		local function updateUnit(set, unit, doexternal)
			if (set[unit]) then
				local now = Inspect.Time.Frame()
				
				if (string.find(unit, "group")) then
					local gunit
					for i=1,20 do
						gunit = _subGroup(unit, i)
						set[gunit]:Layout(doexternal)
						set[gunit]:Recache()						
					end
				else
					set[unit]:Layout(doexternal)
					set[unit]:Recache()
				end
			
			end
		end
		local function changeSetting(target, value, doexternal)
			local settings = _aprofile[target.set][target.unit]
			if (target.subset) then
				settings = settings[target.subset]
			end
			settings[target.setting] = value
			updateUnit(iif(target.set == "barsets", _barsets, _boxsets), target.unit, doexternal)
		end
		
		local function onCheck(chk)
			if (not _configFrame._loading) then
				changeSetting(chk, chk:GetChecked(), false)
			end
		end
		local function numberClick(ctrl)
			for k, picker in pairs(_configFrame.pickers[ctrl.set]) do
				if (k ~= "number") then picker:SetVisible(false) end
			end
			
			local picker = _configFrame.pickers[ctrl.set].number
			if (picker.target == ctrl) then
				picker:SetVisible(not picker:GetVisible())
			else
				picker.target = nil
				picker:SetPoint("TOPLEFT", ctrl, "BOTTOMLEFT", 0, 2)
				
				local min = ctrl.min
				local max = ctrl.max
				local incr = ifnil(ctrl.incr, 1)
				
				
				picker.slider.incr = incr
				picker.slider:SetRange(floor(min / incr), floor(max / incr))
				picker.slider:SetPosition(floor(tonumber(ctrl:GetText()) / incr))
				picker.target = ctrl
				picker:SetVisible(true)
			end
			
		end
		local function colorClick(ctrl)
			for k, picker in pairs(_configFrame.pickers[ctrl.set]) do
				if (k ~= "color") then picker:SetVisible(false) end
			end
			
			local cpicker = _configFrame.pickers[ctrl.set].color
			if (cpicker.target == ctrl) then
				cpicker:SetVisible(not cpicker:GetVisible())
			else
				cpicker.target = ctrl
				cpicker:SetColor(ctrl:GetBackgroundColor())
				cpicker:SetPoint("TOPLEFT", ctrl, "BOTTOMLEFT", 0, 2)
				cpicker:SetVisible(true)
			end			
		end
		local function anchorClick(ctrl)
			for k, picker in pairs(_configFrame.pickers[ctrl.set]) do
				if (k ~= "anchor") then picker:SetVisible(false) end
			end
			
			local cpicker = _configFrame.pickers[ctrl.set].anchor
			if (cpicker.target == ctrl) then
				cpicker:SetVisible(not cpicker:GetVisible())
			else
				cpicker.target = ctrl
				cpicker:SetAnchor(ctrl:GetText())
				cpicker:SetPoint("TOPLEFT", ctrl, "BOTTOMLEFT", 0, 2)
				cpicker:SetVisible(true)
			end
		end
		local function onKey(txt, key)
			local code = string.byte(key)
			if (code == 13) then
				changeSetting(txt, txt:GetText(), false)
			end
		end		
		local function onColorChanged(picker, r, g, b, a)
			if (not _configFrame._loading) then
				picker.target:SetBackgroundColor(r,g,b,a)
				changeSetting(picker.target, {r,g,b,a}, false)	

								
			end
		end
		local function onNumberChanged(slider)
			local target = slider:GetParent().target
			
			if (not _configFrame._loading) and (target) then
				target:SetText(tostring(floor(slider:GetPosition() * slider.incr)))
				changeSetting(target, floor(slider:GetPosition() * slider.incr), true)
			end
		end
		local function onAnchorChanged(picker, anchor)
			if (not _configFrame._loading) then
				picker.target:SetText(anchor)
				changeSetting(picker.target, anchor, false)
			end		
		end
		local function makeNumberControl(parent, set, unit, col)
			local txt = myUI:CreateFrame("Text", parent)
			txt:SetBackgroundColor(rgba(myuicolors.Selected))
			txt.set = set
			txt.unit = unit
			txt.subset = col.subsetting
			txt.setting = col.setting
			txt.min = col.min
			txt.max = col.max
			txt.incr = col.incr
			txt:SetText(" ")
			txt:SetWidth(40)
			txt.Event.LeftDown = numberClick
			
			return txt
		end
		local function makeAnchorControl(parent, set, unit, col)
			local txt = myUI:CreateFrame("Text", parent)
			txt:SetBackgroundColor(rgba(myuicolors.Selected))
			txt.set = set
			txt.unit = unit
			txt.subset = col.subsetting
			txt.setting = col.setting
			txt:SetText(" ")
			txt:SetWidth(110)
			txt.Event.LeftDown = anchorClick
			
			return txt
		end
		local function makeCheckControl(parent, set, unit, col)
			local chk = myUI:CreateFrame("RiftCheckbox", parent)
			chk.set = set
			chk.unit = unit
			chk.subset =col.subsetting
			chk.setting = col.setting
			chk.Event.CheckboxChange = onCheck
			
			return chk
		end
		local function makeColorControl(parent, set, unit, col)
			local cbox = myUI:CreateFrame("Frame", parent)
			cbox:SetSize(18)
			--cbox:SetBackgroundColor(0,0,0,0.4)
			cbox.set = set
			cbox.unit = unit
			cbox.subset = col.subsetting
			cbox.setting = col.setting
			cbox.Event.LeftDown = colorClick
			
			return cbox
		end
		local function makeTextboxControl(parent, set, unit, col)
			local txt = myUI:CreateFrame("RiftTextfield", parent)
			txt:SetWidth(ifnil(col.cx, 140))
			txt:SetBackgroundColor(rgba(myuicolors.Selected))
			txt.set = set
			txt.unit = unit
			txt.subset = col.subsetting
			txt.setting = col.setting
			txt.Event.KeyType = onKey
		
			return txt
		end
		local function makeHeader(parent, hdrText, cols)
			local hdr = myUI:CreateFrame("Text", parent)
			hdr:SetBackgroundColor(rgba(myuicolors.Hover))
			hdr:SetLayer(1)
			hdr:SetText(hdrText)
			
			for _, col in ipairs(cols) do
				local chdr = myUI:CreateFrame("Text", parent)
				chdr:SetLayer(2)
				chdr:SetText(col.label)
				chdr:SetPoint("TOPLEFT", hdr, "TOPLEFT", col.x, 0)
			end
			
			return hdr
		end
		
		
		local function makeTable(parent, anchor, yoff, hdrText, cols, set, unitsTable)
			local hdr = makeHeader(parent, hdrText, cols)
			
			hdr:SetPoint("TOPLEFT", anchor, "TOPLEFT", 0, yoff)
			hdr:SetPoint("TOPRIGHT", anchor, "TOPRIGHT", 0, yoff)
			
			local anchor = hdr
			local xoff = 4
			
			local ctrls = _configFrame.buffcontrols[set]
			
			for _, unit in ipairs(unitsTable) do
				local label = myUI:CreateFrame("Text", parent)
				label:SetText(L["u-" .. unit])
				label:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT", xoff, 2)
				
				if (not ctrls[unit]) then  ctrls[unit] = {} end
				
				for i, col in ipairs(cols) do
					local ctrl
					if (col.type == "NUMBER") then
						ctrl = makeNumberControl(parent, set, unit, col)
					elseif (col.type == "BOOL") then
						ctrl = makeCheckControl(parent, set, unit, col)
					elseif (col.type == "COLOR") then
						ctrl = makeColorControl(parent, set, unit, col)
					elseif (col.type == "ANCHOR") then
						ctrl = makeAnchorControl(parent, set, unit, col)
					elseif (col.type == "TEXT") then
						ctrl = makeTextboxControl(parent, set, unit, col)
					end
					
					if (ctrl) then
						ctrl:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT", col.x + xoff, 2)
						
						if (col.subsetting) then
							if (not ctrls[unit][col.subsetting]) then ctrls[unit][col.subsetting] = {} end
							ctrls[unit][col.subsetting][col.setting] = ctrl
						else
							ctrls[unit][col.setting] = ctrl
						end
					end
				end
				
				xoff = 0
				anchor = label
			end
			
			return anchor
		end
		
		
		if (true) then	-- globals tab
			local tpage = myUI:CreateFrame("myUI_Tabpage", _configFrame)
			tpage:SetPoint("TOPLEFT", tstrip, "TOPRIGHT")
			tpage:SetPoint("BOTTOMRIGHT", _configFrame, "BOTTOMRIGHT")
			tpages["global"] = tpage
			
			tstrip:AddTab(L["global"])
			
			local vp = myUI:CreateFrame("myUI_Viewport", tpage)
			vp:SetPoint("TOPLEFT", tpage, "TOPLEFT", margin, margin)
			vp:SetPoint("BOTTOMRIGHT", tpage, "BOTTOMRIGHT", -margin/2, -margin)
			
			local content = vp.Content
			content:SetHeight(480)
			--content:SetBackgroundColor(1,0,0,0.1)
		
			-- enabled boxes
			local yoff
			
			if (true) then	-- enabled boxes
				local hdr = myUI:CreateFrame("Text", content)
				hdr:SetPoint("TOPLEFT", content, "TOPLEFT")
				hdr:SetPoint("TOPRIGHT", content, "TOPRIGHT")
				hdr:SetBackgroundColor(rgba(myUI.Colors.Hover))
				hdr:SetText(L["enabled boxes"])
			
				local function onBoxEnabled(checkbox)
					if (not _configFrame._loading) then
						local unit = checkbox.unit
						
						_enabledUnitBOXSET(unit, checkbox:GetChecked())
					end
				end
				
				_configFrame.BoxChecks = {}
				
				for i, u in ipairs(_available_units.boxes) do
					local chk = myUI:CreateFrame("myUI_Checkbox", content)
					chk:SetText(L["u-" .. u])
					chk.unit = u
					chk.Event.CheckboxChange = onBoxEnabled
					
					_configFrame.BoxChecks[u] = chk
					

					if (i == 1) then
						chk:SetPoint("TOPLEFT", hdr, "BOTTOMLEFT", 4, 4)
					elseif (i == 5) or (i == 9) then
						chk:SetPoint("TOPLEFT", _configFrame.BoxChecks[_available_units.boxes[i-4]], "BOTTOMLEFT", 0, 8)
					else
						chk:SetPoint("LEFTCENTER", _configFrame.BoxChecks[_available_units.boxes[i-1]], "RIGHTCENTER", 140, 0)
					end
					
					yoff = chk:GetBottom()
				end
			end
			yoff = yoff - content:GetTop() + margin
			if (true) then	-- enabled bars
				local hdr = myUI:CreateFrame("Text", content)
				hdr:SetPoint("TOPLEFT", content, "TOPLEFT", 0, yoff)
				hdr:SetPoint("TOPRIGHT", content, "TOPRIGHT", 0, yoff)
				hdr:SetBackgroundColor(rgba(myUI.Colors.Hover))
				hdr:SetText(L["enabled bars"])
				
				local function onBarEnabled(checkbox)
					if (not _configFrame._loading) then
						local unit = checkbox.unit
						
						_enableUnitBARSET(unit, checkbox:GetChecked())
					end
				end
				
				_configFrame.BarChecks = {}
				
				for i, u in ipairs(_available_units.bars) do
					local chk = myUI:CreateFrame("myUI_Checkbox", content)
					chk:SetText(L["u-" .. u])
					chk.unit = u
					chk.Event.CheckboxChange = onBarEnabled
					
					_configFrame.BarChecks[u] = chk
					

					if (i == 1) then
						chk:SetPoint("TOPLEFT", hdr, "BOTTOMLEFT", 4, 4)
					elseif (i == 5) or (i == 9) then
						chk:SetPoint("TOPLEFT", _configFrame.BarChecks[_available_units.bars[i-4]], "BOTTOMLEFT", 0, 8)
					else
						chk:SetPoint("LEFTCENTER", _configFrame.BarChecks[_available_units.bars[i-1]], "RIGHTCENTER", 140, 0)
					end
					
					yoff = chk:GetBottom()
				end
			end
			yoff = yoff - content:GetTop() + margin
			if (true) then	-- colors
				local function onGlobalColorClick(ctrl)
					local picker = _configFrame.pickers.global.color
					
					if (picker.target == ctrl) then
						picker:SetVisible(not picker:GetVisible())
					else
						picker.target = ctrl
						picker:SetColor(ctrl:GetBackgroundColor())
						picker:SetPoint("TOPLEFT", ctrl, "BOTTOMLEFT", 0, 2)
						picker:SetVisible(true)						
					end
				end
				local function onGlobalColorChanged(picker, r, g, b, a)
					if (not _configFrame._loading) then
						picker.target:SetBackgroundColor(r,g,b,a)
						_aprofile.colors[picker.target.setting] = {r,g,b,a}

						for _, sets in _setPairs() do
							for _, set in pairs(sets) do
								set:Reset()
							end
						end
					end
				end
			
				-- color picker
				local picker = myUI:CreateFrame("myUI_ColorPicker", tpage)
				picker:SetLayer(10)
				picker:SetVisible(false)
				picker:SetBackgroundColor(rgba(clrDropdownBg))			
				picker.Event.ColorChanged = onGlobalColorChanged
				
				_configFrame.pickers.global.color = picker
			
				local hdr = myUI:CreateFrame("Text", content)
				hdr:SetPoint("TOPLEFT", content, "TOPLEFT", 0, yoff)
				hdr:SetPoint("TOPRIGHT", content, "TOPRIGHT", 0, yoff)
				hdr:SetBackgroundColor(rgba(myUI.Colors.Hover))
				hdr:SetText(L["colors"])
				
				local function makeColorBox(setting)
					local box = myUI:CreateFrame("Frame", content)
					box:SetBackgroundColor(0,0,0,0.4)
					box:SetSize(24)
					box.setting = setting
					box.Event.LeftDown = onGlobalColorClick
					
					local txt = myUI:CreateFrame("Text", box)
					txt:SetText(L[setting])
					txt:SetPoint("LEFTCENTER", box, "RIGHTCENTER", 4, 0)
					
					return box
				end
			
				local cboxes = {}
				
				for setting, _ in pairs(_aprofile.colors) do cboxes[setting] = makeColorBox(setting) end
				
				cboxes.cast_by_player:SetPoint("TOPLEFT", hdr, "BOTTOMLEFT", 4, 4)
				cboxes.cast_by_unit:SetPoint("TOPLEFT", cboxes.cast_by_player, "BOTTOMLEFT", 0, 4)
				cboxes.cast_by_other:SetPoint("TOPLEFT", cboxes.cast_by_unit, "BOTTOMLEFT", 0, 4)
				
				cboxes.buff:SetPoint("TOPLEFT", cboxes.cast_by_player, "TOPRIGHT", 220, 0)
				cboxes.buff_nocancel:SetPoint("TOPLEFT", cboxes.buff, "BOTTOMLEFT", 0, 4)
				cboxes.debuff:SetPoint("TOPLEFT", cboxes.buff_nocancel, "BOTTOMLEFT", 0, 4)
				
				cboxes.debuff_poison:SetPoint("TOPLEFT", cboxes.buff, "TOPRIGHT", 220, 0)
				cboxes.debuff_disease:SetPoint("TOPLEFT", cboxes.debuff_poison, "BOTTOMLEFT", 0, 4)
				cboxes.debuff_curse:SetPoint("TOPLEFT", cboxes.debuff_disease, "BOTTOMLEFT", 0, 4)
				
				yoff = cboxes.cast_by_other:GetBottom()
				
				_configFrame.ColorBoxes = cboxes
			end
			yoff = yoff - content:GetTop() + margin
			
			
			
			
		end
		
		
		if (true) then 	-- buffboxes tab
			local tpage = myUI:CreateFrame("myUI_Tabpage", _configFrame)
			tpage:SetPoint("TOPLEFT", tstrip, "TOPRIGHT")
			tpage:SetPoint("BOTTOMRIGHT", _configFrame, "BOTTOMRIGHT")
			tpage:SetVisible(false)
			tpages["units"] = tpage
			
			-- color picker
			local picker = myUI:CreateFrame("myUI_ColorPicker", tpage)
			picker:SetLayer(10)
			picker:SetVisible(false)
			picker:SetBackgroundColor(rgba(clrDropdownBg))			
			picker.set = "boxsets"
			picker.Event.ColorChanged = onColorChanged
			
			_configFrame.pickers.boxsets.color = picker
			
			-- anchor picker
			picker = myUI:CreateFrame("myUI_Anchorbox", tpage)
			picker:SetSize(160,100)
			picker:SetVisible(false)
			picker:SetBackgroundColor(rgba(clrDropdownBg))
			picker:SetLayer(10)
			picker.Event.AnchorChanged = onAnchorChanged
			
			_configFrame.pickers.boxsets.anchor = picker
			
			-- number picker
			picker = myUI:CreateFrame("myUI_BorderBox", tpage)
			picker:SetVisible(false)
			picker:SetBackgroundColor(rgba(clrDropdownBg))
			picker.slider = myUI:CreateFrame("RiftSlider", picker)
			picker.slider:SetPoint("LEFTCENTER", picker, "LEFTCENTER", 16, 8)
			picker.slider:SetPoint("RIGHTCENTER", picker, "RIGHTCENTER", -16, 8)
			picker.slider.Event.SliderChange = onNumberChanged
			picker:SetSize(180,32)
			picker:SetLayer(10)
			
			_configFrame.pickers.boxsets.number = picker
			
			-- viewport 
			local vp = myUI:CreateFrame("myUI_Viewport", tpage)
			vp:SetPoint("TOPLEFT", tpage, "TOPLEFT", margin, margin)
			vp:SetPoint("BOTTOMRIGHT", tpage, "BOTTOMRIGHT", -margin/2, -margin)
			
			local content = vp.Content
			content:SetHeight(1800)
			--content:SetBackgroundColor(1,0,0,0.1)
			
			tstrip:AddTab(L["boxes"])
			tpages["boxsets"] = tpage
			
			local anchor = content
			local yoff = 0
			
			local cols = {
				{label = L["cxybuff"], x = 160, setting="cxybuff", subsetting=nil, type="NUMBER", min = 12, max = 64},
				{label = L["padding"], x = 240, setting="padding", subsetting=nil, type="NUMBER", min = 0, max = 8},
				{label = L["margin"], x = 320, setting="margin", subsetting=nil, type="NUMBER", min  = 1, max = 8},
				{label = L["grow up"], x = 400, setting="grow_up", subsetting=nil, type="BOOL",}
				
			}
			
			anchor = makeTable(content, content, yoff, L["general"], cols, "boxsets", _available_units.boxes)
			yoff = anchor:GetBottom() - content:GetTop() + 16
			
			cols = {
				{label = L["by player"], x = 160, setting="show_own_buffs", subsetting=nil, type="BOOL"},
				{label = L["by unit"], x = 220, setting="show_units_buffs", subsetting=nil, type="BOOL"},
				{label = L["by other"], x = 280, setting="show_others_buffs", subsetting=nil, type="BOOL"},
				{label = L["whitelists"], x = 340, setting="buff_whitelists", subsetting=nil, type="TEXT"},
				{label = L["blacklists"], x = 500, setting="buff_blacklists", subsetting=nil, type="TEXT"}
				
			}
			
			anchor = makeTable(content, content, yoff, L["buffs by"], cols, "boxsets", _available_units.boxes)
			yoff = anchor:GetBottom() - content:GetTop() + 16
			
			cols = {
				{label = L["by player"], x = 160, setting="show_own_debuffs", subsetting=nil, type="BOOL"},
				{label = L["by unit"], x = 220, setting="show_units_debuffs", subsetting=nil, type="BOOL"},
				{label = L["by other"], x = 280, setting="show_others_debuffs", subsetting=nil, type="BOOL"},
				{label = L["whitelists"], x = 340, setting="debuff_whitelists", subsetting=nil, type="TEXT", cx = 140},
				{label = L["blacklists"], x = 500, setting="debuff_blacklists", subsetting=nil, type="TEXT", cx = 140}
				
			}
			
			anchor = makeTable(content, content, yoff, L["debuffs by"], cols, "boxsets", _available_units.boxes)
			yoff = anchor:GetBottom() - content:GetTop() + 16
			
			cols = {
				{label = L["show"], x = 160, setting="show", subsetting="caster_indicator", type="BOOL"},
				{label = L["anchor"], x = 210, setting="anchor", subsetting="caster_indicator", type="ANCHOR"},
				{label = L["xoff"], x = 340, setting="xoff", subsetting="caster_indicator", type="NUMBER", min=-20, max=20},
				{label = L["yoff"], x = 400, setting="yoff", subsetting="caster_indicator", type="NUMBER", min=-20, max=20},
				{label = L["cx"], x = 460, setting="cx", subsetting="caster_indicator", type="NUMBER", min=8, max=24},
				{label = L["cy"], x = 520, setting="cy", subsetting="caster_indicator", type="NUMBER", min=8, max=24},
			}
			anchor = makeTable(content, content, yoff, L["caster_indicator"], cols, "boxsets", _available_units.boxes)
			yoff = anchor:GetBottom() - content:GetTop() + 16
			
			cols = {
				{label = L["show"], x = 160, setting="show", subsetting="stacks", type="BOOL"},
				{label = L["anchor"], x = 210, setting="anchor", subsetting="stacks", type="ANCHOR"},
				{label = L["xoff"], x = 340, setting="xoff", subsetting="stacks", type="NUMBER", min=-20, max=20},
				{label = L["yoff"], x = 400, setting="yoff", subsetting="stacks", type="NUMBER", min=-20, max=20},
				{label = L["fontsize"], x = 460, setting="fontsize", subsetting="stacks", type="NUMBER", min=10, max=24},
				{label = L["fgcolor"], x = 520, setting="fgcolor", subsetting="stacks", type="COLOR"},
				{label = L["bgcolor"], x = 570, setting="bgcolor", subsetting="stacks", type="COLOR"},
			}
			anchor = makeTable(content, content, yoff, L["stacks"], cols, "boxsets", _available_units.boxes)
			yoff = anchor:GetBottom() - content:GetTop() + 15
			
			cols = {
				{label = L["show"], x = 160, setting="show", subsetting="remaining", type="BOOL"},
				{label = L["anchor"], x = 210, setting="anchor", subsetting="remaining", type="ANCHOR"},
				{label = L["xoff"], x = 330, setting="xoff", subsetting="remaining", type="NUMBER", min=-20, max=20},
				{label = L["yoff"], x = 390, setting="yoff", subsetting="remaining", type="NUMBER", min=-20, max=20},
				{label = L["fontsize"], x = 450, setting="fontsize", subsetting="remaining", type="NUMBER", min=10, max=24},
				{label = L["fgcolor"], x = 510, setting="fgcolor", subsetting="remaining", type="COLOR"},
				{label = L["bgcolor"], x = 560, setting="bgcolor", subsetting="remaining", type="COLOR"},
			}
			anchor = makeTable(content, content, yoff, L["remaining"], cols, "boxsets", _available_units.boxes)
			yoff = anchor:GetBottom() - content:GetTop() + 15
			
			
		end
		
		if (true) then 	-- buffbars tab
			local tpage = myUI:CreateFrame("myUI_Tabpage", _configFrame)
			tpage:SetPoint("TOPLEFT", tstrip, "TOPRIGHT")
			tpage:SetPoint("BOTTOMRIGHT", _configFrame, "BOTTOMRIGHT")
			tpage:SetVisible(false)
			tpages["units"] = tpage
			
			-- color picker
			local picker = myUI:CreateFrame("myUI_ColorPicker", tpage)
			picker:SetLayer(10)
			picker:SetVisible(false)
			picker:SetBackgroundColor(rgba(clrDropdownBg))			
			picker.set = "barsets"
			picker.Event.ColorChanged = onColorChanged
			
			_configFrame.pickers.barsets.color = picker
			
			-- number picker
			picker = myUI:CreateFrame("myUI_BorderBox", tpage)
			picker:SetVisible(false)
			picker:SetBackgroundColor(rgba(clrDropdownBg))
			picker.slider = myUI:CreateFrame("RiftSlider", picker)
			picker.slider:SetPoint("LEFTCENTER", picker, "LEFTCENTER", 16, 8)
			picker.slider:SetPoint("RIGHTCENTER", picker, "RIGHTCENTER", -16, 8)
			picker.slider.Event.SliderChange = onNumberChanged
			picker:SetSize(180,32)
			picker:SetLayer(10)
			
			_configFrame.pickers.barsets.number = picker
			
			-- viewport
			
			local vp = myUI:CreateFrame("myUI_Viewport", tpage)
			vp:SetPoint("TOPLEFT", tpage, "TOPLEFT", margin, margin)
			vp:SetPoint("BOTTOMRIGHT", tpage, "BOTTOMRIGHT", -margin/2, -margin)
			
			local content = vp.Content
			content:SetHeight(1300)
			--content:SetBackgroundColor(1,0,0,0.1)
			
			tstrip:AddTab(L["barsets"])
			tpages["barsets"] = tpage			
			
			local anchor = content
			local yoff = 0
			
			local cols = {
				{label = L["cybar"], x = 160, setting="cybar", subsetting=nil, type="NUMBER", min = 14, max = 64},
				{label = L["padding"], x = 240, setting="padding", subsetting=nil, type="NUMBER", min = 0, max = 8},
				{label = L["grow_up"], x = 320, setting="grow_up", subsetting=nil, type="BOOL"},
				{label = L["max_duration"], x = 390, setting="max_duration", subsetting=nil, type="NUMBER", min=10, max=60*30+1, incr=30},
				{label = L["caster"], x = 480, setting="show", subsetting="caster_indicator", type="BOOL"},
				
			}
			
			anchor = makeTable(content, content, yoff, L["general"], cols, "barsets", _available_units.bars)
			yoff = anchor:GetBottom() - content:GetTop() + 16
			
			cols = {
				{label = L["by player"], x = 160, setting="show_own_buffs", subsetting=nil, type="BOOL"},
				{label = L["by unit"], x = 220, setting="show_units_buffs", subsetting=nil, type="BOOL"},
				{label = L["by other"], x = 280, setting="show_others_buffs", subsetting=nil, type="BOOL"},
				{label = L["whitelists"], x = 340, setting="buff_whitelists", subsetting=nil, type="TEXT"},
				{label = L["blacklists"], x = 500, setting="buff_blacklists", subsetting=nil, type="TEXT"}
				
			}
			
			anchor = makeTable(content, content, yoff, L["buffs by"], cols, "barsets", _available_units.bars)
			yoff = anchor:GetBottom() - content:GetTop() + 16
			
			cols = {
				{label = L["by player"], x = 160, setting="show_own_debuffs", subsetting=nil, type="BOOL"},
				{label = L["by unit"], x = 220, setting="show_units_debuffs", subsetting=nil, type="BOOL"},
				{label = L["by other"], x = 280, setting="show_others_debuffs", subsetting=nil, type="BOOL"},
				{label = L["whitelists"], x = 340, setting="debuff_whitelists", subsetting=nil, type="TEXT", cx = 140},
				{label = L["blacklists"], x = 500, setting="debuff_blacklists", subsetting=nil, type="TEXT", cx = 140}
				
			}
			
			anchor = makeTable(content, content, yoff, L["debuffs by"], cols, "barsets", _available_units.bars)
			yoff = anchor:GetBottom() - content:GetTop() + 16
			
			cols = {
				{label = L["name"], x = 160, setting="fontsize", subsetting="name", type="NUMBER", min = 10, max = 24},
				{label = L["fgcolor"], x = 280, setting="fgcolor", subsetting="name", type="COLOR"},
				{label = L["remaining"], x = 330, setting="fontsize", subsetting="remaining", type="NUMBER", min=10, max = 24},
				{label = L["fgcolor"], x = 450, setting="fgcolor", subsetting="remaining", type="COLOR"}
			}
			anchor = makeTable(content, content, yoff, L["text"], cols, "barsets", _available_units.bars)
			yoff = anchor:GetBottom() - content:GetTop() + 16
		end
		
		if (true) then	-- white/black lists tabs
			local tpage = myUI:CreateFrame("myUI_Tabpage", _configFrame)
			tpage:SetPoint("TOPLEFT", tstrip, "TOPRIGHT")
			tpage:SetPoint("BOTTOMRIGHT", _configFrame, "BOTTOMRIGHT")
			tpage:SetVisible(false)
			
			tpages["lists"] = tpage
			tstrip:AddTab(L["whitelists"])
			tstrip:AddTab(L["blacklists"])
			
			local vp = myUI:CreateFrame("myUI_Viewport", tpage)
			vp:SetPoint("TOPLEFT", tpage, "TOPLEFT", margin, margin)
			vp:SetPoint("BOTTOMRIGHT", tpage, "BOTTOMRIGHT", -margin/2, -margin)
			
			local content = vp.Content
			content:SetHeight(720)
			--content:SetBackgroundColor(1,0,0,0.1)
		
			-- event handlers
			local function onAddNewList(txt)
				local wb = _configFrame.activeList
				local newlist = trim(txt:GetText())
				
				if (wb and (string.len(newlist) > 0)) then
					local listsetting = _aprofile[wb]
					
					if (not listsetting[newlist]) then
						listsetting[newlist] = {}
						txt:SetText("")
					
						_configFrame.ListsList:AddItem(newlist)
					end				
				end
			end			
			local function onAddNewListItem(txt)
				local wb = _configFrame.activeList
				local newitem = trim(txt:GetText())
				
				if (wb and (string.len(newitem) > 0)) then
					local selinx = _configFrame.ListsList:GetSelectedIndexes()
					if (selinx and (#selinx > 0)) then
						local list = _configFrame.ListsList:GetItem(selinx[1])
						
						local listsetting = _aprofile[wb][list]
						local exists
						for _, item in ipairs(listsetting) do
							if (item == newitem) then exists = true; break end
						end
						if (not exists) then
							tinsert(listsetting, newitem)
							txt:SetText("")
							print("added " .. newitem)
							
							_configFrame.ListsListItems:AddItem(newitem)
						end
					end
				end
			end
			local function deleteList(button)
				local wb = _configFrame.activeList
				if (wb) then
					local selinx = _configFrame.ListsList:GetSelectedIndexes()
					if (selinx and (#selinx > 0)) then
						local index = selinx[1]
						local list = _configFrame.ListsList:GetItem(index)
						if (_aprofile[wb][list]) then
							_aprofile[wb][list] = nil
							_configFrame.ListsList:RemoveItemAt(index)
					
						end
					end
				end
			end
			local function deleteListItem(button)
				local wb = _configFrame.activeList
				if (wb) then
					local selinx = _configFrame.ListsList:GetSelectedIndexes()
					if (selinx and (#selinx > 0)) then
						local list = _configFrame.ListsList:GetItem(selinx[1])
						if (list) then
							local selinx = _configFrame.ListsListItems:GetSelectedIndexes()
							if (selinx and (#selinx > 0)) then
								tremove(_aprofile[wb][list], selinx[1])
								_configFrame.ListsListItems:RemoveItemAt(selinx[1])

								
							end
						end
					end
				end
			end
			
			local function onSelectedListChanged(listbox, indexes)
				_configFrame.DeleteListItemButton:SetText("")
				_configFrame.DeleteListItemButton:SetEnabled(false)
					
				_configFrame.ListsListItems:ClearItems()
				
				if (indexes and (#indexes > 0)) then
					local list = listbox:GetItem(indexes[1])
					
					_configFrame.ListItemsLabel:SetText(list)
					_configFrame.DeleteListButton:SetText(list)
					_configFrame.DeleteListButton:SetEnabled(true)
					
					for _, listitem in ipairs(_aprofile[_configFrame.activeList][list]) do
						_configFrame.ListsListItems:AddItem(listitem)
					end
				else
					_configFrame.ListItemsLabel:SetText("")
					_configFrame.DeleteListButton:SetText("")
					_configFrame.DeleteListButton:SetEnabled(false)
				end
			end
			
			local function onSelectedListItemChanged(listbox, indexes)
				if (indexes and (#indexes > 0)) then
					_configFrame.DeleteListItemButton:SetText(_configFrame.ListsListItems:GetItem(indexes[1]))
					_configFrame.DeleteListItemButton:SetEnabled(true)						
				else
					_configFrame.DeleteListItemButton:SetText("")
					_configFrame.DeleteListItemButton:SetEnabled(false)
				end
			end
			
			-- controls
			local instr = myUI:CreateFrame("Text", content)
			instr:SetPoint("TOPLEFT", content, "TOPLEFT", margin, margin)
			instr:SetPoint("TOPRIGHT", content, "TOPRIGHT", -margin, margin)
			instr:SetWordwrap(true)
			instr:SetText(L["list instructions"])
		
			local hdr = myUI:CreateFrame("Text", content)
			hdr:SetText("lists ??")
			hdr:SetPoint("TOPLEFT", instr, "BOTTOMLEFT", 0, margin)
			
			_configFrame.ListsLabel = hdr
			
			local lbox = myUI:CreateFrame("myUI_Listbox", content)
			lbox:SetPoint("TOPLEFT", hdr, "BOTTOMLEFT", 0, 0)
			lbox:SetSize(300, 400)
			lbox.Event.SelectedIndexChanged = onSelectedListChanged
		
			_configFrame.ListsList = lbox
			
			lbox = myUI:CreateFrame("myUI_Listbox", content)
			lbox:SetPoint("TOPLEFT", _configFrame.ListsList, "TOPRIGHT", 140, 0)
			lbox:SetPoint("BOTTOMLEFT", _configFrame.ListsList, "BOTTOMRIGHT", 40, 0)
			lbox:SetWidth(300)
			lbox.Event.SelectedIndexChanged = onSelectedListItemChanged
			
			_configFrame.ListsListItems = lbox
			
			hdr = myUI:CreateFrame("Text", content)
			hdr:SetText("??")
			hdr:SetPoint("BOTTOMLEFT", _configFrame.ListsListItems, "TOPLEFT", 0, 0)
			
			_configFrame.ListItemsLabel = hdr
			
			local txt = myUI:CreateFrame("myUI_Textbox", content)
			txt:SetPoint("TOPRIGHT", _configFrame.ListsList, "BOTTOMRIGHT", -22, 8)
			txt:SetWidth(228)
			txt.Event.OnEnter = onAddNewList
			
			_configFrame.NewListText = txt
		
			hdr = myUI:CreateFrame("Text", content)
			hdr:SetText(L["add"])
			hdr:SetPoint("RIGHTCENTER", _configFrame.NewListText, "LEFTCENTER", -4, 2)
			
			local txt = myUI:CreateFrame("myUI_Textbox", content)
			txt:SetPoint("TOPRIGHT", _configFrame.ListsListItems, "BOTTOMRIGHT", -22, 8)
			txt:SetWidth(228)
			txt.Event.OnEnter = onAddNewListItem
			
			_configFrame.NewListItemText = txt
		
			hdr = myUI:CreateFrame("Text", content)
			hdr:SetText(L["add"])
			hdr:SetPoint("RIGHTCENTER", _configFrame.NewListItemText, "LEFTCENTER", -4, 2)
		
			local btn = myUI:CreateFrame("myUI_Button", content)
			btn:SetEnabled(false)
			btn:SetPoint("TOPLEFT", _configFrame.NewListText, "BOTTOMLEFT", 0, 10)
			btn:SetPoint("TOPRIGHT", _configFrame.NewListText, "BOTTOMRIGHT", 0, 10)
			btn.Event.LeftPress = deleteList
			
			hdr = myUI:CreateFrame("Text", content)
			hdr:SetText(L["delete"])
			hdr:SetPoint("RIGHTCENTER", btn, "LEFTCENTER", -4, 0)
			
			_configFrame.DeleteListButton = btn
			
			local btn = myUI:CreateFrame("myUI_Button", content)
			btn:SetEnabled(false)
			btn:SetPoint("TOPLEFT", _configFrame.NewListItemText, "BOTTOMLEFT", 0, 10)
			btn:SetPoint("TOPRIGHT", _configFrame.NewListItemText, "BOTTOMRIGHT", 0, 10)
			btn.Event.LeftPress = deleteListItem
			
			hdr = myUI:CreateFrame("Text", content)
			hdr:SetText(L["delete"])
			hdr:SetPoint("RIGHTCENTER", btn, "LEFTCENTER", -4, 0)
			
			_configFrame.DeleteListItemButton = btn
		end
		
		
		tstrip.Event.TabSelected = function(tstrip, value)
			for _, tp in pairs(_configFrame.TabPages) do
				tp:SetVisible(false)
			end
		
			if (value == L["global"]) then
				_configFrame.TabPages["global"]:SetVisible(true)
			elseif (value == L["whitelists"]) then
				_loadLists("whitelists")
				_configFrame.TabPages["lists"]:SetVisible(true)
			elseif (value == L["blacklists"]) then
				_loadLists("blacklists")
				_configFrame.TabPages["lists"]:SetVisible(true)
			elseif (value == L["boxes"]) then
				_configFrame.TabPages["boxsets"]:SetVisible(true)
			elseif (value == L["barsets"]) then			
				_configFrame.TabPages["barsets"]:SetVisible(true)
			end
			
		end
		
		
		
		
		
		
		_configFrame.TabPages = tpages
		_configFrame.TabStrip = tstrip
		
		_configFrame:LoadProfile()
	end
	return _configFrame
end
function buffsaddon:ShowMovers()	
	for _, sets in _setPairs() do
		for _, set in pairs(sets) do
			if (set.enabled) then
				set.mover:SetVisible(true)
				set.mover.dragging = false
				set.mover.sizer.sizing = false
			end
		end
	end
end
function buffsaddon:HideMovers()
	for _, sets in _setPairs() do
		for _, set in pairs(sets) do
			set.mover:SetVisible(false)
			set.mover.dragging = false
			set.mover.sizer.sizing = false
		end
	end

end

--
