local tracker = ...
table.insert(Event.Addon.Load.End, {function(addon) if (addon == tracker.identifier) then myUI:RegisterModule(tracker) end end, tracker.identifier, "_startup"})

local L = tracker.data.language

local ifnil = myUI.Utility.ifnil
local isnil = myUI.Utility.isnil
local rgba = myUI.Utility.rgba
local iif = myUI.Utility.iif
local pprint = myUI.Utility.pprint
local round = myUI.Utility.round
local minmax = myUI.Utility.minmax
local lpad = myUI.Utility.lpad
local trim = myUI.Utility.trim
local isempty = myUI.Utility.isempty
local ifempty = myUI.Utility.ifempty
local nilstr = myUI.Utility.nilstr
local notnil = myUI.Utility.notnil

local floor = math.floor
local abs = math.abs
local tinsert = table.insert
local tremove = table.remove

local mover, sizer, cframe, powerbar, powertext
local _boxes
local isplayer, isrogue, iswarrior
local myuicolors = myUI.Colors
-- addon --

local _aprofile, _cprofile -- account and character profiles, respectively
local _configFrame = nil

local _updatePoints = function() end	-- empty function (just to prevent errors). will be set to either the warrior or rogue verson once class has been determined
local _updateBar = function() end -- ditto
local _layout = function() end	-- ditto
local _settings	-- which sub-table to use for settings based on class
local _sensitivity = 10	-- moving/resizing

--[[
local function _getAccountDefaults()
	local id = tracker.identifier
	
	local defaults = {	
		Version = tracker.toc.Version,
		rogue = {
			x = 400,
			y = 400,
			cx = 400,
			cy = 200,
			margin = 2,
			padding = 2,
			bgcolor = {0,0,0,0},
			oocAlpha = 0.2,			
			usetextures = false,
			showbar = true,
			showtext = true,
			barcolor = {1,0,1,1},
			fgcolor = {1,1,1,1},
			fontsize = 14,
			barratio = 0.5,
			textures = {
				{src = id, file = "images/combo0.png"},
				{src = id, file = "images/combo1.png"},
				{src = id, file = "images/combo1.png"},
				{src = id, file = "images/combo1.png"},
				{src = id, file = "images/combo2.png"},
				{src = id, file = "images/combo3.png"},
			},
			colors = {
				{0,0,0,0.1},
				{1,1,0,1},
				{1,1,0,1},
				{1,1,0,1},
				{1,0.6,0.2,1},
				{1,0,0,1}
			},
		},
		warrior = {
			x = 400,
			y = 400,
			cx = 400, 
			cy = 200,
			margin = 2,
			padding = 2,
			bgcolor = {0,0,0,0},
			oocAlpha = 1,
			usetextures = false,
			showbar = true,
			showtext = true,
			barcolor = {1,0,0,1},
			fgcolor = {1,1,1,1},
			fontsize = 14,
			barratio = 0.5,
			textures = {
				{src = id, file = "images/combo0.png"},
				{src = id, file = "images/combo1.png"},
				{src = id, file = "images/combo2.png"},
				{src = id, file = "images/combo3.png"},
			},
			colors = {
				{0,0,0,0.1},
				{1,1,0,1},
				{1,0.6,0.2,1},
				{1,0,0,1}
			},
		},
	}
	return defaults
end
]]--

-- tracker mover --
local function _leftDownM(mover)
	if (mover.allowdrag) then
		local m = Inspect.Mouse()
		if (m) then
			mover.x = m.x
			mover.y = m.y
			mover.dragging = true
		end
	end
end

local function _leftUpM(mover)
	mover.dragging = false
end
local function _mouseMoveM(mover, x, y)	
	if (mover.allowdrag) and (mover.dragging) then
		local deltax = floor(x - mover.x)
		local deltay = floor(y - mover.y)
		
		-- apply sensitivity setting
		deltax = floor(deltax / _sensitivity) * _sensitivity
		deltay = floor(deltay / _sensitivity) * _sensitivity
		
		if (math.abs(deltax) > 0) or (math.abs(deltay) > 0) then						
			local moverx = mover:GetLeft() 
			local movery = mover:GetTop() 
			
			local xnow = moverx + deltax
			local ynow = movery + deltay
			
			mover:SetPoint("TOPLEFT", UIParent, "TOPLEFT", xnow, ynow)
			
			mover.coordtext:SetText(tostring(floor(xnow))
				.. ", " .. tostring(floor(ynow)))
			
			_settings.x = xnow
			_settings.y = ynow
			
			mover.x = mover.x + deltax
			mover.y = mover.y + deltay
		end
	end
end
local function _rightDownM(mover)
	if (mover:GetVisible()) then
		local s = _sensitivity - 1
		if (s < 1) then s = 10 end		
		_sensitivity = s
		
		mover.sensitivity:SetText("+/- " .. tostring(_sensitivity))		
	end
end







-- castbar sizer --
local function _leftDownS(sizer)
	if (sizer.allowsize) then
		local m = Inspect.Mouse()
		if (m) then
			sizer.x = m.x
			sizer.y = m.y
			sizer.sizing = true
		end
	end
end
local function _leftUpS(sizer)
	sizer.sizing = false
end

local function _mouseMoveS(sizer, x, y)
	if (sizer.allowsize) and (sizer.sizing) then
		local deltax = floor(x - sizer.x)
		local deltay = floor(y - sizer.y)
		
		-- apply sensitivity setting
		deltax = floor(deltax / _sensitivity) * _sensitivity
		deltay = floor(deltay / _sensitivity) * _sensitivity
		
		if (abs(deltax) > 0) or (abs(deltay) > 0) then
			local cxmover = mover:GetWidth()
			local cymover = mover:GetHeight()
			
			local cxnow = cxmover + deltax
			local cynow = cymover + deltay
			
			cxnow = floor(minmax(cxnow, 100, UIParent:GetWidth()))
			cynow = floor(minmax(cynow, 12, UIParent:GetHeight()))
			
			if (cxnow ~= cxmover) or (cynow ~= cymover) then
				mover.sizetext:SetText(tostring(cxnow) .. " x " .. tostring(cynow))
				
				_settings.cx = cxnow
				_settings.cy = cynow
				
				_layout(true)
				
				sizer.x = sizer.x + deltax
				sizer.y = sizer.y	+ deltay
			end
		
		end
	end
end


-- castbar --
local LAYER_TRACKER = myUI.Layers.UnitFrame + 2
local LAYER_MOVER = LAYER_TRACKER + 20

local function _updateEnergy()
	if (powerbar and powerbar:GetVisible()) then
		local detail = Inspect.Unit.Detail("player")
		if (detail) then
			local r = (detail.energy / detail.energyMax) * powerbar.cx
			powerbar:SetWidth(minmax(r, 0, powerbar.cx))
			
			if (_settings.showtext) then
				powertext:SetText(tostring(detail.energy))
				powertext:SetVisible(r > 30)
			end
		else
			powerbar:SetWidth(0)
			powertext:SetVisible(false)
		end
	
	end
end
local function _updatePower()
	if (powerbar and powerbar:GetVisible()) then
		local detail = Inspect.Unit.Detail("player")
		if (detail) then
			local r = (detail.power / 100) * powerbar.cx
			powerbar:SetWidth(minmax(r, 0, powerbar.cx))
			
			if (_settings.showtext) then
				powertext:SetText(tostring(detail.power))
				powertext:SetVisible(r > 30)
			end
		else
			powerbar:SetWidth(0)
			powertext:SetVisible(false)
		end	
	end
end

local function _onEnergy(units)
	if (units[Inspect.Unit.Lookup("player")]) then
		_updateEnergy()
	end
end
local function _onPower(units)
	if (units[Inspect.Unit.Lookup("player")]) then
		_updatePower()
	end
end

local function _layoutCommon()
	mover:SetPoint("TOPLEFT", UIParent, "TOPLEFT", _settings.x, _settings.y)
	mover:SetSize(_settings.cx, _settings.cy)
	
	local nboxes = iif(isrogue, 5, 3)
	if (not _boxes) then		
		_boxes = {}
		for i=1, nboxes do
			tinsert(_boxes, myUI:CreateFrame("Texture", cframe))
		end
	end
	
	if (_settings) then
		cframe:SetBackgroundColor(rgba(_settings.bgcolor))	
			
		local m = _settings.margin
		local p = _settings.padding
		local cybox = cframe:GetHeight() - (2 * m)
		local cxbox = (cframe:GetWidth() - (2 * m) - ((nboxes - 1) * p)) / nboxes
		
		local ratio = 1 - iif(_settings.showbar, _settings.barratio, 0)
		local xtra = iif(_settings.showbar, m, 0)
		
		local anchor = cframe
		local anchorTo = "TOPLEFT"
		local offx = m
		local offy = m
		for i=1, nboxes do
			_boxes[i]:ClearAll()
			_boxes[i]:SetSize(cxbox, cybox * ratio - xtra)
			_boxes[i]:SetPoint("TOPLEFT", anchor, anchorTo, offx, offy)
			
			if (_settings.usetextures) then
				_boxes[i]:SetBackgroundColor(0,0,0,0)
				_boxes[i]:SetTexture(_settings.textures[1].src, _settings.textures[1].file)
			else
				_boxes[i]:SetTexture(tracker.identifier, "images/blank.png")
				_boxes[i]:SetBackgroundColor(rgba(_settings.colors[1]))
			end
			
			anchor = _boxes[i]
			anchorTo = "TOPRIGHT"
			offx = p			
			offy = 0
		end
		
		if (_settings.showbar) then
			if (not powerbar) then
				powerbar = myUI:CreateFrame("Frame", cframe)				
				powertext = myUI:CreateFrame("Text", powerbar)				
			end			
			
			powerbar:SetPoint("BOTTOMLEFT", cframe, "BOTTOMLEFT", m, -m)
			powertext:SetPoint("RIGHTCENTER", powerbar, "RIGHTCENTER", -4, 0)

			powerbar:SetBackgroundColor(rgba(_settings.barcolor))
			powerbar:SetHeight(cybox * (1 - ratio) - xtra)	
			powerbar.cx = cframe:GetWidth() - (2 * m)		-- pre-compute
			
			if (_settings.showtext) then
				powertext:SetFontColor(rgba(_settings.fgcolor))
				powertext:SetFontSize(_settings.fontsize)
				powertext.customFontSize = true
			else
				powertext:SetVisible(false)
			end
		else
			if (powerbar) then
				powerbar:ClearAll()
				powerbar:SetVisible(false)
			end
		end
		
		cframe:SetAlpha(iif(Inspect.System.Secure(), 1, _settings.oocAlpha))	
	end	
end

local function _layoutRogue()
	_layoutCommon()	
	
	-- this is kinda sloppy, but the myUI hook framework will fail silently if we try to hook the same thing twice, or
	-- if we try to unhook an event that isn't hooked, so no real problem just doing it hear blindly other than the slight
	-- performance hit, but it's miniscule so don't worry about it.
	
	if (_settings.showbar) then
		myUI:HookEvent(Event.Unit.Detail.Energy, _onEnergy, tracker, "_onEnergy")
		myUI:HookEvent(Event.Unit.Detail.EnergyMax, _onEnergy, tracker, "_onEnergy")
		_updateEnergy()
	else
		myUI:UnhookEvent(Event.Unit.Detail.Energy, tracker, "_onEnergy")
		myUI:UnhookEvent(Event.Unit.Detail.EnergyMax, tracker, "_onEnergy")
	end
	
end
local function _layoutWarrior()
	_layoutCommon()	
	
	-- ^ ditto the rogue remarks regarding energy hooking
	if (_settings.showbar) then
		myUI:HookEvent(Event.Unit.Detail.Power, _onPower, tracker, "_onPower")
		_updatePower()
	else
		myUI:UnhookEvent(Event.Unit.Detail.Power, tracker, "_onPower")
	end
	
	
end

local function _updateRoguePoints()
	local detail = Inspect.Unit.Detail("player")
	if (detail) then
		local update = 1	-- by default, sets indicators to zero combo points
		local targetID
		
		-- do we have a possible combo target
		local tdetail = Inspect.Unit.Detail("player.target")
		if (tdetail) then
			if (tdetail.relation == "friendly") then
				-- friendly units can't be combo targets, but check target-of-target in case that targetting mode is in use
				tdetail = Inspect.Unit.Lookup("player.target.target")
				if (tdetail and (tdetail.relation ~= "friendly")) then
					targetID = Inspect.Unit.Lookup("player.target.target")
				end
			else
				targetID = Inspect.Unit.Lookup("player.target")
			end
		end
		
		if (targetID) then
			-- there's a possible display target available, so
			if (detail.comboUnit == targetID) then
				update = ifnil(detail.combo, 0) + 1
			end
		end
		
		if (detail.combat) then cframe:SetAlpha(1) end	-- b/c sometimes the combo event fires before System.Secure.Enter
		
		-- set combo points to correct value
		if (_settings.usetextures) then
			local t =   _settings.textures[update]
			local t0 = _settings.textures[1]
			
			for i, box in ipairs(_boxes) do
				if (i < update) then
					box:SetTexture(t.src, t.file)
				else
					box:SetTexture(t0.src, t0.file)
				end
			end
		else
			local c = _settings.colors[update]
			local c0 = _settings.colors[1]
			
			for i, box in ipairs(_boxes) do
				box:SetBackgroundColor(rgba(iif( i < update, c, c0)))
			end
		end		
	end
end
local function _updateWarriorPoints()
	-- ah, warriors ... so much more straight-forward than rogue combo points
	local detail = Inspect.Unit.Detail("player")
	if (detail) then
		if (detail.combat) then cframe:SetAlpha(1) end
		local update = ifnil(detail.combo, 0) + 1
		
		if (_settings.usetextures) then
			local t = _settings.textures[1 + ifnil(detail.combo, 0)]
			local t0 = _settings.textures[1]
			
			for i, box in ipairs(_boxes) do
				if (i < update) then
					box:SetTexture(t.src, t.file)
				else
					box:SetTexture(t0.src, t0.file)
				end
			end
		else
			local c = _settings.colors[1 + ifnil(detail.combo, 0)]
			local c0 = _settings.colors[1]
			
			for i, box in ipairs(_boxes) do
				box:SetBackgroundColor(rgba(iif(i < update, c, c0)))
			end
		end
		
		
	end
end


local function _createBaseFrames()
	cframe = myUI:CreateFrame("Frame")
	mover = myUI:CreateFrame("Frame")
	sizer = myUI:CreateFrame("Texture", mover)	
		
	mover:SetLayer(LAYER_MOVER)
	mover:SetBackgroundColor(rgba(myuicolors.Mover))
	mover:SetVisible(false)

	mover.unittext = myUI:CreateFrame("Text", mover)
	mover.unittext:SetPoint("BOTTOMLEFT", mover, "TOPLEFT", 0, -4)
	mover.unittext:SetFontSize(12); mover.unittext.customFontSize = true	
	mover.unittext:SetBackgroundColor(0.6,0,0,0.6)
		
	mover.coordtext = myUI:CreateFrame("Text", mover)
	mover.coordtext:SetPoint("TOPLEFT", mover.unittext, "TOPRIGHT", 4, 0)
	mover.coordtext:SetFontSize(12); mover.coordtext.customFontSize = true
	mover.coordtext:SetBackgroundColor(0.6,0,0,0.6)
		
	mover.sizetext = myUI:CreateFrame("Text", mover)
	mover.sizetext:SetPoint("TOPLEFT", mover.coordtext, "TOPRIGHT", 4, 0)
	mover.sizetext:SetFontSize(12); mover.sizetext.customFontSize = true
	mover.sizetext:SetBackgroundColor(0.6,0,0,0.6)
		
	mover.sensitivity = myUI:CreateFrame("Text", mover)
	mover.sensitivity:SetPoint("TOPLEFT", mover.sizetext, "TOPRIGHT", 4, 0)
	mover.sensitivity:SetFontSize(12); mover.sensitivity.customFontSize = true
	mover.sensitivity:SetBackgroundColor(0.6,0,0,0.6)
	mover.sensitivity:SetText("+/- " .. tostring(_sensitivity))
		
	mover.x = 0
	mover.y = 0
	mover.dragging = false
		
	mover.Event.LeftDown = _leftDownM
	mover.Event.LeftUp = _leftUpM
	mover.Event.LeftUpoutside = _leftUpM
	mover.Event.MouseMove = _mouseMoveM
	mover.Event.RightDown = _rightDownM
		
	sizer:SetSize(18)
	sizer:SetTexture("myUI", "images/dragger.png")
	sizer:SetPoint("BOTTOMRIGHT", mover, "BOTTOMRIGHT", 0, 0)
	sizer.x = 0
	sizer.y = 0
	sizer.sizing = false
		
	sizer.Event.LeftDown = _leftDownS
	sizer.Event.LeftUp = _leftUpS
	sizer.Event.LeftUpoutside = _leftUpS
	sizer.Event.MouseMove = _mouseMoveS
		
	cframe:SetAllPoints(mover)
	cframe:SetLayer(LAYER_TRACKER)
		
	mover.allowdrag = true
	sizer.allowsize = true
		
	mover.unittext:SetVisible(true)
	mover.coordtext:SetVisible(true)
	mover.sizetext:SetVisible(true)

	mover.unittext:SetText(L["combotracker"])
end


local function _onCombatEnter()
	if (_settings) then
		cframe:SetAlpha(1)
	end
end
local function _onCombatLeave()
	cframe:SetAlpha(_settings.oocAlpha)
end
local function _onCombo(units)
	_updatePoints()
end

local function _onComboUnit(units)
	_updatePoints()
end

local function _onUnitsChanged(units)
	_updatePoints()
end



local function _onUnit(units)
	local detail = Inspect.Unit.Detail("player")
	if (detail) then
		isplayer = true
		isrogue =  (detail.calling == "rogue")
		iswarrior = (detail.calling == "warrior")
	
		local id = tracker.identifier
		
		if (isrogue or iswarrior) then	-- enabled the appropriate event handlers		
			if (isrogue) then
				if (_aprofile) then _settings = _aprofile.rogue end
				_updatePoints = _updateRoguePoints
				_updateBar = _updateEnergy
				_layout = _layoutRogue
								
				-- needed b/c combo points are specific to a target and we need to know when the target changes.
				tinsert(Event.Unit.Available, {_onUnitsChanged, id, "_onUnitsChanged"})
				tinsert(Event.Unit.Unavailable, {_onUnitsChanged, id, "_onUnitsChanged"})
				tinsert(Event.Unit.Remove, {_onUnitsChanged, id, "_onUnitsChanged"})
				tinsert(Event.Unit.Add, {_onUnitsChanged, id, "_onUnitsChanged"})
			else
				if (_aprofile) then _settings = _aprofile.warrior end
				_updatePoints = _updateWarriorPoints
				_updateBar = _updatePower
				_layout = _layoutWarrior
				
			end
		
			tinsert(Event.Unit.Detail.Combo, {_onCombo, id, "_onCombo"})
			tinsert(Event.Unit.Detail.ComboUnit, {_onComboUnit, id, "_onComboUnit"})

			tinsert(Event.System.Secure.Enter, {_onCombatEnter, id, "_onCombatEnter"})
			tinsert(Event.System.Secure.Leave, {_onCombatLeave, id, "_onCombatLeave"})	
		end

		-- we don't need this check again so unhook this function
		myUI:UnhookEvent(Event.Unit.Available, tracker, "_onUnit")
		
		_layout()
		_updatePoints()
		_updateBar()
	end	
end

-- public interface methods

function tracker:Initialize()
	_createBaseFrames()

	myUI:HookEvent(Event.Unit.Available, _onUnit, tracker, "_onUnit")
end

function tracker:LoadProfile(account, character)
	_aprofile = account
	_cprofile = character

	local adefs = tracker._getAccountDefaults()
	
	myUI.Utility.nilCopy(_aprofile, adefs)	
	
	
	if (isrogue) then	-- we do this both here and in unit available in case the load order changes
		_settings = _aprofile.rogue
		_update = _updateRogue
		_layout = _layoutRogue
	elseif (iswarrior) then
		_settings = _aprofile.warrior
		_update = _updateWarrior
		_layout = _layoutWarrior
	end
	
	if (_configFrame) then _configFrame:LoadProfile() end	
end

function tracker:GetConfigurationFrame(parent)
	if (isrogue or iswarrior) and (not _configFrame) then
		_configFrame = myUI:CreateFrame("Frame", parent)	
		
		local vp = myUI:CreateFrame("myUI_Viewport", _configFrame)
		vp:SetPoint("TOPLEFT", _configFrame, "TOPLEFT", 0, 0)
		vp:SetPoint("BOTTOMRIGHT", _configFrame, "BOTTOMRIGHT", 0, 0)
		
		local content = vp.Content
		content:SetHeight(600)
		--content:SetBackgroundColor(0.6,0,0,0.2)
		
		-- drop downs
		local function colorPickerChanged(cpicker, r, g, b, a)
			cpicker.target:SetBackgroundColor(r,g,b,a)
			
			_settings[cpicker.target.setting] ={r,g,b,a}
			_layout()		
		end
		
		local cpicker = myUI:CreateFrame("myUI_ColorPicker", content)
		cpicker:SetVisible(false)
		cpicker:SetBackgroundColor(0.1, 0.1, 0.1, 1)
		cpicker.Event.ColorChanged = colorPickerChanged		
		
		_configFrame.ColorPicker = cpicker
		
		-- event handlers
		local function onBorderSize(slider)
			if (not _configFrame._loading) then
				_settings.margin = slider:GetPosition()
				_layout()
				_updateBar()
			end
		end
		
		local function onPadding(slider)
			if (not _configFrame._loading) then
				_settings.padding = slider:GetPosition()
				_layout()
				_updateBar()
			end
		end
		local function onAlpha(slider)
			if (not _configFrame._loading) then
				_settings.oocAlpha = round(slider:GetPosition() / 100, 3)
				_layout()
			end
		end
		local function onBarRatio(slider)
			if (not _configFrame._loading) then
				_settings.barratio = round(slider:GetPosition()/100, 2)
				_layout()
			end
		end
		local function onFontSize(slider)
			if (not _configFrame._loading) then
				_settings.fontsize = slider:GetPosition()
				_layout()
			end
		end
		local function onStyle(dropdown, selIndex)
			if (not _configFrame._loading) then
				if (selIndex == 1) then
					_settings.usetextures = true; _layout()
				elseif (selIndex == 2) then
					_settings.usetextures = false; _layout()
				end				
			end
		end
		local function onShowBar(chk)
			if (not _configFrame._loading) then
				_settings.showbar = chk:GetChecked()
				_layout()
				_updateBar()
			end
		end
		local function onShowText(chk)
			if (not _configFrame._loading) then
				_settings.showtext = chk:GetChecked()
				_layout()
				_updateBar()
			end
		end
		local function colorClick(box)
			local cpicker = _configFrame.ColorPicker
			
			if (cpicker.target == box) then
				cpicker:SetVisible(not cpicker:GetVisible())
			else
				cpicker.target = box
				cpicker:SetColor(box:GetBackgroundColor())
				cpicker:SetPoint("TOPLEFT", box, "BOTTOMLEFT", 0, 2)
				cpicker:SetVisible(true)
			end
		end
		
		-- normal controls
		local ctrls = {}
		
		local hdr = myUI:CreateFrame("Text", content)
		hdr:SetBackgroundColor(rgba(myUI.Colors.Hover))
		hdr:SetText(L["layout"])
		hdr:SetPoint("TOPLEFT", content, "TOPLEFT"); hdr:SetPoint("TOPRIGHT", content, "TOPRIGHT")
		
		local label = myUI:CreateFrame("Text", content)
		label:SetText(L["margin"])
		label:SetPoint("TOPLEFT", hdr, "BOTTOMLEFT", 16, 16)
		
		local slider = myUI:CreateFrame("RiftSlider", content)
		slider:SetPoint("LEFTCENTER", label, "LEFTCENTER", 120, 8)
		slider:SetWidth(100)
		slider:SetRange(0,10)
		slider.Event.SliderChange = onBorderSize
		ctrls.margin = slider
		
		local label2 = myUI:CreateFrame("Text", content)
		label2:SetText(L["padding"])
		label2:SetPoint("TOPLEFT", label, "BOTTOMLEFT", 0, 8)
		
		slider = myUI:CreateFrame("RiftSlider", content)
		slider:SetPoint("LEFTCENTER", label2, "LEFTCENTER", 120, 8)
		slider:SetWidth(100)
		slider:SetRange(0, 20)
		slider.Event.SliderChange = onPadding
		
		ctrls.padding = slider
		
		label = myUI:CreateFrame("Text", content)
		label:SetText(L["barratio"])
		label:SetPoint("TOPLEFT", label2, "BOTTOMLEFT", 0, 8)
		
		slider = myUI:CreateFrame("RiftSlider", content)
		slider:SetPoint("LEFTCENTER", label, "LEFTCENTER", 120, 8)
		slider:SetWidth(100)
		slider:SetRange(0, 100)
		slider.Event.SliderChange = onBarRatio
		
		ctrls.barratio = slider
		
		label2 = myUI:CreateFrame("Text", content)
		label2:SetText(L["fontsize"])
		label2:SetPoint("TOPLEFT", label, "BOTTOMLEFT", 0, 8)
		
		slider = myUI:CreateFrame("RiftSlider", content)
		slider:SetPoint("LEFTCENTER", label2, "LEFTCENTER", 120, 8)
		slider:SetWidth(100)
		slider:SetRange(6, 32)
		slider.Event.SliderChange = onFontSize
		
		ctrls.fontsize = slider
		
		label = myUI:CreateFrame("Text", content)
		label:SetText(L["oocAlpha"])
		label:SetPoint("TOPLEFT", label2, "BOTTOMLEFT", 0, 8)
		
		slider = myUI:CreateFrame("RiftSlider", content)
		slider:SetWidth(100)
		slider:SetRange(0, 100)
		slider:SetPoint("LEFTCENTER", label, "LEFTCENTER", 120, 8)
		slider.Event.SliderChange = onAlpha
		
		ctrls.oocAlpha = slider
		
		label = myUI:CreateFrame("Text", content)
		label:SetText(L["usetextures"] .. ":")
		label:SetPoint("LEFTCENTER", ctrls.margin, "RIGHTCENTER", 40, -8)
		
		local combo = myUI:CreateFrame("myUI_DropdownList", content)
		combo:AddItem({value = true, tostring = function(self) return L["usetextures-true"] end})
		combo:AddItem({value = false, tostring = function(self) return L["usetextures-false"] end})
		combo:SetPoint("LEFTCENTER", label, "RIGHTCENTER", 8, 0)
		combo.List:SetBackgroundColor(0,0,0,0.8)
		combo:SetLayer(10)
		combo.List:SetHeight(60)
		combo:SetWidth(160)
		combo.Event.SelectedIndexChanged = onStyle
		ctrls.usetextures = combo
		
		local chk = myUI:CreateFrame("myUI_Checkbox", content)
		chk:SetText(L["showbar"])
		chk:SetPoint("TOPLEFT", combo, "BOTTOMLEFT", 0, 8)
		chk.Event.CheckboxChange = onShowBar
		ctrls.showbar = chk
		
		chk = myUI:CreateFrame("myUI_Checkbox", content)
		chk:SetText(L["showtext"])
		chk:SetPoint("TOPLEFT", ctrls.showbar, "BOTTOMLEFT", 0, 8)
		chk.Event.CheckboxChange = onShowText
		
		ctrls.showtext = chk
		
		hdr = myUI:CreateFrame("Text", content)
		hdr:SetBackgroundColor(rgba(myUI.Colors.Hover))
		hdr:SetText(L["colors"])
		hdr:SetPoint("TOPLEFT", content, "TOPLEFT", 0, 200); 
		hdr:SetPoint("TOPRIGHT", content, "TOPRIGHT", 0, 200)
		
		-- TODO : allow editing of textures and colors
		label = myUI:CreateFrame("Text", content)
		label:SetText(L["bgcolor"])
		label:SetPoint("TOPLEFT", hdr, "BOTTOMLEFT", 16, 16)
		
		local box = myUI:CreateFrame("Frame", content)
		box:SetSize(22)
		box.setting = "bgcolor"
		box:SetPoint("LEFTCENTER", label, "LEFTCENTER", 160, 0)
		box.Event.LeftDown = colorClick
		
		ctrls.bgcolor = box
		
		label2 = myUI:CreateFrame("Text", content)
		label2:SetText(L["barcolor"])		
		label2:SetPoint("TOPLEFT", label, "BOTTOMLEFT", 0, 8)
		
		box = myUI:CreateFrame("Frame", content)
		box:SetSize(22)
		box.setting = "barcolor"
		box:SetPoint("LEFTCENTER", label2, "LEFTCENTER", 160, 0)
		box.Event.LeftDown = colorClick
		
		ctrls.barcolor = box
		
		label = myUI:CreateFrame("Text", content)
		label:SetText(L["fgcolor"])
		label:SetPoint("TOPLEFT", label2, "BOTTOMLEFT", 0, 8)
		
		box = myUI:CreateFrame("Frame", content)
		box:SetSize(22)
		box.setting = "fgcolor"
		box:SetPoint("LEFTCENTER", label, "LEFTCENTER", 160, 0)
		box.Event.LeftDown = colorClick
		ctrls.fgcolor = box
		
		_configFrame.controls = ctrls
		
		_configFrame.LoadProfile = function(self)
			self._loading = true
			
			local ctrls = _configFrame.controls
			
			ctrls.margin:SetPosition(_settings.margin)
			ctrls.padding:SetPosition(_settings.padding)			
			ctrls.oocAlpha:SetPosition(_settings.oocAlpha * 100)
			ctrls.usetextures:SetItemSelected(iif(_settings.usetextures, 1, 2), true)
			ctrls.barratio:SetPosition(_settings.barratio * 100)
			ctrls.fontsize:SetPosition(_settings.fontsize)
			ctrls.showbar:SetChecked(_settings.showbar)
			ctrls.showtext:SetChecked(_settings.showtext)
			ctrls.bgcolor:SetBackgroundColor(rgba(_settings.bgcolor))
			ctrls.barcolor:SetBackgroundColor(rgba(_settings.barcolor))
			ctrls.fgcolor:SetBackgroundColor(rgba(_settings.fgcolor))
			
			self._loading = false
		end
		
		_configFrame:LoadProfile()
	end
	return _configFrame
end
function tracker:ShowMovers()
	if (isrogue or iswarrior) then
		mover.sizetext:SetText(tostring(_settings.cx) .. " x " .. tostring(_settings.cy))
		mover.coordtext:SetText(tostring(_settings.x) .. ", " .. tostring(_settings.y))
		mover:SetVisible(true)
		mover.dragging = false
		sizer.sizing = false	
	end
end
function tracker:HideMovers()
	if (isrogue or iswarrior) then
		mover:SetVisible(false)
		mover.dragging = false
		sizer.sizing = false	
	end
end

--
