local myUI = ...
local L = myUI.data.language

local _configWindow
local pprint = myUI.Utility.pprint
local ifnil = myUI.Utility.ifnil
local rgba = myUI.Utility.rgba
local trim = myUI.Utility.trim
local slen = string.len

function myUI:ShowConfiguration()
	if (_configWindow == nil) then
		_configWindow = myUI:CreateFrame("RiftWindow")
		_configWindow:SetVisible(false)
		_configWindow:SetLayer(myUI.Layers.Configuration)
		_configWindow:SetSize(1200, 800)		
		
		if (myUI_Account.ConfigY < 0)  then 
			myUI_Account.ConfigY = 0 
		else
			local cyScreen = UIParent:GetHeight()
			if (myUI_Account.ConfigY > (cyScreen - 80)) then myUI_Account.ConfigY = cyScreen - 80 end
			
		end
		
		_configWindow:SetPoint("TOPLEFT", UIParent, "TOPLEFT", myUI_Account.ConfigX, myUI_Account.ConfigY)		
		_configWindow:SetTitle(myUI.identifier .. " " .. L["Configuration"] .. " - v" .. myUI.toc.Version)
		
		local content = _configWindow:GetContent()
		local border = _configWindow:GetBorder()
		local margin = 16
		
		local trimLeft, trimTop, trimRight, trimBottom = _configWindow:GetTrimDimensions()
		
		if (true) then -- close button
			local btnClose = myUI:CreateFrame("RiftButton", _configWindow)
			btnClose:SetSkin("close")
			btnClose:SetPoint("TOPRIGHT", border, "TOPRIGHT", -8, 16)
			
			btnClose.Event.LeftPress = function(btnClose)
				btnClose:GetParent():SetVisible(false)
				
				for _, m in myUI:Modules() do
					if (m.HideMovers) then m:HideMovers() end
				end
			end
			
			
		end
		
		
		if (true) then -- titlebar dragger
			local dragger = myUI:CreateFrame("Frame", _configWindow)
			dragger:SetPoint("TOPCENTER", border, "TOPCENTER", 0, trimTop * 0.25)
			dragger:SetHeight(trimTop * 0.5)
			dragger:SetWidth(border:GetWidth() - 84)
			
			dragger.Event.LeftDown = function(dragger)
				local m = Inspect.Mouse()
				if (m) then
					dragger.dragging = true
					dragger.x = m.x
					dragger.y = m.y
				end
			end
			dragger.Event.LeftUp = function(dragger)
				dragger.dragging = false
			end
			dragger.Event.LeftUpoutside = function(dragger)
				dragger.dragging = false
			end
			dragger.Event.MouseMove = function(dragger, x, y)
				if (dragger.dragging) then
					local deltax = math.floor(x - dragger.x)
					local deltay = math.floor(y - dragger.y)
					
					if (math.abs(deltax) > 2) or (math.abs(deltay) > 2) then
						local p = dragger:GetParent()
						local xnow = p:GetLeft() + deltax
						local ynow = p:GetTop() + deltay
						
						local cxScreen = UIParent:GetWidth()
						local cyScreen = UIParent:GetHeight()
						local cxMe = p:GetWidth()
						local cyMe = p:GetHeight()
						
						if (ynow < 0) then ynow = 0 end
						if (xnow < (80 - cxMe)) then xnow = 80 - cxMe end
						if (xnow > (cxScreen + cxMe - 80)) then xnow = cxScreen + cxMe - 80 end
						if (ynow > (cyScreen - 80)) then ynow = cyScreen - 80 end
						
						p:SetPoint("TOPLEFT", UIParent, "TOPLEFT", xnow, ynow)
						
						myUI_Account.ConfigX = xnow
						myUI_Account.ConfigY = ynow
					
						dragger.x = x
						dragger.y = y
					end
				end
			end
		end
		
	
		
	
		
		if (true) then -- module listing			
			local lbox = myUI:CreateFrame("myUI_Listbox", _configWindow)
			lbox:SetItemSpacing(8)
			lbox:SetPoint("TOPLEFT",	content, "TOPLEFT", margin, margin)
			lbox:SetPoint("BOTTOMRIGHT", content, "BOTTOMLEFT", 240 + margin, -margin)
			
			lbox.Event.SelectedIndexChanged = function(lbox, selIndexes)
				if (selIndexes) and (#selIndexes > 0) then
					if (_configWindow.CurrentConfigFrame) then
						_configWindow.CurrentConfigFrame:SetVisible(false)
						_configWindow.CurrentConfigFrame = nil
					end
				
					local m = lbox:GetItem(selIndexes[1])
					
					local f = nil
					
					if (m.GetConfigurationFrame) then
						f = m:GetConfigurationFrame(_configWindow.ConfigArea)
					end
				
					if (f) then
						_configWindow.NoConfigMessage:SetVisible(false)
						
						f:ClearAll()
						f:SetAllPoints()
						f:SetVisible(true)
						
						_configWindow.CurrentConfigFrame = f
					else
						_configWindow.NoConfigMessage:SetVisible(true)
					end
				end
			end
						
			lbox:AddItem(myUI)
			
			for _, m in myUI:Modules() do
				lbox:AddItem(m)
			end
			
			_configWindow.ModuleList = lbox			
		end
	
		if (true) then -- module config area
			local configArea = myUI:CreateFrame("Frame", _configWindow)
			configArea:SetPoint("TOPLEFT", _configWindow.ModuleList, "TOPRIGHT", margin, 0)
			configArea:SetPoint("BOTTOMRIGHT", content, "BOTTOMRIGHT", -margin, -margin)
			
			_configWindow.ConfigArea = configArea
		end

		if (true) then -- no config message
			local msg = myUI:CreateFrame("Text", _configWindow.ConfigArea)
			msg:SetText(L["No Config"])
			msg:SetPoint("TOPLEFT", _configWindow.ConfigArea, "TOPLEFT", 4, 4)
			
			_configWindow.NoConfigMessage = msg
		end
		
		
		
		_configWindow.ModuleList:SetItemSelected(1, true)
	
		
	end
	
	for _, m in myUI:Modules() do 
		if (m.ShowMovers) then m:ShowMovers() end
	end

	_configWindow:SetVisible(true)
end


local _configFrame = nil

function myUI:GetConfigurationFrame(parent)
	if (not _configFrame) then
		_configFrame = myUI:CreateFrame("Frame", parent)
		
		local margin = 16
		local tstrip = myUI:CreateFrame("myUI_Tabstrip", _configFrame)
		
		tstrip:SetPoint("TOPLEFT", _configFrame, "TOPLEFT")
		tstrip:SetPoint("BOTTOMRIGHT", _configFrame, "BOTTOMLEFT", 160, 0)
		
		local tpages = {}
		
		if (true) then -- general tab page
			local tpage = myUI:CreateFrame("myUI_Tabpage", _configFrame)
			table.insert(tpages, tpage)
			
			tpage:SetPoint("TOPLEFT", tstrip, "TOPRIGHT")
			tpage:SetPoint("BOTTOMRIGHT", _configFrame, "BOTTOMRIGHT")
			
			tpage.tostring = function(tpage) return L["General"] end
			tstrip:AddTab(tpage)
			
			local msg = myUI:CreateFrame("Text", tpage)
			msg:SetWordwrap(true)
			msg:SetText(L["GeneralMsg"])			
			msg:SetPoint("TOPLEFT", tpage, "TOPLEFT", margin, margin)
			msg:SetPoint("TOPRIGHT", tpage, "TOPRIGHT", -margin, margin)
			
			local usage = myUI:CreateFrame("Text", tpage)
			usage:SetWordwrap(true)
			usage:SetPoint("TOPLEFT", msg, "BOTTOMLEFT", 32, 32)
			usage:SetPoint("TOPRIGHT", msg, "BOTTOMRIGHT", -32, 32)
			
			local tusage = myUI:GetSlashCommandUsage()
			local sz = ""
			for _, line in ipairs(tusage) do
				sz = sz .. line .. "\n"
			end
			usage:SetText(sz)
			
		end
		
		if (true) then -- profiles tab page
			local tpage = myUI:CreateFrame("myUI_Tabpage", _configFrame)
			table.insert(tpages, tpage)
			
			tpage:SetPoint("TOPLEFT", tstrip, "TOPRIGHT")
			tpage:SetPoint("BOTTOMRIGHT", _configFrame, "BOTTOMRIGHT")		

			tpage.tostring = function(tpage) return L["Profiles"] end
			tpage:SetVisible(false)
			tstrip:AddTab(tpage)
			
			local msg = myUI:CreateFrame("Text", tpage)
			msg:SetWordwrap(true)
			msg:SetText(L["ProfilesMsg"])
			msg:SetPoint("TOPLEFT", tpage, "TOPLEFT", margin, margin)
			msg:SetPoint("TOPRIGHT", tpage, "TOPRIGHT", -margin, margin)
			
			local plist = myUI:CreateFrame("myUI_Listbox", tpage)
			plist:SetPoint("TOPLEFT", msg, "BOTTOMLEFT", 32, 48)
			plist:SetSize(300, 340)
			
			for _, n in ipairs(myUI:GetProfiles()) do
				plist:AddItem(n)
			end
			
			plist.Event.SelectedIndexChanged = function(plist, selindexes)
				local tpage = plist:GetParent()
				
				if (selindexes) and (#selindexes > 0) then
					local profile = plist:GetItem(selindexes[1])
					local cprofile = myUI:GetCurrentProfile()
					
					if (profile == "default") or (profile == cprofile) then
						tpage.btnDelete:SetEnabled(false)
						tpage.btnDelete:SetText(L["delete profile"])
					else
						tpage.btnDelete:SetEnabled(true)
						tpage.btnDelete:SetText(L["delete profile"] .. " \"" .. profile .. "\"")
					end
					if (profile == cprofile) then
						tpage.btnActivate:SetEnabled(false)
						tpage.btnActivate:SetText(L["activate profile"])
					else
						tpage.btnActivate:SetEnabled(true)
						tpage.btnActivate:SetText(L["activate profile"] .. " \"" .. profile .. "\"")
					end
				else
					tpage.btnActivate:SetEnabled(false)
					tpage.btnActivate:SetText(L["activate profile"])
					
					tpage.btnDelete:SetEnabled(false)
					tpage.btnDelete:SetText(L["delete profile"])
				end
			end
			
			tpage.plist = plist
			
			local lbl = myUI:CreateFrame("Text", tpage)
			lbl:SetText(L["current profiles"])
			lbl:SetPoint("BOTTOMLEFT", plist, "TOPLEFT")
			
			local txtadd = myUI:CreateFrame("myUI_Textbox", tpage)
			txtadd:SetPoint("TOPRIGHT", plist, "BOTTOMRIGHT", -22, 8)			
			txtadd:SetWidth(160)
			txtadd:SetText("")
			
			txtadd.Event.OnEnter = function(txtadd)
				local newprofile = trim(txtadd:GetText())
				
				for _, profile in ipairs(myUI:GetProfiles()) do
					if (newprofile == profile) then
						newprofile = nil	-- already exists
						break
					end
				end
				
				if (newprofile) then
					if (myUI:CopyCurrentProfile(newprofile)) then				
						txtadd:GetParent().plist:AddItem(newprofile)
						txtadd:SetText("")
					end
				end
			end
			
			lbl = myUI:CreateFrame("Text", tpage)
			lbl:SetText(L["add profile"])
			lbl:SetPoint("RIGHTCENTER", txtadd, "LEFTCENTER", -8, 0)
			
			local btn = myUI:CreateFrame("myUI_Button", tpage)
			btn:SetEnabled(false)
			btn:SetText(L["activate profile"])
			btn:SetWidth(180)
			btn:SetPoint("LEFTCENTER", plist, "RIGHTCENTER", margin, -32)
			
			btn.Event.LeftPress = function(btnActivate)
				local plist = btnActivate:GetParent().plist
				local selinx = plist:GetSelectedIndexes()
				if (selinx and (#selinx > 0)) then
					local profile = plist:GetItem(selinx[1])
					if (profile) then
						if (myUI:SetCurrentProfile(profile, true)) then
							myUI:ReloadUI()
						end
					end
				end
			end
			
			tpage.btnActivate = btn
			
			btn = myUI:CreateFrame("myUI_Button", tpage)
			btn:SetEnabled(false)
			btn:SetText(L["delete profile"])
			btn:SetWidth(180)
			btn:SetPoint("TOPLEFT", tpage.btnActivate, "BOTTOMLEFT", 0, 16)
			
			btn.Event.LeftPress = function(btnDelete)
				local plist = btnDelete:GetParent().plist
				local selinx = plist:GetSelectedIndexes()
				if (selinx and (#selinx > 0)) then
					local profile = plist:GetItem(selinx[1])
					if (profile) then
						if (myUI:DeleteProfile(profile)) then
							plist:RemoveItemAt(selinx[1])
						end
					end
				end
				
			end
			
			tpage.btnDelete = btn
			
		end
		
		--[[
		if (true) then -- lists tab page
			local tpage = myUI:CreateFrame("myUI_Tabpage", _configFrame)
			table.insert(tpages, tpage)
			
			tpage:SetPoint("TOPLEFT", tstrip, "TOPRIGHT")
			tpage:SetPoint("BOTTOMRIGHT", _configFrame, "BOTTOMRIGHT")
			
			tpage.tostring = function(tpage) return L["Lists"] end
			tpage:SetVisible(false)
			tstrip:AddTab(tpage)		
		end
		]]--
		if (true) then -- fonts tab page
			local tpage = myUI:CreateFrame("myUI_Tabpage", _configFrame)
			table.insert(tpages, tpage)
			
			tpage:SetPoint("TOPLEFT", tstrip, "TOPRIGHT")
			tpage:SetPoint("BOTTOMRIGHT", _configFrame, "BOTTOMRIGHT")
			
			tpage.tostring = function(tpage) return L["Fonts"] end
			tpage:SetVisible(false)
			tstrip:AddTab(tpage)
			
			local msg = myUI:CreateFrame("Text", tpage)
			msg:SetWordwrap(true)
			msg:SetPoint("TOPLEFT", tpage, "TOPLEFT", margin, margin)
			msg:SetPoint("TOPRIGHT", tpage, "TOPRIGHT", -margin, margin)
			msg:SetText(L["FontsMsg"])
			
			local flist = myUI:CreateFrame("myUI_Listbox", tpage)
			flist:SetSize(300, 360)
			flist:SetPoint("TOPLEFT", msg, "BOTTOMLEFT", 32, 60)
			
			
			
			for name, _ in pairs(myUI_Account.Global.Fonts) do
				flist:AddItem(name)
			end
			
			flist.Event.SelectedIndexChanged = function (flist, selindexes)
				if (selindexes and (#selindexes > 0)) then
					local fname =  flist:GetItem(selindexes[1])
					local font = myUI_Account.Global.Fonts[fname]
					for _, p in ipairs(flist.previews) do
						p:SetFont(font.src, font.file)
					end

					flist:GetParent().btnRemoveFont:SetEnabled(fname ~= "Rift Default")
				end
			end
			
			local label = myUI:CreateFrame("Text", tpage)
			label:SetPoint("BOTTOMLEFT", flist, "TOPLEFT")
			label:SetText(L["Installed Fonts"])
			
			local anchor = flist
			local anchorTo = "TOPRIGHT"
			local offx = 24
			local offy = 0
			
			flist.previews = {}
			for i=10,28, 4 do
				local p = myUI:CreateFrame("Text", tpage)
				p:SetWordwrap(true)
				p:SetWidth(240)
				p:SetText(L["FontPreview"])
				p:SetFontSize(i)
				p.customFontSize = true
				p.customFont = true
				p:SetBackgroundColor(rgba(myUI.Colors.Selected))
				p:SetPoint("TOPLEFT", anchor, anchorTo, offx, offy)
				
				anchor = p
				anchorTo = "BOTTOMLEFT"
				offx = 0
				offy = 8
				
				table.insert(flist.previews, p)
			end
			
			
			local btn = myUI:CreateFrame("myUI_Button", tpage)
			btn:SetText(L["RemoveFont"])
			btn:SetEnabled(false)
			btn:SetPoint("TOPRIGHT", flist, "BOTTOMRIGHT", -22, 12)
			btn.Event.LeftPress = function(btn)
				local p = btn:GetParent()
				local flist = p.lstFonts
				local selindexes = flist:GetSelectedIndexes()
				
				if (selindexes and (#selindexes > 0)) then
					local fname = flist:GetItem(selindexes[1])
					if (fname ~= "Rift Default") then
						myUI_Account.Global.Fonts[fname] = nil
						flist:RemoveItemAt(selindexes[1])
						
						flist:SetItemSelected(1)
					end
				end
			end
			
			tpage.btnRemoveFont = btn
			
			btn = myUI:CreateFrame("myUI_Button", tpage)
			btn:SetText(L["FontUpdate"])
			--btn:SetPoint("TOPRIGHT", flist, "BOTTOMRIGHT", -22, 12)
			btn:SetPoint("RIGHTCENTER", tpage.btnRemoveFont, "LEFTCENTER", -margin, 0)
			btn.Event.LeftPress = function(btn)
				local selindexes = flist:GetSelectedIndexes()
				if (selindexes and (#selindexes > 0)) then
					local font = flist:GetItem(selindexes[1])
					myUI_Account.Profiles[myUI:GetCurrentProfile()].GlobalFont = font
					myUI:ReloadUI()
				end			
			end
			
			
			local anchorMsg
			
			label = myUI:CreateFrame("Text", tpage)
			label:SetText(L["FontName"])
			label:SetPoint("BOTTOMLEFT", tpage, "BOTTOMLEFT", margin, -margin)
			
			local txt = myUI:CreateFrame("myUI_Textbox", tpage)
			txt:SetPoint("LEFTCENTER", label, "RIGHTCENTER", 8, 0)
			txt:SetWidth(140)
			txt:SetText("")
			
			tpage.txtCFontName = txt
			
			label = myUI:CreateFrame("Text", tpage)
			label:SetText(L["FontFile"])
			label:SetPoint("LEFTCENTER", txt, "RIGHTCENTER", margin, 0)
			
			txt = myUI:CreateFrame("myUI_Textbox", tpage)
			txt:SetPoint("LEFTCENTER", label, "RIGHTCENTER", 8, 0)
			txt:SetWidth(200)
			txt:SetText("")
			
			tpage.txtCFontFile = txt
			
			btn = myUI:CreateFrame("myUI_Button", tpage)
			btn:SetText(L["AddFont"])
			btn:SetPoint("LEFTCENTER", txt, "RIGHTCENTER", margin, 0)
			btn:SetEnabled(false)
			
			tpage.btnAddCFont = btn
			
			local label = myUI:CreateFrame("Text", tpage)
			label:SetWordwrap(true)
			label:SetText(L["CustomFontMsg"])
			label:SetPoint("BOTTOMLEFT", tpage, "BOTTOMLEFT", margin, -60)
			label:SetPoint("BOTTOMRIGHT", tpage, "BOTTOMRIGHT", -margin, -60)
			
			flist:SetItemSelected(1)
			tpage.lstFonts = flist
			
			local function _checkCFont(tbox)
				local p = tbox:GetParent()
				
				local txtCFontFile = p.txtCFontFile
				local txtCFontName = p.txtCFontName
				
				p.btnAddCFont:SetEnabled((slen(trim(txtCFontFile:GetText())) > 0) and (slen(trim(txtCFontName:GetText())) > 0))
				
			end
			
			tpage.txtCFontName.Event.OnEnter = function(tbox)
				tbox:GetParent().txtCFontFile:SetKeyFocus(true)
				_checkCFont(tbox)
			end
			
			tpage.txtCFontFile.Event.OnEnter = function(tbox)
				_checkCFont(tbox)
			end
			
			tpage.btnAddCFont.Event.LeftPress = function(btn)
				local p = btn:GetParent()
				local font = p.txtCFontName:GetText()
				
				-- check if this font already exists
				if (not myUI_Account.Global.Fonts[font]) then
					myUI_Account.Global.Fonts[font] = {
						src = "myUI", file = "fonts/" .. p.txtCFontFile:GetText()
					}
					
					p.lstFonts:AddItem(font)
					
					p.txtCFontFile:SetText("")
					p.txtCFontName:SetText("")
					btn:SetEnabled(false)
				
				end
			end
			
		end

		
		if (true) then -- icons tab page
			local tpage = myUI:CreateFrame("myUI_Tabpage", _configFrame)
			table.insert(tpages, tpage)
			
			tpage:SetPoint("TOPLEFT", tstrip, "TOPRIGHT")
			tpage:SetPoint("BOTTOMRIGHT", _configFrame, "BOTTOMRIGHT")
			
			tpage.tostring = function(tpage) return L["GameIcons"] end
			tpage:SetVisible(false)
			tstrip:AddTab(tpage)
			
			local msg = myUI:CreateFrame("Text", tpage)
			msg:SetWordwrap(true)
			msg:SetPoint("TOPLEFT", tpage, "TOPLEFT", margin, margin)
			msg:SetPoint("TOPRIGHT", tpage, "TOPRIGHT", -margin, margin)
			msg:SetText(L["GameIconsMsg"])
			
			local vp = myUI:CreateFrame("myUI_Viewport", tpage)
			local content = vp.Content
			
			vp:SetPoint("TOPLEFT", msg, "BOTTOMLEFT", 0, margin)
			vp:SetPoint("BOTTOMRIGHT", tpage, "BOTTOMRIGHT", -margin, -40)
			
			--content:SetBackgroundColor(1,0,0,0.2)
			--content:SetHeight(800)
			
			tpage.vp = vp
			tpage.icons = {} -- holds all the texture frames for our icons
			
			local function _iconClick(icon)
				local src, res = icon:GetTexture()
				pprint(src, ", ", res)
			end
			
			local function _loadIcons()
				local cxy = 52 -- size of our icon display + border
				local rows = 0
				
				local col = 0
				local row = 0
				local maxX = 600
				local i = 1
				for texture, src in myUI:KnownGameIcons() do
					if (i > #tpage.icons) then
						table.insert(tpage.icons, myUI:CreateFrame("Texture", content))
						tpage.icons[i]:SetSize(cxy-4)
						tpage.icons[i].Event.LeftDown = _iconClick
					end
					
					local icon = tpage.icons[i]
					
					if ((col * cxy) > maxX) then
						col = 0
						row = row + 1					
					end
					
					icon:SetTexture(src, texture)
					icon:SetPoint("TOPLEFT", content, "TOPLEFT", (col * cxy), (row * cxy))
					
					col = col + 1
					i = i + 1
				end
				
				content:SetHeight((row+1) * cxy)
				
			end
		
			local btn = myUI:CreateFrame("myUI_Button", tpage)
			btn:SetPoint("BOTTOMRIGHT", tpage, "BOTTOMRIGHT", -margin - 22, -margin)
			btn:SetText(L["GameIconsScan"])
			btn.Event.LeftPress = function(btn)
				myUI:ScanForGameIcons()
				
				_loadIcons()
			end
			
			_loadIcons()
		end
		
		
		if (true) then -- performance tab page
			local tpage = myUI:CreateFrame("myUI_Tabpage", _configFrame)
			table.insert(tpages, tpage)
			
			tpage:SetPoint("TOPLEFT", tstrip, "TOPRIGHT")
			tpage:SetPoint("BOTTOMRIGHT", _configFrame, "BOTTOMRIGHT")
		
			tpage.tostring = function(tpage) return L["Performance"] end
			tpage:SetVisible(false)
			tstrip:AddTab(tpage)
			
			local msg = myUI:CreateFrame("Text", tpage)
			msg:SetWordwrap(true)
			msg:SetPoint("TOPLEFT", tpage, "TOPLEFT", margin, margin)
			msg:SetPoint("TOPRIGHT", tpage, "TOPRIGHT", -margin, margin)
			msg:SetText(L["PerformanceMsg"])
			
			local label = myUI:CreateFrame("Text", tpage)
			label:SetPoint("TOPLEFT", msg, "BOTTOMLEFT", 64, 32)
			label:SetText(L["Animation Speed"] .. ":")
			
			local value = myUI:CreateFrame("Text", tpage)
			value:SetPoint("LEFTCENTER", label, "RIGHTCENTER", 16, 0)
			value:SetText(tostring(myUI:GetAnimationSpeed()) .. L["second"])
			
			local slider = myUI:CreateFrame("RiftSlider", tpage)
			slider:SetWidth(300)
			slider:SetRange(4,30)
			slider:SetPosition(myUI:GetAnimationSpeed())
			slider:SetPoint("TOPLEFT", label, "BOTTOMLEFT", 24, 18)
			slider.label = value
			slider.Event.SliderChange = function(slider)
				myUI:SetAnimationSpeed(slider:GetPosition())
				slider.label:SetText(tostring(myUI:GetAnimationSpeed()) .. L["second"])
			end
			
			label = myUI:CreateFrame("Text", tpage)
			label:SetPoint("TOPLEFT", slider, "BOTTOMLEFT", -24, 32)
			label:SetText(L["Unit Update Speed"] .. ":")
			
			value = myUI:CreateFrame("Text", tpage)
			value:SetPoint("LEFTCENTER", label, "RIGHTCENTER", 16, 0)
			value:SetText(tostring(myUI:GetUnitUpdateInterval()) .. L["second"])
			
			slider = myUI:CreateFrame("RiftSlider", tpage)
			slider:SetWidth(300)
			slider:SetRange(4,20)
			slider:SetPosition(myUI:GetUnitUpdateInterval())
			slider:SetPoint("TOPLEFT", label, "BOTTOMLEFT", 24, 18)
			slider.label = value
			slider.Event.SliderChange = function(slider)
				myUI:SetUnitUpdateInterval(slider:GetPosition())
				slider.label:SetText(tostring(myUI:GetUnitUpdateInterval()) .. L["second"])
			end
			
			label = myUI:CreateFrame("Text", tpage)
			label:SetPoint("TOPLEFT", slider, "BOTTOMLEFT", -24, 32)
			label:SetText(L["Slow Update Speed"] .. ":")
			
			value = myUI:CreateFrame("Text", tpage)
			value:SetPoint("LEFTCENTER", label, "RIGHTCENTER", 16, 0)
			value:SetText(tostring(myUI:GetSlowUpdateInterval()) .. L["second"])
			
			slider = myUI:CreateFrame("RiftSlider", tpage)
			slider:SetWidth(300)
			slider:SetRange(1,4)
			slider:SetPosition(myUI:GetSlowUpdateInterval())
			slider:SetPoint("TOPLEFT", label, "BOTTOMLEFT", 24, 18)
			slider.label = value
			slider.Event.SliderChange = function(slider)
				myUI:SetSlowUpdateInterval(slider:GetPosition())
				slider.label:SetText(tostring(myUI:GetSlowUpdateInterval()) .. L["second"])
			end
		end
		
		
		
		tstrip.Event.TabSelected = function(tstrip, tpage)
			for _, p in ipairs(_configFrame.TabPages) do
				p:SetVisible(false)
			end
			
			
			tpage:SetVisible(true)
		end
		
		_configFrame.TabPages = tpages
		_configFrame.TabStrip = tstrip
	
	end
	
	return _configFrame
end