local myUI = ...
local L = myUI.data.language
local trim = myUI.Utility.trim

--[[

Saved variables format:

myUI_Account = {
	Version = myUI.toc.Version,
	Global = {
		AnimationSpeed = 20,
		Fonts = {
			["Rift Default"] = {src="Rift", file = "#Flareserif_medium"},
			etc ...
		},
		Lists = {},	
	}
	Profiles = {
		["default"] = {
			GlobalFont = "Rift Default",
			GlobalFontSize = 14,
			Addons = {
				["addon1"] = {},
				["addon2"] = {},
				etc ...
			}
		},
		["profile2"] = {
			GlobalFont = "Rift Default",
			Addons = {
				["addon1"] = {},
				["addon2"] = {},
				etc ...
			}
		},
		etc ...
	},
	

}

myUI_Character = {
	Version = myUI.toc.Version,
	CurrentProfile = "default",
	Profiles = {
		["default"] = {
			Addons = {
				["addon1"] = {},
				["addon2"] = {},
				etc ...
			}
		},
		["profile2"] = {
			Addons = {
				["addon1"] = {},
				["addon2"] = {},
				etc ...
			}
		},
		etc ...
	}	
}



]]--



local function _getAccountDefaults()
	local defaults = {
		Version = myUI.toc.Version,
		ConfigX = 100,
		ConfigY = 100,
		Global = {
			AnimationSpeed = 20,
			UnitUpdateSpeed = 10,
			SlowUpdateSpeed = 2,
			Fonts = {
				["Rift Default"] = {src = "Rift", file = "$Flareserif_medium"},
				["Arial"] = {src = "myUI", file = "fonts/arial.ttf"},
				["Groovy Fast"] = {src = "myUI", file = "fonts/Groovy Fast.ttf"},
				["Highland Gothic"] = {src = "myUI", file = "fonts/HighlandGothicFLF.ttf"},
				["Liberation"] = {src = "myUI", file = "fonts/LiberationMono-Regular.ttf"},
				["Permanent Marker"] = {src = "myUI", file = "fonts/PermanentMarker.ttf"},
				["Sansumi"] = {src = "myUI", file = "fonts/Sansumi-Bold.ttf"},			
			},
			Lists = {},
			Icons = {
				-- index = resource filename, value = source (ie Rift, myUI, etc)
			
			},
		},		
		Profiles = {
			["default"] = {
				GlobalFont = "Arial",
				GlobalFontSize = 14,
				Addons = {
					[myUI.identifier] = {Version = myUI.toc.Version}
				}
			}	
		},
		
	}
	return defaults
end

local function _getCharacterDefaults()
	local defaults = {
		Version = myUI.toc.Version,
		CurrentProfile = "default",
		Profiles = {
			["default"] = {
				Addons = {
					[myUI.identifier] = {Version = myUI.toc.Version} -- add settings for the core by default
				}
			}		
		}
	
	}
	return defaults
end

local function _checkDefaults(settings, defaults)
	-- Version checking can be added here if needed
	if (settings == nil) then settings = {} end
	
	myUI.Utility.nilCopy(settings, defaults)
	
	return settings
end

local function _varsLoaded(addonID)
	if (addonID == myUI.identifier) then
		myUI_Account = _checkDefaults(myUI_Account, _getAccountDefaults())
		myUI_Character = _checkDefaults(myUI_Character, _getCharacterDefaults())	
	end	
end







-- public profile management API

function myUI:GetCurrentProfile()
	return myUI_Character.CurrentProfile
end
function myUI:SetCurrentProfile(profile, skipReload)
	--if (not Inspect.System.Secure()) then
		if (profile == nil) then profile = "default" end
		
		if (myUI_Character.Profiles[profile]) and (myUI_Account.Profiles[profile]) then
			myUI_Character.CurrentProfile = profile
			
			if (not skipReload) then
				-- ensure that each module has at least an empty table in this profile
				for _, module in myUI:Modules() do
					if (myUI_Character.Profiles[profile].Addons[module.identifier] == nil) then
						myUI_Character.Profiles[profile].Addons[module.identifier] = {}
					end
					if (myUI_Account.Profiles[profile].Addons[module.identifier] == nil) then
						myUI_Account.Profiles[profile].Addons[module.identifier] = {}
					end
					
					if (module.LoadProfile) then
						module:LoadProfile(
							myUI_Account.Profiles[profile].Addons[module.identifier],
							myUI_Character.Profiles[profile].Addons[module.identifier]
						)
					end
				end
				
				myUI:SetGlobalFont(myUI_Account.Profiles[profile].GlobalFont, myUI_Account.Profiles[profile].GlobalFontSize)
				
				for _, module in myUI:Modules() do
					if (module.PostLoadProfile) then
						module:PostLoadProfile()
					end
				end
			end 
			
			print(L["Profile Loaded"] .. " \"" .. profile .. "\"")
			return true
		else
			print(L["Bad Profile"] .. " \"" .. profile .. "\"")
		end
	--else
	--	print(L["Profile Combat"])
	--end
end

function myUI:CopyCurrentProfile(newProfile)
	if (newProfile == nil) then return end
	
	newProfile = trim(newProfile)
	if (string.len(newProfile) == 0) then return end
	
	local current = myUI:GetCurrentProfile()
	if (current ~= newProfile) then
		myUI_Account.Profiles[newProfile] = myUI.Utility.shallowCopy(myUI_Account.Profiles[current])
		myUI_Character.Profiles[newProfile] = myUI.Utility.shallowCopy(myUI_Character.Profiles[current])
		
		
		return true
	end
end

function myUI:DeleteProfile(profile)
	if (profile == nil) then return end
	
	profile = trim(profile)
	if (string.len(profile) == 0) then return end
	if (profile == "default") then return end -- can't delete the default profile
	
	local current = myUI:GetCurrentProfile()
	if (current == profile) then return end -- can't delete current profile
	
	myUI_Account.Profiles[profile] = nil
	myUI_Character.Profiles[profile] = nil
	
	return true 	
end





function myUI:GetProfiles()
	local t = {}

	for k, _ in pairs(myUI_Account.Profiles) do
		table.insert(t, k)
	end
	
	return t
end

table.insert(Event.Addon.SavedVariables.Load.End, {_varsLoaded, myUI.identifier, "_varsLoaded"})
