nkWardrobe = {
	addonName = "nkWardrobe",
	addonVersion = "3.0.0",
	configUI = nil,
	extUI = nil,
	confirmDialog = nil,
	errDialog = nil,
	macroButton = nil,
	secure = false,
	calling = nil
}

nkWardrobe.context = UI.CreateContext("nkWardrobe")

function nkWardrobe.main(addon)

	if (addon == nkWardrobe.addonName) then
	
		local items = {
		{ label = nkWardrobeTexts.buttonMenuConfig, func = function () if nkWardrobe.configUI == nil then nkWardrobe.configUI = nkWardrobe.Config:new() else nkWardrobe.configUI:Show(true) end end},
		
		{ seperator = true } }
		
		for k, v in pairs (nkWardrobeSettings.sets) do
			local macro = string.format ('role %d\nloadequip %d', v.role, v.equipment)
			local macro = string.format ('role %d', v.role)
			if nkRebuff ~= nil then macro = macro .. string.format('\nnkRebuff set=%d', v.nkrebuff) end
		
			table.insert ( items, { label = string.format (nkWardrobeTexts.loadSet, v.name), macro = macro, func = function() nkWardrobe.wearEquip(v.equipment) end })
		end
				
		nkManager.init('nkWardrobe', items, nil)
		
		table.insert(Event.System.Secure.Enter, {function() nkWardrobe.secure = true end, "nkWardrobe", "secureEnter"})
		table.insert(Event.System.Secure.Leave, {function() nkWardrobe.secure = false end, "nkWardrobe", "secureLeave"})
		
		nkWardrobe.UIExtension()
		
		print (string.format(nkWardrobeTexts.msgLoaded, nkWardrobe.addonVersion))		
		
	end
	
end

function nkWardrobe.startup()

	if nkWardrobe.calling == nil then
		local userDetails = Inspect.Unit.Detail ('player')
		nkWardrobe.calling = userDetails.calling
	end

	if nkWardrobeSettings.button == true and nkWardrobe.macroButton == nil then nkWardrobe.macroButton = nkWardrobe.Button:new() end
	
end

function nkWardrobe.commandHandler(commandline)

	if string.find(commandline, 'equip=') ~= nil then
		local pos = string.find(commandline, 'equip=')
		local setno = tonumber(string.sub(commandline, pos+6, string.len(commandline)))
		
		if setno < 1 or setno > #nkWardrobeSettings.equipSets then
			print (string.format(nkWardrobeTexts.commandSetRefuse, setno, #nkWardrobeSettings.equipSets ))
		else
			nkWardrobe.wearEquip (setno)
		end
	else
		if nkWardrobe.configUI == nil then nkWardrobe.configUI = nkWardrobe.Config:new() else nkWardrobe.configUI:Show(true) end
	end
	
end

function nkWardrobe.settingsHandler() 
	
	local settings = {
		version = nkWardrobe.addonVersion,
		open = 'bottom',
		border = true,
		scale = 80,
		activeSet = 1,
		button = true,
		buttonX = -190,
		buttonY = 260,
		equipSets = {},
		currentSet = 1,
	}
	
	if nkWardrobeSettings == nil then
		nkWardrobeSettings = settings
	else
		for k, v in pairs (settings) do if nkWardrobeSettings[k] == nil then nkWardrobeSettings[k] = v end end
	end
	
	if nkWardrobeSettings.sets == nil then
		local sets = {}
		for idx = 1, 6, 1 do
			table.insert (sets, {name = string.format(nkWardrobeTexts.set, idx), role = idx, equipment = idx, nkrebuff = idx, icon = 1, KAMain = -1, KASub = -1 } )
		end
		
		nkWardrobeSettings.sets = sets
	end
	
	-- Routinen zur Rckwrtskompatibilitt um Daten zu lschen / retten
	
	if nkWardrobeSettings.version == nil then
		nkWardrobeSettings.equipSets = nil
	else
		for idx = 1, #nkWardrobeSettings.equipSets, 1 do
			if nkWardrobeSettings.equipSets[idx].bank == nil then nkWardrobeSettings.equipSets[idx].bank = 0 end
			if nkWardrobeSettings.equipSets[idx].bag == nil then nkWardrobeSettings.equipSets[idx].bag = 1 end
			if nkWardrobeSettings.equipSets[idx].name == nil then nkWardrobeSettings.equipSets[idx].name = string.format(nkWardrobeTexts.txtEquipment, idx) end
		end
	end
			
end

table.insert(Event.Addon.Load.End, {nkWardrobe.main, "nkWardrobe", "addonLoaded"})
table.insert(Event.Unit.Available, {nkWardrobe.startup, "nkWardrobe", "systemLoaded"})
table.insert(Command.Slash.Register("nkWardrobe"), {nkWardrobe.commandHandler, "nkWardrobe", "commandHandler"})
table.insert(Command.Slash.Register("nkw"), {nkWardrobe.commandHandler, "nkWardrobe", "commandHandler"})
table.insert(Event.Addon.SavedVariables.Load.End, {nkWardrobe.settingsHandler, "nkWardrobe", "variablesLoaded"})

function nkWardrobe.wearEquip (setNo)

	if nkWardrobeSettings.equipSets[setNo].name ~= nil then
		print(string.format(nkWardrobeTexts.loadSet, nkWardrobeSettings.equipSets[setNo].name))
	else
		print(string.format(nkWardrobeTexts.loadSet, tostring(setNo)))
	end
	
	local language = Inspect.System.Language()
	
	nkWardrobe.initMove = math.floor(Inspect.Time.Frame())
		
	for k, v in pairs (nkWardrobeSettings.equipSets[setNo]) do
		if k ~= 'name' and k ~= 'bag' and k ~= 'bank' then
			local err, sourceSlot = pcall(Inspect.Item.Find, v)
			
			if err then		
				local slot = Utility.Item.Slot.Equipment(k)
				
				if sourceSlot ~= slot then				
					pcall (Command.Item.Move, sourceSlot, slot)
				end
			else
				local itemDetails = Inspect.Item.Detail(v)
				print (string.format(nkWardrobeTexts.errGettingItem, itemDetails.name))
			end
		end
	end

end

function nkWardrobe.UIExtension ()

	nkWardrobe.extUI = UI.CreateFrame ('nkExtRiftWindow', 'extUIWindow', nkWardrobe.context, { width = 300, height = 430, title = nkWardrobe.addonName, anchors = {{from = "TOPLEFT", object = UI.Native.Character, to = "TOPRIGHT"}}, visible = false})
	
	local selection = {}
	
	if nkWardrobeSettings.equipSets[1] == nil then nkWardrobeSettings.equipSets[1] = { name = string.format(nkWardrobeTexts.txtEquipment, 1) } end
	nkWardrobeSettings.currentSet = 1
	
	for idx = 1, #nkWardrobeSettings.equipSets, 1 do
		table.insert (selection, { label = nkWardrobeSettings.equipSets[idx].name, value = idx })		
	end
	
	nkWardrobe.extUI.fldSelEquipmentSet = UI.CreateFrame('nkExtCombo', 'fldSelEquipmentSet', nkWardrobe.extUI:getElement(), {	layer = 3, value = 1, width = 200, fontsize = 13, color = {border = '666666', body = '000000', label = 'FFFFFF'}, selection = selection, 
																																anchors = {{ from = "TOPLEFT", object = nkWardrobe.extUI:getBody(), to = "TOPLEFT", x = 10, y = 10 }}, 
																																func = function (value) nkWardrobe.UIExtensionSelectSet(value) end })
	
	nkWardrobe.extUI.editIcon = UI.CreateFrame('nkExtTexture', 'editIcon', nkWardrobe.extUI:getElement(), { type = "nkWardrobe", path = "gfx/editPen.png", width = 15, height = 15, anchors = {{ from = "CENTERLEFT", object = nkWardrobe.extUI.fldSelEquipmentSet:getElement(), to = "CENTERRIGHT", x = 5, y = 0 }} })
	
	nkWardrobe.extUI.editIcon:SetEvent('LeftClick', function ()
		if nkWardrobe.extUI.fldEquipmentSetName:GetVisible() == true then
			nkWardrobe.extUI.fldEquipmentSetName:leave()
			nkWardrobe.extUI.fldSelEquipmentSet:SetVisible(true)
			nkWardrobe.extUI.fldEquipmentSetName:SetVisible(false)
		else
			nkWardrobe.extUI.fldEquipmentSetName:setValue(nkWardrobeSettings.equipSets[nkWardrobe.extUI.fldSelEquipmentSet:getValue()].name)
			nkWardrobe.extUI.fldSelEquipmentSet:SetVisible(false)
			nkWardrobe.extUI.fldEquipmentSetName:SetVisible(true)
		end
	end)
	
	local fldTemplate = { width = 40, height = 20, fontsize = 13, labelwidth = 100, color = { label = 'FFFFFF', border = '666666', body = '000000', focus = '999999' } }
	
	nkWardrobe.extUI.fldEquipmentSetName = UI.CreateFrame ('nkExtTextField', 'fldEquipmentSetName', nkWardrobe.extUI:getElement(true), {	layer = 2, visible = false, width = 200, value = nkWardrobeSettings.equipSets[1].name, anchors = {{ from = "TOPLEFT", object = nkWardrobe.extUI.fldSelEquipmentSet:getElement(), to = "TOPLEFT" }}, 
																																			func = function (value) nkWardrobeSettings.equipSets[nkWardrobe.extUI.fldSelEquipmentSet:getValue()].name = value; nkWardrobe.UIExtensionUpdSelection() end, template = fldTemplate})	

	nkWardrobe.extUI.btnWear = UI.CreateFrame ('nkExtRiftButton', 'btnWearEquip', nkWardrobe.extUI:getElement(), { layer = 1, label = nkWardrobeTexts.btWearEquip, anchors = {{from = "TOPLEFT", object = nkWardrobe.extUI:getBody(), to = "TOPLEFT", x = 5, y = 50}}, func = function () nkWardrobe.wearEquip(nkWardrobe.extUI.fldSelEquipmentSet:getValue()) end})
	
	nkWardrobe.extUI.btLoadFromEquip  = UI.CreateFrame ('nkExtRiftButton', 'btLoadFromEquip', nkWardrobe.extUI:getElement(), { layer = 1, label = nkWardrobeTexts.btLoadFromEquip, anchors = {{from = "TOPRIGHT", object = nkWardrobe.extUI:getBody(), to = "TOPRIGHT", x = -5, y = 50}}, func = function () nkWardrobe.UIExtensionLoadCurrentEquip(nkWardrobe.extUI.fldSelEquipmentSet:getValue()) end})
	
	selection = {}
	for idx = 1, 5, 1 do table.insert(selection, { label = string.format ( nkWardrobeTexts.txtBag, idx ), value = idx } ) end
	
	nkWardrobe.extUI.fldValueStoreInBag = UI.CreateFrame('nkExtCombo', 'fldStoreInBag', nkWardrobe.extUI:getElement(), { layer = 2, value = nkWardrobeSettings.equipSets[1].bag, width = 110, template = fldTemplate,
																													selection = selection, anchors = {{ from = "TOPLEFT", object = nkWardrobe.extUI.btnWear:getElement(), to = "BOTTOMLEFT", x = 0, y = 30}}, 
																													func = function (value) nkWardrobeSettings.equipSets[nkWardrobe.extUI.fldSelEquipmentSet:getValue()].bag = value end})
	
	selection = { { label = nkWardrobeTexts.mainBankSlot, value = 0 }}
	for idx = 1, 8, 1 do table.insert(selection, { label = string.format ( nkWardrobeTexts.txtBank, idx ), value = idx } ) end
																													
	nkWardrobe.extUI.fldValueStoreInBank = UI.CreateFrame('nkExtCombo', 'fldStoreInBank', nkWardrobe.extUI:getElement(), { layer = 2, value = nkWardrobeSettings.equipSets[1].bank, width = 110, template = fldTemplate,
																													selection = selection, anchors = {{ from = "TOPRIGHT", object = nkWardrobe.extUI.btLoadFromEquip:getElement(), to = "BOTTOMRIGHT", x = 0, y = 30}}, 
																													func = function (value) nkWardrobeSettings.equipSets[nkWardrobe.extUI.fldSelEquipmentSet:getValue()].bank = value end})
	
	nkWardrobe.extUI.btGetFromBank = UI.CreateFrame ('nkExtRiftButton', 'btGetFromBank', nkWardrobe.extUI:getElement(), { layer = 1, label = nkWardrobeTexts.btGetFromBank, anchors = {{from = "TOPLEFT", object = nkWardrobe.extUI.fldValueStoreInBag:getElement(), to = "BOTTOMLEFT", x = 0, y = 10}}, func = function() nkWardrobe.UIExtensionGetFromBank() end, template = btTemplate})	
	nkWardrobe.extUI.btPutToBank = UI.CreateFrame ('nkExtRiftButton', 'btPutToBank', nkWardrobe.extUI:getElement(), { layer = 1, label = nkWardrobeTexts.btPutToBank, anchors = {{from = "TOPRIGHT", object = nkWardrobe.extUI.fldValueStoreInBank:getElement(), to = "BOTTOMRIGHT", x = 0, y = 10}}, func = function () nkWardrobe.UIExtensionPutToBank() end, template = btTemplate})		
	
	nkWardrobe.extUI.btAddSet = UI.CreateFrame ('nkExtRiftButton', 'btAddSet', nkWardrobe.extUI:getElement(), { layer = 1, label = nkWardrobeTexts.btAddSet, anchors = {{from = "TOPLEFT", object = nkWardrobe.extUI.btGetFromBank:getElement(), to = "BOTTOMLEFT", x = 0, y = 30}}, func = function () nkWardrobe.UIExtensionAddSet() end})	
	nkWardrobe.extUI.btRemoveSet = UI.CreateFrame ('nkExtRiftButton', 'btRemoveSet', nkWardrobe.extUI:getElement(), { layer = 1, label = nkWardrobeTexts.btRemoveSet, anchors = {{from = "TOPRIGHT", object = nkWardrobe.extUI.btPutToBank:getElement(), to = "BOTTOMRIGHT", x = 0, y = 30}}, func = function () nkWardrobe.UIExtensionRemoveSet() end})	
	nkWardrobe.extUI.btCopySet = UI.CreateFrame ('nkExtRiftButton', 'btCopySet', nkWardrobe.extUI:getElement(), { layer = 1, label = nkWardrobeTexts.btCopySet, anchors = {{from = "TOPCENTER", object = nkWardrobe.extUI:getBody(), to = "TOPCENTER", x = 0, y = 240}}, func = function () nkWardrobe.UIExtensionCopySet() end})
					
	nkWardrobe.extUI.btConfig = UI.CreateFrame ('nkExtRiftButton', 'btConfig', nkWardrobe.extUI:getElement(), { layer = 1, label = nkWardrobeTexts.btConfig, anchors = {{from = "BOTTOMCENTER", object = nkWardrobe.extUI:getBody(), to = "BOTTOMCENTER", x = 0, y = -30}}, func = function () if nkWardrobe.configUI == nil then nkWardrobe.configUI = nkWardrobe.Config:new() else nkWardrobe.configUI:Show(true) end end})

	nkWardrobe.extUI.txtVersion = UI.CreateFrame ('nkExtText', 'txtVersion', nkWardrobe.extUI:getElement(), { layer = 1, fonsize = 13, color = 'ECE4BD', text = string.format(nkWardrobeTexts.txtHeader, nkWardrobe.addonName, nkWardrobe.addonVersion), anchors = {{from = "BOTTOMCENTER", object = nkWardrobe.extUI:getBody(), to = "BOTTOMCENTER", x = 0, y = -0}} })
	
	function UI.Native.Character.Event:Loaded() 
		nkWardrobe.extUI:SetVisible(UI.Native.Character:GetLoaded()) 
	end

																								
end

function nkWardrobe.UIExtensionSelectSet(selSet)

	nkWardrobe.extUI.fldSelEquipmentSet:setByValue(selSet)
	nkWardrobe.extUI.fldValueStoreInBag:setByValue(nkWardrobeSettings.equipSets[selSet].bag)
	nkWardrobe.extUI.fldValueStoreInBank:setByValue(nkWardrobeSettings.equipSets[selSet].bank)

end

function nkWardrobe.UIExtensionUpdSelection()

	local selection = {}
	
	for idx = 1, #nkWardrobeSettings.equipSets, 1 do
		table.insert (selection, { label = nkWardrobeSettings.equipSets[idx].name, value = idx })
	end

	nkWardrobe.extUI.fldSelEquipmentSet:updateSelection(selection)
	
	nkWardrobe.extUI.fldEquipmentSetName:SetVisible(false)
	nkWardrobe.extUI.fldSelEquipmentSet:SetVisible(true)
	
end

function nkWardrobe.UIExtensionLoadCurrentEquip(loadToSet)

	local slotTable = {'helmet', 'shoulders', 'chest', 'gloves', 'belt', 'legs', 'feet', 'focus', 'handmain', 'handoff', 'ranged', 'neck', 'trinket', 'ring1', 'ring2', 'synergy'}

	local funcYes = function ()
		local bank = nkWardrobeSettings.equipSets[loadToSet].bank
		local bag = nkWardrobeSettings.equipSets[loadToSet].bag
		local name = nkWardrobeSettings.equipSets[loadToSet].name
		
		nkWardrobeSettings.equipSets[loadToSet] = {}
		
		for k, v in pairs (slotTable) do
			local equipSlot = Utility.Item.Slot.Equipment(v)
			local itemID = Inspect.Item.List (equipSlot)
			if itemID ~= false then 
				nkWardrobeSettings.equipSets[loadToSet][v] = itemID 
			else
				nkWardrobeSettings.equipSets[loadToSet][v] = nil
			end
		end
		
		nkWardrobeSettings.equipSets[loadToSet].bank = bank
		nkWardrobeSettings.equipSets[loadToSet].bag = bag
		nkWardrobeSettings.equipSets[loadToSet].name = name
	end

	if nkWardrobe.confirmDialog == nil then
		nkWardrobe.confirmDialog = UI.CreateFrame('nkExtDialog', 'nkWardrobeConfirmDialog', nkWardrobe.context, { layer = 2, width = 300, height = 125, confirmButton = true, color = {border = "000000", body = "333333", header = "3B5998", title = "FFFFFF", message = 'FFFFFF'}, 
																										title = string.format(nkWardrobeTexts.txtHeader, nkWardrobe.addonName, nkWardrobe.addonVersion), message = nkWardrobeTexts.msgLoadCurrentEquip, 
																										anchors = {{ from = "CENTER", object = UIParent, to = "CENTER"}}, funcYes = funcYes })
	else
		nkWardrobe.confirmDialog:update ({ message = nkWardrobeTexts.msgLoadCurrentEquip, anchors = {{ from = "CENTER", object = UIParent, to = "CENTER"}}, funcYes = funcYes })
		nkWardrobe.confirmDialog:SetVisible(true)
	end
	
end

function nkWardrobe.UIExtensionGetFromBank()

	local setNo = nkWardrobe.extUI.fldSelEquipmentSet:getValue()

	local availSlots = {}
	local allSlots = nil
	
	if nkWardrobeSettings.equipSets[setNo].bag ~= nil then
		local bagSlot = Utility.Item.Slot.Inventory(nkWardrobeSettings.equipSets[setNo].bag)
		local bagInfo = Inspect.Item.List(bagSlot)
		
		for slot, details in pairs (bagInfo) do
			if details == false then table.insert (availSlots, slot) end		
		end
	end
	
	if #availSlots < 16 then	
		allSlots = Inspect.Item.List('si') 
		for slot, details in pairs (allSlots) do
			if nkTools.table.isMember (availSlots, slot) == false and details == false then table.insert (availSlots, slot) end		
		end
	end
	
	local bankSlots = Inspect.Item.List('sb')
	
	if #availSlots == 0 then return end
	
	for k, v in pairs (nkWardrobeSettings.equipSets[setNo]) do
		if k ~= 'name' and k ~= 'bank' and k ~= 'bag' then
			local err, retVal = pcall (Inspect.Item.Find, v)
			
			if err then
				if bankSlots[retVal] ~= nil then
					if #availSlots > 0 then 
						pcall (Command.Item.Move, retVal, availSlots[1])
						table.remove (availSlots, 1)
					end
				else
					local itemDetails = Inspect.Item.Detail(v)
					print (string.format(nkWardrobeTexts.errGettingItem, itemDetails.name))
				end
			end
		end
	end

end

function nkWardrobe.UIExtensionPutToBank()

	local setNo = nkWardrobe.extUI.fldSelEquipmentSet:getValue()

	local availSlots = {}
	local allSlots = nil
	
	if nkWardrobeSettings.equipSets[setNo].bank ~= nil then
	
		local bankSlot = nil
		if nkWardrobeSettings.equipSets[setNo].bank == 0 then 
			bankSlot = Utility.Item.Slot.Bank('main')
		else
			bankSlot = Utility.Item.Slot.Bank(nkWardrobeSettings.equipSets[setNo].bank)
		end

		local bankInfo = Inspect.Item.List(bankSlot)
		
		for slot, details in pairs (bankInfo) do
			if details == false then table.insert (availSlots, slot) end
		end
		
	end

	if #availSlots < 16 then	
		allSlots = Inspect.Item.List('sb') 
		for slot, details in pairs (allSlots) do
			if nkTools.table.isMember (availSlots, slot) == false and details == false then table.insert (availSlots, slot) end		
		end
	end

	if #availSlots == 0 then return end
	
	local inventorySlots = Inspect.Item.List('si')
	local equipmentSlots = Inspect.Item.List('seqp')
		
	for k, v in pairs (nkWardrobeSettings.equipSets[setNo]) do
		if k ~= 'name' then
			local err, retVal = pcall (Inspect.Item.Find, v)
			
			if err then
				if (inventorySlots[retVal] ~= nil or equipmentSlots[retVal] ~= nil) and #availSlots > 0 then 
					pcall (Command.Item.Move, retVal, availSlots[1])
					table.remove (availSlots, 1)
				end
			end
		end
	end

end

function nkWardrobe.UIExtensionAddSet()

	local noOfSets = #nkWardrobeSettings.equipSets
	nkWardrobeSettings.equipSets[noOfSets+1] = { name = string.format(nkWardrobeTexts.txtEquipment, noOfSets+1), bag = 1, bank = 0 }
	
	nkWardrobeSettings.currentSet = noOfSets+1		
	nkWardrobe.UIExtensionUpdSelection()
	nkWardrobe.UIExtensionSelectSet(noOfSets+1)

end

function nkWardrobe.UIExtensionRemoveSet()

	if #nkWardrobeSettings.equipSets == 1 then 
		nkWardrobe.UIExtensionError(nkWardrobeTexts.errLastSet)
	else			
		local funcYes = function ()
			local currentSet = nkWardrobe.extUI.fldSelEquipmentSet:getValue()
			
			for idx = 1, #nkWardrobeSettings.sets, 1 do
				if nkWardrobeSettings.sets[idx].equipment > currentSet then
					nkWardrobeSettings.sets[idx].equipment = nkWardrobeSettings.sets[idx].equipment - 1
				elseif nkWardrobeSettings.sets[idx].equipment == currentSet then
					nkWardrobeSettings.sets[idx].equipment = 1
				end
			end
		
			table.remove (nkWardrobeSettings.equipSets,currentSet)
			nkWardrobe.UIExtensionUpdSelection()
			nkWardrobe.UIExtensionSelectSet(1)
		end
	
		if nkWardrobe.confirmDialog == nil then
			nkWardrobe.confirmDialog = UI.CreateFrame('nkExtDialog', 'nkWardrobeConfirmDialog', nkWardrobe.context, { layer = 2, width = 300, height = 125, confirmButton = true, color = {border = "000000", body = "333333", header = "3B5998", title = "FFFFFF", message = 'FFFFFF'}, 
																											title = string.format(nkWardrobeTexts.txtHeader, nkWardrobe.addonName, nkWardrobe.addonVersion), message = nkWardrobeTexts.msgRemoveSetConfirm, 
																											anchors = {{ from = "CENTER", object = UIParent, to = "CENTER"}}, funcYes = funcYes })
		else
			nkWardrobe.confirmDialog:update ({ message = nkWardrobeTexts.msgRemoveSetConfirm, anchors = {{ from = "CENTER", object = UIParent, to = "CENTER"}}, funcYes = funcYes })
			nkWardrobe.confirmDialog:SetVisible(true)
		end
	end
	
end

function nkWardrobe.UIExtensionCopySet()

	local currentSet = nkWardrobe.extUI.fldSelEquipmentSet:getValue()
	local noOfSets = #nkWardrobeSettings.equipSets
	local newSet = nkTools.table.copy (nkWardrobeSettings.equipSets[currentSet])
	newSet.name = string.format(nkWardrobeTexts.txtCopiedSet, nkWardrobeSettings.equipSets[currentSet].name)
	
	table.insert ( nkWardrobeSettings.equipSets, newSet)
	
	nkWardrobe.UIExtensionUpdSelection()
	nkWardrobe.UIExtensionSelectSet(#nkWardrobeSettings.equipSets)

end

function nkWardrobe.UIExtensionError (message)

	if nkWardrobe.errDialog == nil then
		nkWardrobe.errDialog = UI.CreateFrame('nkExtDialog', 'nkWardrobeConfirmDialog', nkWardrobe.context, { layer = 2, width = 300, height = 150, okButton = true, color = {border = "000000", body = "333333", header = "3B5998", title = "FFFFFF", message = 'FF0000'}, 
																										title = nkWardrobeTexts.errDialogTitle, message = message, anchors = {{ from = "CENTER", object = UIParent, to = "CENTER"}} })
	else
		nkWardrobe.errDialog:update ({ message = nkWardrobeTexts.errLastSet, anchors = {{ from = "CENTER", object = UIParent, to = "CENTER"}}, okButton = nil })
		nkWardrobe.errDialog:SetVisible(true)
	end

end